
#include "gome_L2.h"


char GOME_L2_MOL_NAME [GOME_L2_MAX][6] = {
    "1O3", "2O3", "2NO2" };
int GOME_L2_MOL_WIN [GOME_L2_MAX] = { 1,2,2 };


void UTC_s_getbin (FILE* unit, UTC_s *var)
{
    uint_getbin (unit, &var->Days);
    uint_getbin (unit, &var->Msecs);
}



void MPH_GOME_getbin (FILE* unit, MPH_GOME *var)
{
    char_array_getbin (unit, var->pir, 38);
    var->pir[38] = '\0';
    ushort_getbin (unit, &var->n_sph);
    int_getbin (unit, &var->length_sph);
    ushort_getbin (unit, &var->n_doas);
    int_getbin (unit, &var->length_doas);
}
    


void SPH_GOME_getbin (FILE* unit, SPH_GOME *var)
{
    int n;
    char_array_getbin (unit, var->pir, 38);
    var->pir[38] = '\0';
    
    char_array_getbin (unit, var->version_DOAS, 5);
    char_array_getbin (unit, var->version_keyparam, 5);
    char_array_getbin (unit, var->version_format, 5);    
    var->version_DOAS[5] = '\0';
    var->version_keyparam[5] = '\0';
    var->version_format[5] = '\0';
    
    ushort_getbin (unit, &var->Nwin);
    if (var->Nwin > GOME_NWIN_MAX)
    {
       fprintf (stderr,"Fatal error in SPH_GOME_getbin (...).\n"
		"GOME_NWIN_MAX to small in scia_defs.h, at least Nwin=%d\n"
	        "needed.\n", var->Nwin );
       exit (255);
    }

    for (n=0; n<var->Nwin; n++)
	float_array_getbin (unit, var->windowpair[n], 2);    
    ushort_getbin (unit, &var->Nmol);
    if (var->Nmol > GOME_NMOL_MAX)
    {
       fprintf (stderr,"Fatal error in SPH_GOME_getbin (...).\n"
		"GOME_NMOL_MAX to small in scia_defs.h, at least Nmol=%d\n"
	        "needed.\n", var->Nmol );
       exit (255);
    }
    for (n=0; n<var->Nmol; n++)
    {
	char_array_getbin (unit, var->molpair[n], 6);
	var->molpair[n][6] = '\0';
    }
    float_getbin (unit, &var->atmos_height);
}



MJD UTC_s_2_MJD (UTC_s *utc_s)
{
    MJD mjd;
    /*
    static const double jday_01011950 = 2433282.5;
    static const double jday_01012000 = 2451544.5;
    */
    static const int jday_diff = 18262;
    mjd.days = utc_s->Days - jday_diff;
    mjd.secnd = utc_s->Msecs / 1000;
    mjd.musec = (utc_s->Msecs % 1000) * 1000;
    return mjd;
}



void Geo_L2gome_getbin (FILE* unit, GeoN_L2_user *var)
{
    UTC_s time;
    int_getbin (unit, &var->gp);
    int_getbin (unit, &var->subset_counter);
    UTC_s_getbin (unit, &time);
    var->dsr_time = UTC_s_2_MJD (&time);
    float_array_getbin (unit,var->sol_zen_angle_sat , 3);
    float_array_getbin (unit,var->los_zen_angle_sat , 3);
    float_array_getbin (unit,var->rel_azi_angle_sat , 3);
    float_array_getbin (unit,var->sol_zen_angle_toa , 3);
    float_array_getbin (unit,var->los_zen_angle_toa , 3);
    float_array_getbin (unit,var->rel_azi_angle_toa , 3);
    float_getbin (unit, &var->sat_geod_ht);
    float_getbin (unit, &var->earth_rad);
    Coordf_array_getbin (unit,var->cor_coor_nad, 4);
    Coordf_getbin (unit,&var->cen_coor_nad);
}





void Gome_IRR_getbin (FILE* unit, Gome_IRR *var, int Nmol, int Nwin)
{
    float_array_getbin (unit,var->VCD_dat , Nmol);
    float_array_getbin (unit,var->VCD_err, Nmol);
    ushort_getbin (unit, &var->VCD_ind);
    float_array_getbin (unit,var->SlantCol, Nmol);
    float_array_getbin (unit,var->SlantCol_err, Nmol);
    float_array_getbin (unit,var->windowinf, 4*Nwin);
    ushort_getbin (unit, &var->DOAS_ind);
    float_array_getbin (unit,var->AMF_ground, Nmol);
    float_array_getbin (unit,var->AMF_top, Nmol);
    float_array_getbin (unit,var->AMF_winground, Nwin);
    float_array_getbin (unit,var->AMF_wintop, Nwin);
    float_array_getbin (unit,var->AMF_winmeasured, Nwin);
    ushort_getbin (unit, &var->AMF_ind);
    float_array_getbin (unit,var->cloud, 5);
    float_getbin (unit, &var->CCA_cf);
    uchar_array_getbin (unit, var->CCA_subpixel, 16);
    ushort_getbin (unit, &var->cloud_ind);
    float_array_getbin (unit,var->stat_contrast, 6);
    float_array_getbin (unit,var->stat_pixcolour, 16);
    float_getbin (unit, &var->stat_gradient);
    ushort_getbin (unit, &var->stat_ind);
}

void Gome_IRR_4_getbin (FILE* unit, Gome_IRR_4 *var, int Nmol, int Nwin)
{
    float_array_getbin (unit,var->VCD_dat , Nmol);
    float_array_getbin (unit,var->VCD_err, Nmol);
    ushort_getbin (unit, &var->VCD_ind);
    float_array_getbin (unit,var->SlantCol, Nmol);
    float_array_getbin (unit,var->SlantCol_err, Nmol);
    float_array_getbin (unit,var->windowinf, 4*Nwin);
    float_getbin (unit, &var->ozone_temp);
    float_getbin (unit, &var->ozone_ring);    
    ushort_getbin (unit, &var->DOAS_ind);
    float_array_getbin (unit,var->AMF_ground, Nmol);
    float_array_getbin (unit,var->AMF_ground_err, Nmol);
    float_array_getbin (unit,var->AMF_top, Nmol);
    float_array_getbin (unit,var->AMF_top_err, Nmol);
    ushort_getbin (unit, &var->AMF_ind);
    float_getbin (unit, &var->gvc);    
    float_array_getbin (unit,var->cloud, 2);
    float_array_getbin (unit,var->cldtop_height, 2);
    float_array_getbin (unit,var->cldtop_p, 2);
    float_array_getbin (unit,var->cldtop_alb, 2);
    float_array_getbin (unit,var->surface, 3);    
    uchar_array_getbin (unit, var->spare, 12*Nwin - 8*Nmol + 80);
}


void Gome_Doas_getbin (FILE* unit, Gome_Doas *var, int Nmol, int Nwin)
{
    /*  read content in common geo_nadir structure */
    Geo_L2gome_getbin (unit, &var->geo);
				
    float_getbin (unit, &var->ozon);
    float_getbin (unit, &var->ozon_err);
    Gome_IRR_getbin (unit, &var->gome_irr, Nmol, Nwin);
}    

void Gome_Doas_4_getbin (FILE* unit, Gome_Doas_4 *var, int Nmol, int Nwin)
{
    /*  read content in common geo_nadir structure */
    Geo_L2gome_getbin (unit, &var->geo);
				
    float_getbin (unit, &var->ozon);
    float_getbin (unit, &var->ozon_err);
    Gome_IRR_4_getbin (unit, &var->gome_irr, Nmol, Nwin);
}    


		       

