#include<stdlib.h>
#include<stdio.h>
#include <string.h>

#include "bin_read.h"
#include "scia_common.h"
#include "mipas_L2NRT.h"




/* Array with DSD Names */

char DS_NAME_MIP_NL__2P [26][29] = {
    "SUMMARY QUALITY ADS",
    "SCAN GEOLOCATION ADS",
    "DATASET STRUCTURE ADS",
    "SCAN INFORMATION MDS",
    "PT RETRIEVAL MDS",
    "H2O RETRIEVAL MDS",
    "N2O RETRIEVAL MDS",
    "HNO3 RETRIEVAL MDS",
    "CH4 RETRIEVAL MDS",
    "O3 RETRIEVAL MDS",
    "NO2 RETRIEVAL MDS",
    "CONTINUUM AND OFFSET MDS",
    "PCD INFORMATION ADS",
    "MICROWINDOW OCCUPATION ADS",
    "RESIDUAL SPECTRA ADS",
    "PROCESSING PARAMETERS ADS",
    "REFERENCE TO USED LEVEL_1B",
    "REFERENCE TO USED MIP_PS2_AX",
    "REFERENCE TO USED MIP_MW2_AX",
    "REFERENCE TO USED MIP_SP2_AX",
    "REFERENCE TO USED MIP_IG2_AX",
    "REFERENCE TO USED MIP_FM2_AX",
    "REFERENCE TO USED MIP_OM2_AX",
    "REFERENCE TO USED MIP_CS2_AX",
    "REFERENCE TO USED MIP_PI2_AX",
    "ECMWF_FILE" };



SCIA_err Read_SPH_MIP_NL__2P (FILE* unit, SPH_MIP_NL__2P *sph)

{
    int n, err;
    char *tmp = (char*) sph;
    /* Zeros in the complete structure, so all strings have trailing zero */
    for (n=0; n<sizeof(SPH_MIP_NL__2P); n++)
	tmp[n] = 0;
    
    err = fscanf(unit,
		 "SPH_DESCRIPTOR=\"%28c\"\n"
		 "STRIPLINE_CONTINUITY_INDICATOR=%4c\n"
		 "SLICE_POSITION=%4c\n"
		 "NUM_SLICES=%4c\n"
		 "START_TIME=\"%27c\"\n"
		 "STOP_TIME=\"%27c\"\n"
		 "FIRST_TANGENT_LAT=%11c<10-6degN>\n"
		 "FIRST_TANGENT_LONG=%11c<10-6degE>\n"
		 "LAST_TANGENT_LAT=%11c<10-6degN>\n"
		 "LAST_TANGENT_LONG=%11c<10-6degE>\n"
		 "NUM_SCANS=%6c\n"
		 "NUM_LOS_GEOMS=%6c\n"
		 "NUM_SCANS_PER_DS=%6c\n"
		 "NUM_SCANS_PROC=%6c\n"
		 "NUM_SP_NOT_PROC=%6c\n"
		 "NUM_SPECTRA=%6c\n"
		 "NUM_SPECTR_PROC=%6c\n"
		 "NUM_GAIN_CAL=%6c\n"
		 "TOT_GRANULES=%6c\n"
		 "MAX_PATH_DIFF=%15c<cm>\n"
		 "ORDER_OF_SPECIES=\"%30c\"\n"
		 "NUM_SWEEPS_PER_SCAN=%6c\n"
		 ,
		 sph->sph_descriptor,
		 sph->stripline_continuity_indicator,
		 sph->slice_position,
		 sph->num_slices,
		 sph->start_time,
		 sph->stop_time,
		 sph->first_tangent_lat,
		 sph->first_tangent_long,
		 sph->last_tangent_lat,
		 sph->last_tangent_long,
		 sph->num_scans,
		 sph->num_los_geoms,
		 sph->num_scans_per_ds,
		 sph->num_scans_proc,
		 sph->num_sp_not_proc,
		 sph->num_spectra,
		 sph->num_spectr_proc,
		 sph->num_gain_cal,
		 sph->tot_granules,
		 sph->max_path_diff,
		 sph->order_of_species,
		 sph->num_sweeps_per_scan);

    if ( strncmp ("MIP_NL__2P", sph->sph_descriptor,10) != 0
	 && strncmp ("MIP_NLE_2P", sph->sph_descriptor,10) != 0 )
	return SPH_ERROR;	
    if (err != 22)
	return SPH_ERROR;
    return OK;
}




void ds_pointer_getbin (FILE* unit, ds_pointer *var)
{
    int_getbin (unit, &var->dsr_offset);
    uint_getbin (unit, &var->dsr_length);
}


void Dataset_Structure_Ads_getbin (FILE* unit, Dataset_Structure_Ads *var)
{
   MJD_getbin (unit, &var->dsr_time);
   char_getbin (unit, &var->attach_flag);
   ushort_getbin (unit, &var->num_sweeps);
   ushort_getbin (unit, &var->num_press_temp_pts);
   ushort_array_getbin (unit, var->num_vmr_pts, 6);
   ushort_array_getbin (unit, var->flags_p_t_error_flag, 6);
   ushort_getbin (unit, &var->num_con_params_p_t);
   ushort_array_getbin (unit, var->num_con_params_vmr, 6);
   ushort_getbin (unit, &var->num_instr_offset_p_t);
   ushort_array_getbin (unit, var->num_instr_offset_vmr, 6);
   ushort_getbin (unit, &var->max_num_micro_p_t);
   ushort_array_getbin (unit, var->max_num_micro_vmr, 6);
   ushort_getbin (unit, &var->tot_num_p_t_micro_all_alt);
   ushort_array_getbin (unit, var->tot_num_vmr_micro_all_alt, 6);
   ushort_getbin (unit, &var->tot_num_spect_grid_p_t);
   ushort_array_getbin (unit, var->tot_num_spect_vmr, 6);
   ushort_getbin (unit, &var->num_grid_con_p_t);
   ushort_array_getbin (unit, var->num_grid_con_vmr, 6);
   short_getbin (unit, &var->num_evo_steps_p_t);
   short_array_getbin (unit, var->num_evo_steps_vmr, 6);
   ushort_getbin (unit, &var->num_pcd_info);
   ds_pointer_getbin (unit, &var->mds_scan_info_pointer);
   ds_pointer_getbin (unit, &var->mds_p_t_pointer);
   ds_pointer_getbin (unit, &var->mds_vmr1_pointer);
   ds_pointer_getbin (unit, &var->mds_vmr2_pointer);
   ds_pointer_getbin (unit, &var->mds_vmr3_pointer);
   ds_pointer_getbin (unit, &var->mds_vmr4_pointer);
   ds_pointer_getbin (unit, &var->mds_vmr5_pointer);
   ds_pointer_getbin (unit, &var->mds_vmr6_pointer);
   ds_pointer_getbin (unit, &var->mds_cont_pointer);
   ds_pointer_getbin (unit, &var->pcd_ads_pointer);
   ds_pointer_getbin (unit, &var->mw_ads_pointer);
   ds_pointer_getbin (unit, &var->res_spect_ads_pointer);
   ds_pointer_getbin (unit, &var->param_ads_pointer);
   char_array_getbin (unit, var->spare_1, 55);
}
    



void Scan_Geolocation_Ads_getbin (FILE* unit, Scan_Geolocation_Ads *var)
{
   MJD_getbin (unit, &var->dsr_time);
   char_getbin (unit, &var->attach_flag);
   Coord_deg_getbin (unit, &var->loc_first);
   double_getbin (unit, &var->first_alt);
   Coord_deg_getbin (unit, &var->loc_last);
   double_getbin (unit, &var->last_alt);
   Coord_deg_getbin (unit, &var->loc_mid);
   char_array_getbin (unit, var->spare_1, 47);
}


void Retrieval_P_T_getbin (FILE* unit, Retrieval_P_T *var, int Nsw)
{
   char_array_getbin (unit, var->lrv_p_t_flag, Nsw);
   float_array_getbin (unit, var->pressure, Nsw);
   float_array_getbin (unit, var->pressure_variance, Nsw);
   float_array_getbin (unit, var->tangent_altitude, Nsw);
   float_array_getbin (unit, var->height_cor_variance, Nsw);
   float_array_getbin (unit, var->temp, Nsw);
   float_array_getbin (unit, var->temp_variance, Nsw);
}


void Retrieval_Vmr_getbin (FILE* unit, Retrieval_Vmr *var, int Nsw)
{
   char_array_getbin (unit, var->lrv_vmr_flag, Nsw);
   float_array_getbin (unit, var->vmr, Nsw);
   float_array_getbin (unit, var->vmr_variance, Nsw);
   float_array_getbin (unit, var->concentration, Nsw);
   double_array_getbin (unit, var->concentration_variance, Nsw);
   float_array_getbin (unit, var->vertical_col_density, Nsw);
   double_array_getbin (unit, var->vcd_variance, Nsw);
}

void Retrieval_Vmr_array_getbin (FILE* unit, Retrieval_Vmr *var,
				 int Ngas, int Nsw)
{
    int n;
    for (n=0; n<Ngas; n++)
    {
	Retrieval_Vmr_getbin (unit, &var[n], Nsw);
    }
}



void Scan_Information_Mds_getbin (FILE* unit, Scan_Information_Mds *var,
				  int Nsw, int Ngas)
{
    if (Nsw  > N_SWEEP_MAX)
    {
	fprintf (stderr, "FATAL ERROR!!\n"
		 "scan_info_getbin (...):  N_SWEEP_MAX to small,"
		 " update programm! (Nsw = %i )\n",Nsw);
	exit(254);
    }	
    if (Ngas  > N_GAS_MAX)
    {
	fprintf (stderr, "FATAL ERROR!!\n"
		 "scan_info_getbin (...):  N_GAS_MAX to small,"
		 " update programm! (Ngas = %i )\n", Ngas);
	exit(254);
    }	
   MJD_getbin (unit, &var->dsr_time);
   uint_getbin (unit, &var->dsr_length);
   char_getbin (unit, &var->quality_flag);
   MJD_array_getbin (unit, var->zpd_crossing_time, Nsw);
   Coord_deg_array_getbin (unit, var->geolocation_los_tangent, Nsw);
   double_array_getbin (unit, var->tangent_altitude_los, Nsw);
   ushort_getbin (unit, &var->appl_process_id);
   char_getbin (unit, &var->retrieval_p_t_flag);
   char_array_getbin (unit, var->retrieval_vmr_flag, Ngas);
   char_array_getbin (unit, var->spare_1, 54);
   Retrieval_P_T_getbin (unit, &var->retrieval_p_t, Nsw);
   Retrieval_Vmr_array_getbin (unit, var->retrieval_vmr, Ngas, Nsw);
}


