#ifndef  __MIPAS_L2NRT                       /* Avoid multiple includes */
#define  __MIPAS_L2NRT

#include "scia_common.h"

#define N_SWEEP_MAX 30
#define N_GAS_MAX 6



typedef struct SPH_MIP_NL__2P {
        /*  SPH descriptorASCII string describing the product. */
    char sph_descriptor[29];
        /*  Value: +000= No stripline continuity, the product is a complete
            segment Other: Stripline Counter */
    char stripline_continuity_indicator[5];
        /*  Value: +001 to NUM_SLICESDefault value if no stripline continuity
            = +001 */
    char slice_position[5];
        /*  Number of slices in this striplineDefault value if no continuity
            = +001 */
    char num_slices[5];
        /*  ZPD time of first MDSR of the first scan in the product. */
    char start_time[28];
        /*  ZPD time of last MDSR of the last scan in the product. */
    char stop_time[28];
        /*  Latitude of LOS tangent point at center of scan (refraction corrected)
            of the first scan in the product. Positive north. */
    char first_tangent_lat[12];
        /*  Longitude of LOS tangent point at center of scan (refraction
            corrected) of the first scan in the product. Positive north. */
    char first_tangent_long[12];
        /*  Latitude of LOS tangent point at center of scan (refraction corrected)
            of the last scan in the product. Positive East. */
    char last_tangent_lat[12];
        /*  Longitude of LOS tangent point at center of scan (refraction
            corrected) of the last scan in the product. Positive East. */
    char last_tangent_long[12];
        /*  Spare */
    char spare_1[49];
        /*  No. of acquired elevation scans in Level 1 B input file */
    char num_scans[7];
        /*  Number of LOS geometries per elev. scan in nominal mode [Nacq] */
    char num_los_geoms[7];
        /*  Number of elevation scans per D.S. calibration cycle in nominal
            mode */
    char num_scans_per_ds[7];
        /*  No. of elevation scans in nominal mode processed */
    char num_scans_proc[7];
        /*  No. of elevation scans in special events mode not processed */
    char num_sp_not_proc[7];
        /*  No. of scene spectra in measurement interval */
    char num_spectra[7];
        /*  No. of scene spectra processed */
    char num_spectr_proc[7];
        /*  No. of gain calibration cycles in measurement time interval.
            This value will be 0 if no Gain Calibration ADSRs appeared in the
            input Level-1B product and 1 otherwise */
    char num_gain_cal[7];
        /*  Total number of elevation scans (&laquo;granules&raquo;)
            processed [Nscan] */
    char tot_granules[7];
        /*  Maximum path difference in nominal scene measurements */
    char max_path_diff[16];
        /*  String describing the sequence of species within this product.
            Example: H2O,N2O,HNO3,CH4,O3 unused characters set to blank.
            String is left justified. */
    char order_of_species[31];
        /*  Number of sweeps per nominal elevation scan */
    char num_sweeps_per_scan[7];
        /*  Spare */
    char spare_2[21];
} SPH_MIP_NL__2P ;




typedef struct ds_pointer {
        /*  Offset of first DSR. Set to -1 if DSR is missing. */
    int dsr_offset;
        /*  Size of DSR's */
    unsigned int dsr_length;
} ds_pointer;    




typedef struct Dataset_Structure_Ads {
        /*  Time of DSR (ZPD time of sweep closest to mean time of first scan
            this ADSR refers to) */
    MJD dsr_time;
        /*  Attachment flag */
    char attach_flag;
        /*  Number of sweeps per scan */
    unsigned short num_sweeps;
        /*  Number of retrieved pressure/temperature profile points */
    unsigned short num_press_temp_pts;
        /*  Number of retrieved VMR profile points */
    unsigned short num_vmr_pts[6];
        /*  Flags indicating existence of p,T error propagation data */
    unsigned short flags_p_t_error_flag[6];
        /*  Number of fitted continuum parameters in p,T retrievals */
    unsigned short num_con_params_p_t;
        /*  Number of fitted continuum parameters in VMR retrievals */
    unsigned short num_con_params_vmr[6];
        /*  Number of fitted instrument offset values in p,T retrievals */
    unsigned short num_instr_offset_p_t;
        /*  Number of fitted instrument offset values in VMR retrievals */
    unsigned short num_instr_offset_vmr[6];
        /*  Max number of microwindows per acquisition tangent height used
            for p, T retrieval */
    unsigned short max_num_micro_p_t;
        /*  Max number of microwindows per acquisition tangent height used
            for VMR retrieval */
    unsigned short max_num_micro_vmr[6];
        /*  Total number of p, T microwindows (all altitudes) processed
            in current scan */
    unsigned short tot_num_p_t_micro_all_alt;
        /*  Total number of VMR retrieval microwindows (all altitudes)
            processed in current scan */
    unsigned short tot_num_vmr_micro_all_alt[6];
        /*  Total number of spectral grid points contained in selected p,T
            microwindows */
    unsigned short tot_num_spect_grid_p_t;
        /*  Total number of spectral grid points contained in selected microwindows
            for VMR removals */
    unsigned short tot_num_spect_vmr[6];
        /*  Number of profile grid points to represent continuum profile
            data for p,T retrieval */
    unsigned short num_grid_con_p_t;
        /*  Number of profile grid points to represent continuum profile
            data for VMR retrievals */
    unsigned short num_grid_con_vmr[6];
        /*  Maximum number of evolution steps reported for p,T retrievals.
            Refers to corresponding PCD info ADSR. */
    short num_evo_steps_p_t;
        /*  Maximum number of evolution steps reported for VMR retrievals.
            Refers to corresponding PCD info ADSR. */
    short num_evo_steps_vmr[6];
        /*  Maximum number of PCD information strings in corresponding
            PCD info ADSR. */
    unsigned short num_pcd_info;
        /*  Offset and size of first corresponding record in Scan Info MDS */
    ds_pointer mds_scan_info_pointer;
        /*  Offset and size of first corresponding record in p, T profile
            MDS */
    ds_pointer mds_p_t_pointer;
        /*  Offset and size of first corresponding record in VMR #1 profile
            MDS */
    ds_pointer mds_vmr1_pointer;
        /*  Offset and size of first corresponding record in VMR #2 profile
            MDS */
    ds_pointer mds_vmr2_pointer;
        /*  Offset and size of first corresponding record in VMR #3 profile
            MDS */
    ds_pointer mds_vmr3_pointer;
        /*  Offset and size of first corresponding record in VMR #4 profile
            MDS */
    ds_pointer mds_vmr4_pointer;
        /*  Offset and size of first corresponding record in VMR #5 profile
            MDS */
    ds_pointer mds_vmr5_pointer;
        /*  Offset and size of first corresponding record in VMR #6 profile
            MDS */
    ds_pointer mds_vmr6_pointer;
        /*  Offset and size of first corresponding record in Offset and Continuum
            MDS */
    ds_pointer mds_cont_pointer;
        /*  Offset and size of first corresponding record in PCD ADS */
    ds_pointer pcd_ads_pointer;
        /*  Offset and size of first corresponding record in Microwindows
            Ocupation Matrix ADS */
    ds_pointer mw_ads_pointer;
        /*  Offset and size of first corresponding record in Residual Spectra
            ADS */
    ds_pointer res_spect_ads_pointer;
        /*  Offset and size of first corresponding record in Parameters
            ADS */
    ds_pointer param_ads_pointer;
        /*  Spare */
    char spare_1 [55];
} Dataset_Structure_Ads ;



/* DSD name enum */
extern char DS_NAME_MIP_NL__2P [26][29];

typedef enum DSD_MIP_NL__2P
{
    SUMMARY_QUALITY_ADS,
    SCAN_GEOLOCATION_ADS,
    DATASET_STRUCTURE_ADS,
    SCAN_INFORMATION_MDS,
    PT_RETRIEVAL_MDS,
    H2O_RETRIEVAL_MDS,
    N2O_RETRIEVAL_MDS,
    HNO3_RETRIEVAL_MDS,
    CH4_RETRIEVAL_MDS,
    O3_RETRIEVAL_MDS,
    NO2_RETRIEVAL_MDS,
    CONTINUUM_AND_OFFSET_MDS,
    PCD_INFORMATION_ADS,
    MICROWINDOW_OCCUPATION_ADS,
    RESIDUAL_SPECTRA_ADS,
    PROCESSING_PARAMETERS_ADS,
    REFERENCE_TO_USED_LEVEL_1B,
    REFERENCE_TO_USED_MIP_PS2_AX,
    REFERENCE_TO_USED_MIP_MW2_AX,
    REFERENCE_TO_USED_MIP_SP2_AX,
    REFERENCE_TO_USED_MIP_IG2_AX,
    REFERENCE_TO_USED_MIP_FM2_AX,
    REFERENCE_TO_USED_MIP_OM2_AX,
    REFERENCE_TO_USED_MIP_CS2_AX,
    REFERENCE_TO_USED_MIP_PI2_AX,
    ECMWF_FILE,
    MAX_DS_NAME_MIP_NL__2P
} DSD_MIP_NL__2P;




/* Type of Coords changed to real floats! Conversion done by eading routine */

typedef struct Scan_Geolocation_Ads {
        /*  Time of DSR (ZPD time of sweep closest to scans mean time) */
    MJD dsr_time;
        /*  Attachment Flag (set to 1 if all MDSRs corresponding to this ADSR
            are blank, set to zero otherwise) */
    char attach_flag;
        /*  Geolocation (lat. / long.) of first scene LOS tangent pointWGS84
            reference, refraction corrected */
    Coord_deg loc_first;
        /*  Tangent altitude of first scene LOS tangent point */
    double first_alt;
        /*  Geolocation (lat. / long.) of last scene LOS tangent pointWGS84
            reference, refraction corrected */
    Coord_deg loc_last;
        /*  Tangent altitude of last scene LOS tangent point */
    double last_alt;
        /*  Geolocation (lat. / long.) of LOS tangent point closest to scans
            mean time (WGS84reference, refraction corrected) */
    Coord_deg loc_mid;
        /*  Spare */
    char spare_1[47];
} Scan_Geolocation_Ads ;


typedef struct Retrieval_P_T {
        /*  Logical retrieval vector for p,T retrieval (Nsw). Set to zero
            if N/A. */
    char lrv_p_t_flag[N_SWEEP_MAX];
        /*  Retrieved pressure for each sweep. Set to NaN if N/A */
    float pressure[N_SWEEP_MAX];
        /*  Pressure variance for each sweep. Set to NaN if N/A */
    float pressure_variance[N_SWEEP_MAX];
        /*  Corrected tangent altitudes of sweeps. Set to NaN if N/A */
    float tangent_altitude[N_SWEEP_MAX];
        /*  Height correction variance for each sweep. Set to NaN if N/A */
    float height_cor_variance[N_SWEEP_MAX];
        /*  Retrieved temperatures for each sweep. Set to NaN if N/A */
    float temp[N_SWEEP_MAX];
        /*  Temperature variance for each sweep. Set to NaN if N/A */
    float temp_variance[N_SWEEP_MAX];
} Retrieval_P_T ;


typedef struct Retrieval_Vmr {
        /*  Logical retrieval vector for VMR retrieval (Nsw). Set to zero
            if N/A. */
    char lrv_vmr_flag[N_SWEEP_MAX];
        /*  Retrieved VMR for each sweep. Set to NaN if N/A */
    float vmr[N_SWEEP_MAX];
        /*  VMR variance for each sweep. Set to NaN if N/A */
    float vmr_variance[N_SWEEP_MAX];
        /*  Concentration for each sweep. Set to NaN if N/A */
    float concentration[N_SWEEP_MAX];
        /*  Concentration variance for each sweep. Set to NaN if N/A */
    double concentration_variance[N_SWEEP_MAX];
        /*  Vertical column density for each sweep. Set to NaN if N/A */
    float vertical_col_density[N_SWEEP_MAX];
        /*  Vertical column density variance for each sweep. Set to NaN if
            N/A */
    double vcd_variance[N_SWEEP_MAX];
} Retrieval_Vmr ;


typedef struct Scan_Information_Mds {
        /*  Time of DSR. ZPD time of sweep closest to scans mean time, s. */
    MJD dsr_time;
        /*  DSR length */
    unsigned int dsr_length;
        /*  Quality indicator (PCD). */
    char quality_flag;
        /*  ZPD crossing time of ith scene spectrun in elevation scan (i =
            1, ..., Nsw) */
    MJD zpd_crossing_time[N_SWEEP_MAX];
        /*  Geolocation (lat./long.) of ith scene LOS tangent point (i =
            1, ..., Nsw) */
    Coord_deg geolocation_los_tangent[N_SWEEP_MAX];
        /*  Tangent altitude of ith scene LOS tangent point (i = 1, ..., Nsw) */
    double tangent_altitude_los[N_SWEEP_MAX];
        /*  Application process ID */
    unsigned short appl_process_id;
        /*  Flag indicating successful p,T retrieval */
    char retrieval_p_t_flag;
        /*  Flag indicating successful VMR retrievals */
    char retrieval_vmr_flag[N_GAS_MAX];
        /*  Spare */
    char spare_1[54];
        /*  p, T retrievals. */
    Retrieval_P_T retrieval_p_t;
        /*  VMR retrievals for species 1 to 6. */
    Retrieval_Vmr retrieval_vmr[N_GAS_MAX];
} Scan_Information_Mds ;





void ds_pointer_getbin (FILE* unit, ds_pointer *var);
SCIA_err Read_SPH_MIP_NL__2P (FILE* unit, SPH_MIP_NL__2P *sph);
void Dataset_Structure_Ads_getbin (FILE* unit, Dataset_Structure_Ads *var);
void Scan_Geolocation_Ads_getbin (FILE* unit, Scan_Geolocation_Ads *var);
void Retrieval_P_T_getbin (FILE* unit, Retrieval_P_T *var, int Nsw);
void Retrieval_Vmr_getbin (FILE* unit, Retrieval_Vmr *var, int Nsw);
void Retrieval_Vmr_array_getbin (FILE* unit, Retrieval_Vmr *var,
				 int Ngas, int Nsw);
void Scan_Information_Mds_getbin (FILE* unit, Scan_Information_Mds *var,
				  int Nsw, int Ngas);







//void GAS_MDS_getbin (FILE* unit, GAS_MDS *var, int NV);
//
//SCIA_err Read_SPH_mipas (FILE* unit, SPH_mipas *sph);
//void Geo_mipas_getbin (FILE* unit, Geo_mipas *var);
//
//void sub_scan_info_pt_getbin (FILE* unit, sub_scan_info_pt *var, int Nsw);
//void sub_scan_info_vmr_getbin (FILE* unit, sub_scan_info_vmr *var, int Nsw);
//void scan_info_getbin (FILE* unit, scan_info *var,
//		       int Nsw);
//

#endif
