
/* all standard libraries are included by scia_defs.h */
#include "scia_defs.h"
#include "scia_L2_lib.h"

/* set by autoconf */
#define PROGRAM_VERSION PACKAGE_VERSION
#define PROG_NAME PACKAGE_NAME

/* some defaults for geo/time searches */
#define RMAX_DEFAULT 1000.0
#define SZAMAX_DEFAULT 180.0
#define DIST_MIN_DEFAULT 100000.0
#define DIST_DEFAULT -999.0
#define DTIME_DEFAULT 12.0
/* MIPAS error value */
#define ERR_VAL -99.0


				/*  Flags used for user driven flow control */
enum { HEAD, NOHEAD,
       INFOMODE, DATAMODE, OL_INITIAL_MODE,
       FPOINT_SET, FPOINT_UNSET, FPOINT_ALL, FPOINT_SINGLE, FPOINT_MATCH,
       FPOINT_AREA, FPOINT_INSIDE,
       SSC_CLEAR, SSC_BACK, SSC_BACKLONG, SSC_FOR, SSC_FORLONG, SSC_NUMBER,
       FTIME_SET, FTIME_UNSET, FILES_ORDERED, FILES_UNORDERED,
       AREA, NOAREA,
       VERBOSE, NONVERBOSE, 
       LIMB_FILE_WITH_ORBIT, LIMB_FILE_NO_ORBIT };


/* Deklaration of help functions */
void help_general (FILE* out,
		   const char* default_datastr_normal,
		   const char* default_datastr_fpoint);

void help_data_spec(FILE *out);
void help_limb (FILE* out,
		const char* default_datastr_limb,
		const char* default_datastr_limb_fpoint );
void help_mipas (FILE* out);




enum LIMB_PROFILE_MODE {LMM_MAIN, 
			LMM_ND,		       
			LMM_MAIN_ALL, LMM_ND_ALL, LMM_CONVERS, LMM_SCALED, LMM_GRID, LMM_VECTOR,
		     LMM_COR, LMM_RES, LMM_ADD, LMM_INFO,
		     LMM_GEO, LMM_GEO_STATE, LMM_TEST, LMM_MAX};

char lmm [LMM_MAX][10] = {
    "main",
    "num_dens",
    "main_all",    
    "num_d_all",
    "convers",
    "scaled",
    "grid",
    "vector",
    "cor",
    "res",
    "add",
    "info",
    "geo",
    "stategeo",
    "test"};


/* Main output routine for limp PROFILE mode 
   char *dir                      output directory, 
   user_data *ud                  SCIA data structure
   int limb_mode_many[LMM_MAX]    list with output modes
   int n_lmm                      number of elements in list above
   char *name_text                add. text to output file name
   int orbit_flag                 if set, orbit number is added to filename
 */

void write_limb_many (char *dir, user_data *ud,
		      int limb_mode_many[LMM_MAX],
		      int n_lmm, char *name_text, int orbit_flag)
{
    int n_write;
    int n,i;
    char out_name[2001];
    char date_for_name[16];
    char orbit_string[7] = "";
    FILE* out;
    int lmm_mode;
    char *fitting_window;
    int n1_out=1;		/* number of main species for output */

    static int n_state = -1;
    static int n_profile_per_state = 0;

    if (ud->prod.prod_o < SCI_OL__2P_LIM_IR0_H2O)
	fitting_window = ud->doas_fitting_window;
    else
	fitting_window = ud->bias_fitting_window;
    /* No fitting window for limb PTH */
    if (ud->prod.prod_o == SCI_OL__2P_LIM_PTH)
	fitting_window = "";
    
    /* check for new state */
    n_profile_per_state = ud->n_profile_in_state;
    /*
    if (n_state != ud->n_state)
    {
	n_profile_per_state = 0;
	n_state = ud->n_state;
    }
    */
    yyyymmdd_string (ud->state_date_str, date_for_name);
    /* add orbitnumber to name, if wanted */
    if (orbit_flag == LIMB_FILE_WITH_ORBIT)
	sprintf (orbit_string, "_%5.5d", ud->abs_orbit);

    for (n_write=0; n_write<n_lmm; n_write++)
    {
	lmm_mode = limb_mode_many[n_write];
				/*  open output file */
	sprintf (out_name, "%s/SCIA_OLlimb_%s%s_%02i_%s_%s%s.dat",
		 dir, date_for_name,
		 orbit_string,
		 n_profile_per_state,
		 ud->molecule, lmm[lmm_mode],
		 name_text);
	
	out = fopen(out_name, "w");

	if (! out )
	{			/* Cannot open file for writing! */
	    fprintf (stderr, "FATAL error: Cannot open file for "
		     "limb output!\n %s\n",
		     out_name);
	    exit(3);
	}
	/* write header for each info */
				/*  Header */
	fprintf (out,
		 "#Molecule   : %s  %s\n"
		 "#Product    : %s\n"
		 "#Date       : %s\n"
		 "#Tangentpt  : %7.2f %7.2f\n",
		 ud->molecule,
		 fitting_window,
		 ud->product,
		 ud->state_date_str,
		 ud->geo_L.tangent_coord[1].lat,
		 ud->geo_L.tangent_coord[1].lon);

	lmm_mode = limb_mode_many[n_write];
 	switch (lmm_mode)
	{
	    case LMM_MAIN:
	    case LMM_ND:
	    case LMM_MAIN_ALL:
	    case LMM_ND_ALL:
	    case LMM_SCALED:
		fprintf (out,
			 "#SZA(TOA)   : %7.2f\n"			 
			 "#Levels     : %i\n",			 
			 ud->geo_L.sol_zen_angle_toa[1],
			 ud->lim_ol.n_main);
		break;
	    default:
		;
		break;
	}

	
 	switch (lmm_mode = limb_mode_many[n_write])
	{

	    case LMM_MAIN_ALL:
		/* change n1_out from default 1 to number of main species */
		n1_out = ud->lim_ol.n1;
		/* continue with LMM_MAIN */
	    case LMM_MAIN:
		fprintf (out, "\n#Height  p           T        ");

		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "VMR          Err_VMR      "
			     "VCD          Err_VCD      ");
		}
		fprintf(out, "\n#km      hPa         K        ");
		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "parts Vol    %%            "
			     "mol/cm^2     %%            ");
		}
		fprintf(out, "\n");
		/* data lines  */
		for (n=0; n<ud->lim_ol.n_main; n++)
		{
		    int i;
		    fprintf(out,
			    "%7.2f %11.4e %7.2f ",
			    ud->lim_ol.tangent_height[n],
			    ud->lim_ol.tangent_pressure[n],
			    ud->lim_ol.tangent_temp[n]);
		    for (i=0; i < n1_out; i++)
		    {
			int idx = n * ud->lim_ol.n1 + i;
			fprintf(out,"%12.4e %12.4e %12.4e %12.4e",
				ud->lim_ol.main_species[idx].tang_vmr,
				ud->lim_ol.main_species[idx].err_tang_vmr,
				ud->lim_ol.main_species[idx].vert_col,
				ud->lim_ol.main_species[idx].err_vert_col);
		    }
		    fprintf(out, "\n");
		}
		break;
		/* print out Number density on height grid at middle of layer */
	    case LMM_ND_ALL:
		n1_out = ud->lim_ol.n1;
		/* only the first main species (n1_out = 1) */
	    case LMM_ND:

		fprintf (out,"\n#Height  ");

		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "NumDen       ");
		}
		fprintf(out, "\n#km      ");
		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "mol/cm^3     ");
		}
		fprintf(out, "\n");
		
		{
		    /* top layer boundary is 100 km  */
		    float prev_tang_height = 100;
		    for (n=0; n<ud->lim_ol.n_main; n++)
		    {
			int i;
			float th = (ud->lim_ol.tangent_height[n] + prev_tang_height) / 2.0;
			prev_tang_height = ud->lim_ol.tangent_height[n];
			fprintf(out,
				"%6.2f ",
				th );
			for (i=0; i < n1_out; i++)
			{
			    int idx = n + i * ud->lim_ol.n_main;
			    fprintf(out,"%12.4e ",
				    ud->lim_ol.add_diagnostic.num_den[idx]);
			}
			fprintf(out, "\n");
		    }
		}
		break;

		/* for later test checks */
	    case LMM_TEST   :

		fprintf (out,"\n#Height  ");

		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "NumDen       Num_Den(clc) covarianz    Num_Den(conv)");
		}
		fprintf(out, "\n#km      hPa          K        ");
		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "mol/cm^3     mol/cm^3      mol/cm^3      mol/cm^3      ");
		}
		fprintf(out, "\n");
		
		{
		    /* top layer boundary is 100 km  */
		    float prev_tang_height = 100;
		    for (n=0; n<ud->lim_ol.n_main; n++)
		    {
			int i;
			float th = (ud->lim_ol.tangent_height[n] + prev_tang_height) / 2.0;
			float d_th =  (prev_tang_height - ud->lim_ol.tangent_height[n]) * 1e5;
			prev_tang_height = ud->lim_ol.tangent_height[n];
			fprintf(out,
				"%7.2f %12.4e %7.2f ",
				th,
				ud->lim_ol.tangent_pressure[n],
				ud->lim_ol.tangent_temp[n]);
			for (i=0; i < n1_out; i++)
			{
			    int idx_data = n * ud->lim_ol.n1 + i;
			    int idx = n + i * ud->lim_ol.n_main;
			    fprintf(out,"%12.4e %12.4e %12.4e %12.4e ",
				    ud->lim_ol.add_diagnostic.num_den[idx] ,
				    ud->lim_ol.main_species[idx_data].vert_col / d_th, 
				    ud->lim_ol.add_diagnostic.diag_covar_matrix[idx] 
				    * ud->lim_ol.add_diagnostic.conv_num_den[ n ],
				    ud->lim_ol.main_species[idx_data].vert_col 
				    * ud->lim_ol.add_diagnostic.conv_num_den[ n ]);
			}
			fprintf(out, "\n");
		    }
		}
		break;

		
	    case LMM_CONVERS:
		n1_out = ud->lim_ol.n1;

		fprintf (out,"\n#Height  p           T        VMR_conv_fac ND_conv_fac  ");

		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "NumDen       Ini_NumDen   ");
		}
		fprintf(out, "\n#km      hPa         K        "
			"?            cm^-1(?)     ");
		for (i=0; i < n1_out; i++)
		{
		    fprintf (out, "mol/cm^3     mol/cm^3      ");
		}
		fprintf(out, "\n");
		
		for (n=0; n<ud->lim_ol.n_main; n++)
		{
		    int i;
		    fprintf(out,
			    "%7.2f %11.4e %7.2f %12.4e %12.4e ",
			    ud->lim_ol.tangent_height[n],
			    ud->lim_ol.tangent_pressure[n],
			    ud->lim_ol.tangent_temp[n],
			    ud->lim_ol.add_diagnostic.conv_vmr[n],
			    ud->lim_ol.add_diagnostic.conv_num_den[n]);
		    for (i=0; i < n1_out; i++)
		    {
			int idx_data = n * ud->lim_ol.n1 + i;
			int idx = n + i * ud->lim_ol.n_main;
			fprintf(out,"%12.4e %12.4e ",
				ud->lim_ol.add_diagnostic.num_den[idx],
				ud->lim_ol.add_diagnostic.ini_num_den[idx]);
		    }
		    fprintf(out, "\n");
		}
		break;
		

	    case LMM_SCALED:
				/*  Header */
		fprintf (out, "\n#Height  p            T        ");

		for (i=0; i < ud->lim_ol.n4; i++)
		{
		    fprintf (out, "VMR          Err_VMR      "
			     "VCD          Err_VCD      ");
		}
		fprintf(out,  "\n#km      hPa          K        ");
		for (i=0; i < ud->lim_ol.n4; i++)
		{
		    fprintf (out, "ppmV         %%            "
			     "mol/cm^2     %%            ");
		}
		fprintf(out, "\n");
		
		for (n=0; n<ud->lim_ol.n_main; n++)
		{
		    int i;
		    fprintf(out,
			    "%7.2f %12.4e %7.2f ",
			    ud->lim_ol.tangent_height[n],
			    ud->lim_ol.tangent_pressure[n],
			    ud->lim_ol.tangent_temp[n]);
		    for (i=0; i < ud->lim_ol.n4; i++)
		    {
			int idx = n * ud->lim_ol.n4 + i;
			fprintf(out,"%12.4e %12.4e %12.4e %12.4e",
				ud->lim_ol.scaled_profiles[idx].tang_vmr,
				ud->lim_ol.scaled_profiles[idx].err_tang_vmr,
				ud->lim_ol.scaled_profiles[idx].vert_col,
				ud->lim_ol.scaled_profiles[idx].err_vert_col);
		    }
		    fprintf(out, "\n");
		}
		break;

	    case LMM_GRID:
				/*  Header */
		fprintf (out, "#Levels     : %i\n"			 
			 "\n", ud->lim_ol.n_meas);
		fprintf (out,
			 "#Date                           "
			 "Height  p            T        "
			 "num_windows win_min win_max\n"
			 "#                               "
			 "km      hPa          K        "
			 "            nm      nm     \n");
		for (n=0; n<ud->lim_ol.n_meas; n++)
		{
		    char time[28];
		    UTC_String (&ud->lim_ol.measurement_grid[n].dsr_time,
				time);
		    fprintf (out, "%s   %7.2f %12.4e %7.2f %6i      "
			     "%7.1f %7.1f\n",
			     time,
			     ud->lim_ol.measurement_grid[n].tangent_height,
			     ud->lim_ol.measurement_grid[n].tangent_pressure,
			     ud->lim_ol.measurement_grid[n].tangent_temp,
			     ud->lim_ol.measurement_grid[n].num_windows,
			     ud->lim_ol.measurement_grid[n].win_min,
			     ud->lim_ol.measurement_grid[n].win_max );		    
		}
		break;
				/*  Statevektor */
	    case LMM_VECTOR:
				/*  Header */
		fprintf (out,
			 "#Levels     : %i\n"			 
			 "\n",
			 ud->lim_ol.n_state_vec);
		fprintf (out,
			 "#Value        Error         Type\n"
			 "#                             \n");
		for (n=0; n<ud->lim_ol.n_state_vec; n++)
		{
		    fprintf (out, "%12.4g  %12.4g  %.4s\n",
			     ud->lim_ol.state_vector[n].value,
			     ud->lim_ol.state_vector[n].error,
			     ud->lim_ol.state_vector[n].type);		    
		}
		break;
			 
	    case LMM_COR:
				/*  Header */
		fprintf (out,
			 "#Levels     : %i\n"			 
			 "\n",
			 ud->lim_ol.m_f);
		fprintf (out,
			 "#cor_matrix_entry\n"
			 "\n");
		for (n=0; n<ud->lim_ol.m_f   ; n++)
		{
		    fprintf (out, "%12.4g\n",
			     ud->lim_ol.correlation_matrix[n]);
		}
		break;
		
	    case LMM_RES:
				/*  Header */
		fprintf (out,
			 "#Levels     : %i\n"			 
			 "\n",
			 ud->lim_ol.n_state_vec);
		fprintf (out,"#");
		for (n=0; n<ud->lim_ol.n_i; n++)
		    fprintf (out, "Iteration_%1i  ", n+1);
		fprintf (out,"\n#\n");
		for (n=0; n<ud->lim_ol.n_state_vec; n++)
		{		
		    for (i=0; i<ud->lim_ol.n_i; i++)
		    {
			fprintf (out, "%12.4g  ",
				 ud->lim_ol.residuals
				 [ n +  i*ud->lim_ol.n_state_vec ] );
		    }
		    fprintf (out, "\n");
		}
		break;
		
	    case LMM_ADD:
				/*  Header */
		fprintf (out,
			 "\n#add_diagnostic_entry\n"
			 "\n");
		for (n=0; n<ud->lim_ol.n_ad   ; n++)
		{
		    fprintf (out, "%12.4g\n", ud->lim_ol.add_diag[n]);		    
		}
		break;
		
	    case LMM_INFO:
		fprintf (out,
			 "#Integration Time [s] : %8.4f\n"
			 "#Retrieval method     : %c\n"
			 "#Ref. Height     [km] : %8.4f\n"
			 "#Ref. pressure   hPa] : %8.4f\n"
			 "#Ref. p source        : %c\n"
			 "#RMS of the fit       : %12.4e\n"
			 "#Chi2 of the fit      : %12.4e\n"
			 "#Goodness of the fit  : %12.4e\n"
			 "#Number of Iterations : %2i\n"
			 "#N used wavelenghts   : %3i\n"
			 "#N reject. wavel.     : %3i\n"
			 "#Criteria flag        : %i\n",
			 ud->lim_ol.integr_time / (1.0 * INT_TIME_FAC),
			 ud->lim_ol.method,
                         ud->lim_ol.ref_height,
                         ud->lim_ol.ref_pressure,
                         ud->lim_ol.ref_pressure_source,
    			 ud->lim_ol.rms_fit,
    			 ud->lim_ol.chi_2_fit,
    			 ud->lim_ol.goodness_fit,
    			 ud->lim_ol.n_i,
    			 ud->lim_ol.n_used_wl,
    			 ud->lim_ol.n_rejected_wl,
     			 ud->lim_ol.criteria_flag);
		break;
	    case LMM_GEO:
				/*  Header */
		fprintf (out,
			 "#Levels     : %i\n"			 
			 "\n",
			 ud->lim_ol.n_ad);
		fprintf(out,"#Tangentheight(3)            "
			    "Tangentpoint (3* lat/lon)                                 "
			    "SZA TOA (3)                  "
			    "LOS TOA (3)                  "
			    "Rel. Azimuth (3)             "
			    "SubSatpoint (3)     "
			    "Sat.height "
			    "Earth rad. \n");
		    fprintf(out, "#km      km       km         "
			    "deg      deg       deg      deg       deg      deg        "
			    "deg      deg      deg        "
			    "deg      deg      deg        "
			    "deg      deg      deg        "
			    "deg      deg        "
			    "km         "
			    "km         \n");
		for (n=ud->n_level  - ud->n_geo_0 - ud->n_geo_L; 
		     n<ud->n_level  - ud->n_geo_0 ; n++)
		{
		    fprintf(out, "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f  %8.2f %8.2f  %8.2f %8.2f   "
			    "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f   "
			    "%8.2f   "
			    "%8.2f   \n",
			    ud->geo_L_array[n].tangent_height[0],
			    ud->geo_L_array[n].tangent_height[1],
			    ud->geo_L_array[n].tangent_height[2],
			    ud->geo_L_array[n].tangent_coord[0].lat,
			    ud->geo_L_array[n].tangent_coord[0].lon,
			    ud->geo_L_array[n].tangent_coord[1].lat,
			    ud->geo_L_array[n].tangent_coord[1].lon,
			    ud->geo_L_array[n].tangent_coord[2].lat,
			    ud->geo_L_array[n].tangent_coord[2].lon,
			    ud->geo_L_array[n].sol_zen_angle_toa[0],
			    ud->geo_L_array[n].sol_zen_angle_toa[1],
			    ud->geo_L_array[n].sol_zen_angle_toa[2],
			    ud->geo_L_array[n].los_zen_angle_toa[0],
			    ud->geo_L_array[n].los_zen_angle_toa[1],
			    ud->geo_L_array[n].los_zen_angle_toa[2],
			    ud->geo_L_array[n].rel_azi_angle_toa[0],
			    ud->geo_L_array[n].rel_azi_angle_toa[1],
			    ud->geo_L_array[n].rel_azi_angle_toa[2],
			    ud->geo_L_array[n].sub_sat_point.lat,
			    ud->geo_L_array[n].sub_sat_point.lon,
			    ud->geo_L_array[n].sat_geod_ht,
			    ud->geo_L_array[n].earth_rad);
		}
		break;
	
	    case LMM_GEO_STATE:
		/*  Header */
		fprintf (out,
			 "#StateDate  : %s\n"
			 "#Levels     : %i\n"			 
			 "\n",
			 ud->state_date_str,
			 ud->lim_ol.n_ad);
		fprintf(out, "#Tangentheight(3)            "
			"Tangentpoint (3* lat/lon)                                 "
			"SZA TOA (3)                  "
			"LOS TOA (3)                  "
			"Rel. Azimuth (3)             "
			"SubSatpoint (3)     "
			"Sat.height "
			"Earth rad. \n");
		fprintf(out, "#km      km       km         "
			"deg      deg       deg      deg       deg      deg        "
			"deg      deg      deg        "
			"deg      deg      deg        "
			"deg      deg      deg        "
			"deg      deg        "
			"km         "
			"km         \n");
		for (n=0; n<ud->n_level; n++)
		{
		    fprintf(out, "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f  %8.2f %8.2f  %8.2f %8.2f   "
			    "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f %8.2f   "
			    "%8.2f %8.2f   "
			    "%8.2f   "
			    "%8.2f   \n",
			    ud->geo_L_array[n].tangent_height[0],
			    ud->geo_L_array[n].tangent_height[1],
			    ud->geo_L_array[n].tangent_height[2],
			    ud->geo_L_array[n].tangent_coord[0].lat,
			    ud->geo_L_array[n].tangent_coord[0].lon,
			    ud->geo_L_array[n].tangent_coord[1].lat,
			    ud->geo_L_array[n].tangent_coord[1].lon,
			    ud->geo_L_array[n].tangent_coord[2].lat,
			    ud->geo_L_array[n].tangent_coord[2].lon,
			    ud->geo_L_array[n].sol_zen_angle_toa[0],
			    ud->geo_L_array[n].sol_zen_angle_toa[1],
			    ud->geo_L_array[n].sol_zen_angle_toa[2],
			    ud->geo_L_array[n].los_zen_angle_toa[0],
			    ud->geo_L_array[n].los_zen_angle_toa[1],
			    ud->geo_L_array[n].los_zen_angle_toa[2],
			    ud->geo_L_array[n].rel_azi_angle_toa[0],
			    ud->geo_L_array[n].rel_azi_angle_toa[1],
			    ud->geo_L_array[n].rel_azi_angle_toa[2],
			    ud->geo_L_array[n].sub_sat_point.lat,
			    ud->geo_L_array[n].sub_sat_point.lon,
			    ud->geo_L_array[n].sat_geod_ht,
			    ud->geo_L_array[n].earth_rad);
		}

	}
	fclose (out);


    }
    /*  update profile per state counter */
    n_profile_per_state++;
}



void outflag (FILE* out, unsigned short flag)
{
    unsigned short bool  = 32768;
    int i;
    int result;
    for (i=0; i < 16; i++)
    {
	result = (flag & bool) / bool;
	bool = bool >> 1;
	fprintf (out, "%1d", result);
    }
    fprintf (out, " ");
}

/* Function for printing a MIPAS profile */
void outprofile (user_data *ud, double dist, char *comment, char *path,
		 double lat, double lon)                   
{
    /* mipas_ooooo_date.prf */
    int i;
    FILE *out;
    char out_str[2000];
/*    size_t size = strlen(path) + 18 + MAX_UTC_STRING;
      out_str = malloc (size); */
    sprintf (out_str, "%s/mipas_%05d_%11.11s_%15.15s_%s.prf", path, ud->abs_orbit,
	     ud->date_str, ud->date_str+12, ud->species_str);
    printf ("%s\n",out_str);
    out = fopen (out_str, "w");
    if (out == 0)
    {
	fprintf (stderr,"Cannot open Output file: \n  %s.\n", out_str);
	exit (1);
    }
/*    free (out_str); */
    fprintf (out, "%s\n", ud->product);
    fprintf (out, "Version: %s\n", ud->software_ver);
    fprintf (out, "%05d  %14.9f  %s", ud->abs_orbit,
	     ud->frac_day, ud->date_str);
    /* add search modus information */
    if (dist != DIST_DEFAULT)
    {
	fprintf (out, "     Search: %7.1fkm    from (%7.2f,%7.2f)  %s\n",
		 dist,lat,lon, comment);
    }
    else
	fprintf (out, "\n");
	    
    fprintf (out,
	     "Altitude   "
	     "Tan_point  "
	     "Tan_point  "
	     "pressure   "
	     "Var_pres.  "
	     "Cor._altit."
	     "Var_alt.   "
	     "Temperat.  "
	     "Var_Temp.  "
	     "VMR_(%4s)     "
	     "Var_VMR        "
	     "Conc._(%4s)   "
	     "Var_Conc.      "
	     "VCD_(%4s)     "
	     "Var_VCD        \n"
	     "km         "
	     "lat        "
	     "lon        "
	     "hPa        "
	     "hPa^2      "
	     "km         "
	     "m^2        "
	     "K          "
	     "K^2        "
	     "ppm            "
	     "ppm^2          "
	     "cm^-3          "
	     "(cm^-3)^2      "
	     "cm^-2          "
	     "(cm^-2)^2      "
	     "ppm            \n",
	ud->species_str,              	     
	ud->species_str,              	     
	ud->species_str);
    for (i=0; i<ud->N_swp; i++)
    {
	if (isnan(ud->scan.tangent_altitude_los[i]))
	    ud->scan.tangent_altitude_los[i]=ERR_VAL; 
	if (isnan( ud->scan.geolocation_los_tangent[i].lat))
	    ud->scan.geolocation_los_tangent[i].lat=ERR_VAL;
	if (isnan( ud->scan.geolocation_los_tangent[i].lon))
	    ud->scan.geolocation_los_tangent[i].lon=ERR_VAL;
	if (isnan( ud->scan.retrieval_p_t.pressure[i]))
	    ud->scan.retrieval_p_t.pressure[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_p_t.pressure_variance[i]))
	    ud->scan.retrieval_p_t.pressure_variance[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_p_t.tangent_altitude[i]))
	    ud->scan.retrieval_p_t.tangent_altitude[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_p_t.height_cor_variance[i]))
	    ud->scan.retrieval_p_t.height_cor_variance[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_p_t.temp[i]))
	    ud->scan.retrieval_p_t.temp[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_p_t.temp_variance[i]))
	    ud->scan.retrieval_p_t.temp_variance[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_vmr[ud->pos_cur_species].vmr[i]))
	    ud->scan.retrieval_vmr[ud->pos_cur_species].vmr[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_vmr[ud->pos_cur_species].vmr_variance[i]))
	    ud->scan.retrieval_vmr[ud->pos_cur_species].vmr_variance[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_vmr[ud->pos_cur_species].concentration[i]))
	    ud->scan.retrieval_vmr[ud->pos_cur_species].concentration[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_vmr[ud->pos_cur_species].concentration_variance[i]))
	    ud->scan.retrieval_vmr[ud->pos_cur_species].concentration_variance[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_vmr[ud->pos_cur_species].vertical_col_density[i]))
	    ud->scan.retrieval_vmr[ud->pos_cur_species].vertical_col_density[i]=ERR_VAL;
	if (isnan( ud->scan.retrieval_vmr[ud->pos_cur_species].vcd_variance[i]))
	    ud->scan.retrieval_vmr[ud->pos_cur_species].vcd_variance[i]=ERR_VAL; 
	
	fprintf (out, "%10.4f %10.4f %10.4f "
		 "%10.4f %10.4f %10.4f %10.2f %10.4f %10.4f"
		 " %14.7E %14.7E %14.7E %14.7E %14.7E %14.7E\n",
		 ud->scan.tangent_altitude_los[i],
		 ud->scan.geolocation_los_tangent[i].lat,
		 ud->scan.geolocation_los_tangent[i].lon,
		 ud->scan.retrieval_p_t.pressure[i],
		 ud->scan.retrieval_p_t.pressure_variance[i],
		 ud->scan.retrieval_p_t.tangent_altitude[i],
		 ud->scan.retrieval_p_t.height_cor_variance[i],
		 ud->scan.retrieval_p_t.temp[i],
		 ud->scan.retrieval_p_t.temp_variance[i],
		 ud->scan.retrieval_vmr[ud->pos_cur_species].vmr[i],
		 ud->scan.retrieval_vmr[ud->pos_cur_species].vmr_variance[i],
		 ud->scan.retrieval_vmr[ud->pos_cur_species].concentration[i],
		 ud->scan.retrieval_vmr[ud->pos_cur_species].concentration_variance[i],
		 ud->scan.retrieval_vmr[ud->pos_cur_species].vertical_col_density[i],
		 ud->scan.retrieval_vmr[ud->pos_cur_species].vcd_variance[i]);
    }
    fclose (out);
}


/* Function for printing one line of data in  ud */
/*  if  ud  is the NULL vektor print the headerline */
enum { HEADER, UNIT, DATA };

void outline (FILE* out, char *datastring, user_data *ud, int what,
	      double dist, char *comment)
{
    unsigned i,j;
    char tab[]=" ";
    for (i=0; i<strlen(datastring); i++)
    {
	if (i>0)
	    fprintf (out, "%s", tab);	
	switch (datastring[i])
	{
	    /* ---------------------------------------------------- */
	    case 'a':		/* Orbit number */
		switch (what) {
		    case HEADER:
			fprintf (out, "Orbit ");
			break;
		    case UNIT:
			fprintf (out, " #    ");
			break;
		    case DATA:
			fprintf (out, "%05d ", ud->abs_orbit);
			break;
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'b':		/* time and data (ASCII) */
		switch (what) {
		    case HEADER:
			fprintf (out, "Date        Time            ");
			break;
		    case UNIT:
			fprintf (out, "                            ");
			break;
		    case DATA:
			fprintf (out, "%s ",ud->date_str);
			break;
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'c':		/* frac. days since 1.1.2000 */
		switch (what) {
		    case HEADER:
			fprintf (out, "Days 1/1/2000   ");
			break;
		    case UNIT:
			fprintf (out, "                ");
			break;
		    case DATA:
			fprintf (out, "%15.9f ",
				 ud->frac_day);
			break;
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'd':		/* center coordinaates */
		switch (what) {
		    case HEADER:
			fprintf (out, "Center:lat/lon    ");
			break;
		    case UNIT:
			fprintf (out, "deg      deg      ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f %8.3f ", 
					 ud->geo.cen_coor_nad.lat,
					 ud->geo.cen_coor_nad.lon );
				break;
			    case LIMB_OL:  /* Limb OL : mid Tangent point */
				fprintf (out, "%8.3f %8.3f ", 
					 ud->geo_L.tangent_coord[1].lat,
					 ud->geo_L.tangent_coord[1].lon );
				break;
			    case MIPAS:  /* Limb OL : mid Tangent point */
				fprintf (out, "%8.3f %8.3f ", 
					 ud->geo_mipas.loc_mid.lat,
					 ud->geo_mipas.loc_mid.lon );
				break;
			}
			break;
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'e':		/* corner coordinaates */
		switch (what) {
		    case HEADER:
			fprintf (out, "Corner_coord:4*(lat/lon)            "
				 "                                     ");
			break;
		    case UNIT:
			fprintf (out, "deg      deg      deg      deg      "
				 "deg      deg      deg      deg       ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f ",
				 ud->geo.cor_coor_nad[0].lat,
				 ud->geo.cor_coor_nad[0].lon,
				 ud->geo.cor_coor_nad[1].lat,
				 ud->geo.cor_coor_nad[1].lon,
				 ud->geo.cor_coor_nad[2].lat,
				 ud->geo.cor_coor_nad[2].lon,
				 ud->geo.cor_coor_nad[3].lat,
				 ud->geo.cor_coor_nad[3].lon );
				break;
			    case LIMB_OL:  /* Limb OL : mid Tangent point */
				fprintf (out, "%8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f ",
				 ud->corner_limb[0].lat,
				 ud->corner_limb[0].lon,
				 ud->corner_limb[1].lat,
				 ud->corner_limb[1].lon,
				 ud->corner_limb[2].lat,
				 ud->corner_limb[2].lon,
				 ud->corner_limb[3].lat,
				 ud->corner_limb[3].lon );
				break;
			    case MIPAS:
			fprintf (out, "---      ---      ---      ---      "
				 "---      ---      ---      ---       ");
			break;
			}				
			break;
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'f':		/* SZA */
		switch (what) {
		    case HEADER:
			fprintf (out, "SZA_TOA(start,middle,end) ");
			break;
		    case UNIT:
			fprintf (out, "deg      deg     deg      ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo.sol_zen_angle_toa[0],
					 ud->geo.sol_zen_angle_toa[1],
					 ud->geo.sol_zen_angle_toa[2]);
				break;
			    case  METEO:	/* L2met */
			    case MIPAS:
				fprintf (out, "---      ---     ---      ");
				break;
			    case LIMB_OL:  /* Limb OL  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo_L.sol_zen_angle_toa[0],
					 ud->geo_L.sol_zen_angle_toa[1],
					 ud->geo_L.sol_zen_angle_toa[2]);
				break;
			}
			break;
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'g':		/* LOS             */
		switch (what) {
		    case HEADER:
			fprintf (out, "LOS_TOA(start,middle,end)  ");
			break;
		    case UNIT:
			fprintf (out, "deg      deg      deg      ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo.los_zen_angle_toa[0],
					 ud->geo.los_zen_angle_toa[1],
					 ud->geo.los_zen_angle_toa[2]);
				break;
			    case  METEO:	/* L2met */
			    case MIPAS:
				fprintf (out, "---      ---     ---      ");
				break;
			    case LIMB_OL:  /* Limb OL  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo_L.los_zen_angle_toa[0],
					 ud->geo_L.los_zen_angle_toa[1],
					 ud->geo_L.los_zen_angle_toa[2]);
				break;
				
			}
		}
		break;
					
	    /* ---------------------------------------------------- */
	    case 'h':		/* Rel Azim. angle */
		switch (what) {
		    case HEADER:
			fprintf (out, "Rel_Azi(start,middle,end)  ");
			break;
		    case UNIT:
			fprintf (out, "deg      deg      deg      ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo.rel_azi_angle_toa[0],
					 ud->geo.rel_azi_angle_toa[1],
					 ud->geo.rel_azi_angle_toa[2]);
				break;
			    case  METEO:	/* L2met */
			    case MIPAS:
				fprintf (out, "---      ---     ---      ");
				break;
			    case LIMB_OL:  /* Limb OL  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo_L.rel_azi_angle_toa[0],
					 ud->geo_L.rel_azi_angle_toa[1],
					 ud->geo_L.rel_azi_angle_toa[2]);
				break;
				
			}
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'r':		/* SZA, LOS, rel.Azim at satellite (GOME) */
		switch (what) {
		    case HEADER:
			fprintf (out,
				 "SZA_TOA(start,middle,end)  "
				 "LOS_TOA(start,middle,end)  "
				 "Rel_Azi(start,middle,end)  ");
			break;
		    case UNIT:
			fprintf (out,
				 "deg      deg      deg      "
				 "deg      deg      deg      "
				 "deg      deg      deg      ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */
			    case LIMB_OL:  /* Limb OL  */
			    case MIPAS:
				fprintf (out,
					 "---      ---     ---      "
					 "---      ---     ---      "
					 "---      ---     ---      ");
				break;
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo.sol_zen_angle_sat[0],
					 ud->geo.sol_zen_angle_sat[1],
					 ud->geo.sol_zen_angle_sat[2]);
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo.sol_zen_angle_sat[0],
					 ud->geo.sol_zen_angle_sat[1],
					 ud->geo.sol_zen_angle_sat[2]);
				fprintf (out, "%8.3f %8.3f %8.3f ",
					 ud->geo.los_zen_angle_sat[0],
					 ud->geo.los_zen_angle_sat[1],
					 ud->geo.los_zen_angle_sat[2]);
				break;
			}
			break;
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'i':		/* Sat height */
		switch (what) {
		    case HEADER:
			fprintf (out, "Sat_hght ");
			break;
		    case UNIT:
			fprintf (out, "km       ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f ",
					 ud->geo.sat_geod_ht);
				break;
			    case  METEO:	/* L2met */
			    case MIPAS:
				fprintf (out, "---      ---     ---      ");
				break;
			    case LIMB_OL:  /* Limb OL  */
				fprintf (out, "%8.3f ",
					 ud->geo_L.sat_geod_ht);
				break;
				
			}
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'j':		/* Earth read */
		switch (what) {
		    case HEADER:
			fprintf (out, "Earthrad ");
			break;
		    case UNIT:
			fprintf (out, "km       ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%8.3f ",
					 ud->geo.earth_rad);
				break;
			    case  METEO:	/* L2met */
			    case MIPAS:
				fprintf (out, "---      ");
				break;				    
			    case LIMB_OL:  /* Limb OL  */
				fprintf (out, "%8.3f ",
					 ud->geo_L.earth_rad);
				break;
			}
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'p':		/*Subsatellite point (OL only)*/
		switch(what) {
		    case HEADER:
			fprintf (out, "Subsat.:lat/lon   ");
			break;
		    case UNIT:
			fprintf (out, "deg      deg      ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
				fprintf (out, "%8.3f %8.3f ",
					 ud->geo.sub_sat_point.lat,
					 ud->geo.sub_sat_point.lon );
				break;
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  METEO:	/* L2met */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
			    case MIPAS:
				fprintf (out, "---      ---      ");
				break;
			    case LIMB_OL:  /* Limb OL  */
				fprintf (out, "%8.3f %8.3f ",
					 ud->geo_L.sub_sat_point.lat,
					 ud->geo_L.sub_sat_point.lon );
				break;
			}
			break;
		}
		break;
				
	    /* ---------------------------------------------------- */
	    case 'k':		/* Subset Counter */
		switch (what) {
		    case HEADER:
			fprintf (out, "SSC ");
			break;
		    case UNIT:
			fprintf (out, "    ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%3d ",
					 ud->geo.subset_counter);
				break;
			    case LIMB_OL:  /* Limb OL : mid Tangent point */
				fprintf (out, "%3d ",
					 ud->subsetcounter);
				break;
			    case MIPAS:
				fprintf (out, "--  ");
				break;
			}
			break;
		}
		break;
				
	    /* ---------------------------------------------------- */
	    case 'l':		/* IT */
		switch (what) {
		    case HEADER:
			fprintf (out, "Int.time ");
			break;
		    case UNIT:
			fprintf (out, "s        ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */

				fprintf (out, "%8.4f ",
					 (double)ud->geo.integr_time /
					 (double)INT_TIME_FAC);
				break;
			    case LIMB_OL:  /* Limb OL : mid Tangent point */
				fprintf (out, "%8.4f ",
					 (double)ud->geo_L.integr_time /
					 (double)INT_TIME_FAC);
				break;
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
			    case MIPAS:
				fprintf (out, "---      ");
				break;				
			}
			break;
		}
		break;
				
	    /* ---------------------------------------------------- */
	    case 'm':		/* IT */
		switch (what) {
		    case HEADER:
			fprintf (out, "Int.time ");
			break;
		    case UNIT:
			fprintf (out, "1/16s    ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */

				fprintf (out, "%4d     ",
					 ud->geo.integr_time);
				break;
			    case LIMB_OL:  /* Limb OL : mid Tangent point */
				fprintf (out, "%4d     ",
					 ud->geo.integr_time);
				break;
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
			    case MIPAS:
				fprintf (out, "---      ");
				break;				

			}			
			break;
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'q':		/* Groundpixel number */
		switch (what) {
		    case HEADER:
			fprintf (out, "GP-Nr  ");
			break;
		    case UNIT:
			fprintf (out, "       ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  METEO:	/* L2met */
			    case  LIMB_OL:  /* Limb OL : mid Tangent point */
			    case MIPAS:
				fprintf (out, "---    ");
				break;
			    case  GOME:   /* GOME  */
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%4d   ",
					 ud->geo.gp);
				break;				
			}			
			break;
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'n':		/* Filename */
		switch (what) {
		    case HEADER:
			fprintf (out, "Product identification record                                  ");
			break;
		    case UNIT:
			fprintf (out, "                                                               ");
			break;
		    case DATA:
			fprintf (out, "%62s ",
				 ud->product);
			break;
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'o':		/* Software version */
		switch (what) {
		    case HEADER:
			fprintf (out, "Softw-Version  ");
			break;
		    case UNIT:
			fprintf (out, "               ");
			break;
		    case DATA:
			fprintf (out, "%14s ",
				 ud->software_ver);
			break;
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'y':		/* comment */
		if  (comment)
		{
		    unsigned int n;
		    switch (what) {
			case HEADER:
			case UNIT:
			    fputc ('-', out);
			    for (n=1; n<=strlen(comment); n++)
				fputc (' ', out);
			    break;
			case DATA:
			    fprintf (out, "%s ", comment);
			    break;
		    }
		}
		else
		    fprintf (out, "- ");			
		break;

	    /* ---------------------------------------------------- */
	    case 'z':		/* Distance */
		switch (what) {
		    case HEADER:
			fprintf (out, "Distance   ");
			break;
		    case UNIT:
			fprintf (out, "km         ");
			break;
		    case DATA:
			if ( dist != DIST_DEFAULT )				
			    fprintf (out, "%10.4f ",
				     dist);
			else
			    fprintf (out, "---        ");
			break;
		}
		break;		    		    
		
	    /* ---------------------------------------------------- */
	    case 'x':		/* yyyymmdd_hhmmss */
		switch (what){
		    char ymdh[16];
		    case HEADER:
			fprintf (out, "yyyymmdd_hhmmss ");
			break;
		    case UNIT:
			fprintf (out, "                ");
			break;
		    case DATA:
			yyyymmdd_string (ud->orbit_date, ymdh);
			fprintf (out, "%s ", ymdh);
			break;
		}
		break;
		
	    /* ---------------------------------------------------- */
	    case 'A':		/* VCD */
		switch (what) {
		    case HEADER:
			fprintf (out, "VCD:%7s     ", ud->molecule);
			break;
		    case UNIT:
			fprintf (out, "molecules/cm2   ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case DOAS:
				fprintf (out, "%14.7E  ", ud->doas.vcd);
				break;
			    case BIAS:
				fprintf (out, "%14.7E  ",
					 ud->bias.trc_gas_vcd);
				break;
			    case METEO:
				fprintf (out, "%14.7E  ",
					 ud->met.o3_vcd);
				break;
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				fprintf (out, "%14.7E  ",
					 ud->nad_ol.vcd[ud->main_prod]);
				break;
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:     /* L2N CLOUD_AEROSL */
			    case LIMB_OL:      /* Limb OL  */
			    case MIPAS:
				fprintf (out, "---             ");
				
				break;
			    case GOME:         /* GOME */
				fprintf (out, "%14.7E  ",
					 ud->gome.gome_irr.VCD_dat[ud->main_prod]);
				break;
			    case  GOME_4:   /* GOME_4  */
				fprintf (out, "%14.7E  ",
					 ud->gome_4.gome_irr.VCD_dat[ud->main_prod]);
				break;
				
			}
			break;
		}
		break;

	    /* ---------------------------------------------------- */
				/*  print additional fitted columns */
				/*  Nadir OL IR */
		
	    case '1':		/* VCD */
	    case '2':		/* VCD */
	    case '3':		/* VCD */
	    case '4':		/* VCD */
	    case '5':		/* VCD */
	    case '6':		/* VCD */
		if (ud->method == NADIR_OL)		
		{
		    char number[2] = "0";
		    int n_mol =  atoi (number);
		    double err = 0.0;
		    number[0] = datastring[i];
		    if (n_mol <= ud->n_prod)
		    {
			switch (what) {
			    case HEADER:
				fprintf (out, "VCD:%7s     error VCD   ",
					 ud->prod_str_ir[n_mol]);
				break;
			    case UNIT:
				fprintf (out, "molecules/cm2   %%           ");
				break;
			    case DATA:
				fprintf (out, "%14.7E  ",
					 ud->nad_ol.vcd[n_mol]);
				err = ud->nad_ol.vcd_err[n_mol];
				if (err >= 1000.0 )
				    fprintf (out, "%11.4E ", err );
				else
				    fprintf (out, "%11.6f ", err );
				break;

				break;
			}
		    }
		    else
			fprintf (out, "---             ");
		}
		else
		    fprintf (out, "---             ");
		break;			    

	    /* ---------------------------------------------------- */
	    case 'B':		/* VCD error */
	    {
		double err;

		switch (what) {
		    case HEADER:
			fprintf (out, "error VCD   ");
			break;
		    case UNIT:
			fprintf (out, "%%           ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case DOAS:
				err = ud->doas.vcd_err;
				break;
			    case BIAS:
				err =  ud->bias.err_vcd;
				break;
			    case METEO:
				err = ud->met.err_vcd;
				break;
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				err = ud->nad_ol.vcd_err[ud->main_prod];
				break;
			    case GOME:         /* GOME */
				err = ud->gome.
				    gome_irr.VCD_err[ud->main_prod];
				break;
			    case  GOME_4:   /* GOME_4  */
				err = ud->gome_4.
				    gome_irr.VCD_err[ud->main_prod];
				break;

			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:  /* Limb OL  */
			    case MIPAS:
				err = 0.0;
				break;
			}
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  METEO:	/* L2met */
			    case GOME:         /* GOME */
			    case GOME_4:         /* GOME_4 */
				if (err >= 1000.0 )
				    fprintf (out, "%11.4E ", err );
				else
				    fprintf (out, "%11.6f ", err );
				break;
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case  LIMB_OL:  /* Limb OL : mid Tangent point */
			    case MIPAS:
				fprintf (out, "---          ");
				break;
			}
		}
		break;
	    }

		
	    /* ---------------------------------------------------- */
	    case 'C':		/* VCD DU */
	    {
		double vcd;
		switch (what) {
		    case HEADER:
			fprintf (out, "VCD %7s  ", ud->molecule);
			break;
		    case UNIT:
			fprintf (out, "DU           ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case DOAS:
				vcd = ud->doas.vcd * DU_FAC;
				break;
			    case BIAS:
				vcd =  ud->bias.trc_gas_vcd * DU_FAC;
				break;
			    case METEO:
				vcd = ud->met.o3_vcd * DU_FAC;
				break;
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				vcd = ud->nad_ol.vcd[ud->main_prod] * DU_FAC;
				break;
			    case GOME:         /* GOME */
				vcd = ud->gome.ozon; /* available as DU */
				break;
			    case GOME_4:         /* GOME_4 */
				vcd = ud->gome_4.ozon; /* available as DU */
				break;
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:  /* Limb OL  */
			    case MIPAS:
				vcd = 0.0;
				break;
			}
			switch (ud->method)
			{
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
			    case  METEO:	/* L2met */
			    case GOME:         /* GOME */
			    case  GOME_4:   /* GOME_4  */
				if (vcd >= 1000.0 || vcd < 10.0 )
				    fprintf (out, "%11.4E  ", vcd );
				else
				    fprintf (out, "%11.6f  ", vcd );
				break;
				break;
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:  /* Limb OL  */
			    case MIPAS:
				fprintf (out, "---          ");
				break;
			}			    
		}
	    }
	    break;

	    
	    /* ---------------------------------------------------- */
	    case 'L':		/* Limb VCD profile */
		if (ud->method == LIMB_OL)
		{
		    unsigned int n_main = ud->lim_ol.n_main;
		    unsigned int j;
		    
		    switch (what) {
			case HEADER:
			    fprintf (out, "Levels ");
			    break;
			case UNIT:
			    fprintf (out, "#      ");
			    break;
			case DATA:
			    fprintf (out, "%4d   ", n_main);
			    break;
		    }

			/*  lin. fit parameter*/
		    for (j=0; j<n_main; j++)
		    {
			switch (what) {
			    case HEADER:
				fprintf (out,"(%2d)"
					 "Height   "
					 "Press.     "
					 "Temp.    "
					 "VMR         "
					 "err VMR     ", j);
				break;
			    case UNIT:
				fprintf (out,"    "
					 "km       "
					 "hPa        "
					 "K        "
					 "mol/cm^2    "
					 "%%           ");
				break;
			    case DATA:
				fprintf (out, "    "
					 "%8.3f "
					 "%10.3e "
					 "%8.3f "
					 "%11.4e "
					 "%11.4e ",
					 ud->lim_ol.tangent_height[j],
					 ud->lim_ol.tangent_pressure[j],
					 ud->lim_ol.tangent_temp[j],
					 ud->lim_ol.main_species[j].vert_col,
					 ud->lim_ol.main_species[j].err_vert_col);
				break;
			}
		    }
		}
		else
		    fprintf (out, "--- ");
		break;

	    /* ---------------------------------------------------- */
	    case 'P':		/* Limb VCD profile */
		if (ud->method == LIMB_OL)
		{
		    unsigned int n_main = ud->lim_ol.n_main;
		    unsigned int j;
		    
		    switch (what) {
			case HEADER:
			    fprintf (out, "Levels ");
			    break;
			case UNIT:
			    fprintf (out, "#      ");
			    break;
			case DATA:
			    fprintf (out, "%4d   ", n_main);
			    break;
		    }

			/*  loop over levels */
		    for (j=0; j<n_main; j++)
		    {
			switch (what) {
			    case HEADER:
				fprintf (out,"(%2d)"
					 "Height   "
					 "Press.     "
					 "Temp.    "
					 "VMR        "
					 "err VMR    ", j);
				break;
			    case UNIT:
				fprintf (out,"    "
					 "km       "
					 "hPa        "
					 "K        "
					 "ppmv       "
					 "%%          ");
				break;
			    case DATA:
				fprintf (out, "    "
					 "%8.3f "
					 "%10.3e "
					 "%8.3f "
					 "%11.4e "
					 "%11.4e ",
					 ud->lim_ol.tangent_height[j],
					 ud->lim_ol.tangent_pressure[j],
					 ud->lim_ol.tangent_temp[j],
					 ud->lim_ol.main_species[j].tang_vmr,
					 ud->lim_ol.main_species[j].err_tang_vmr);
				break;
			}
		    }
		}
		else
		    fprintf (out, "--- ");
		break;

			    
	    /* ---------------------------------------------------- */
	    case 'D':		/* SCD + err*/
	    {
		double err;
		switch (what) {
		    case HEADER:
			fprintf (out, "SCD            Err SCD     ");
			break;
		    case UNIT:
			fprintf (out, "molec/cm2      %%           ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  DOAS:	/* L2N DOAS */
				fprintf (out, "%14.7E ",
					 ud->doas.slant_col_den);
				err = ud->doas.err_slant_col;
				if (err >= 1000.0 || err < 0.1 )
				    fprintf (out, "%11.4E ", err );
				else
				    fprintf (out, "%11.6f ", err );
				break;
				    
			    case  GOME:	/* GOME */
				fprintf (out, "%14.7E ",
					 ud->gome.gome_irr.SlantCol[ud->main_prod] );
				err = ud->gome.gome_irr.SlantCol_err[ud->main_prod];
				if (err >= 1000.0 || err < 0.1 )
				    fprintf (out, "%11.4E ", err );
				else
				    fprintf (out, "%11.6f ", err );
				break;
				
			    case  GOME_4:	/* GOME_4 */
				fprintf (out, "%14.7E ",
					 ud->gome_4.gome_irr.SlantCol[ud->main_prod] );
				err = ud->gome_4.gome_irr.SlantCol_err[ud->main_prod];
				if (err >= 1000.0 || err < 0.1 )
				    fprintf (out, "%11.4E ", err );
				else
				    fprintf (out, "%11.6f ", err );
				break;
				
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				fprintf (out, "%14.7E ",
					 ud->nad_ol.slant_col_den);
				err = ud->nad_ol.err_slant_col;
				if (err >= 1000.0 || err < 0.1 )
				    fprintf (out, "%11.4E ", err );
				else
				    fprintf (out, "%11.6f ", err );
				break;
				   
			    case  BIAS:       /* L2N BIAS */
			    case  METEO:	/* L2met */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:      /* Limb OL  */
				fprintf (out, "---            ---          ");
				break;
			}
			break;
		}
	    }
	    break;

	    /* ---------------------------------------------------- */
	    case 'E':		/* Flags VCD */
		switch (what) {
		    case HEADER:
			fprintf (out, "Flag VCD         ");
			break;
		    case UNIT:
			fprintf (out, "                 ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  DOAS:	/* L2N DOAS */
				outflag (out, ud->doas.flag_vcd_flags);
				break;
			    case  BIAS:       /* L2N BIAS */
				outflag (out, ud->bias.flag_bias_vcd_fit_flags);
				break;
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				outflag (out, ud->nad_ol.flag_vcd_flags);
				break;
			    case GOME:
				outflag (out, ud->gome.gome_irr.VCD_ind);
				break;
			    case GOME_4:
				outflag (out, ud->gome_4.gome_irr.VCD_ind);
				break;
			    case  METEO:	/* L2met */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:      /* Limb OL  */
			    case MIPAS:
				fprintf (out, "---              ");
				break;
			}
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'F':		/* Fit quality parameters */
		switch (what) {
		    case HEADER:
			fprintf (out, "RMS            Chi-2          Goodn.fit  Iter.num ");
			break;
		    case UNIT:
			fprintf (out, "                                         #        ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  DOAS:	/* L2N DOAS */
				fprintf (out, "%14.7E %14.7E %10.7f ",
					 ud->doas.rms_chi_2_gof[0],
					 ud->doas.rms_chi_2_gof[1],
					 ud->doas.rms_chi_2_gof[2]);
				fprintf (out, "%4d     ", ud->doas.iter_num_fit_win);
				break;
			    case  BIAS:       /* L2N BIAS */
				fprintf (out, "%14.7E %14.7E %10.7f ",
					 ud->bias.rms_good_fit_win[0] ,
					 ud->bias.rms_good_fit_win[1] ,
					 ud->bias.rms_good_fit_win[2]);
				fprintf (out, "%4d     ", ud->bias.iter_num_fit_win);
				break;
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				fprintf (out, "%14.7E %14.7E %10.7f ",
					 ud->nad_ol.rms_fit,
					 ud->nad_ol.chi_2_fit,
					 ud->nad_ol.goodness_fit);
				fprintf (out, "%4d     ", ud->nad_ol.iter_count_nonlin_fit);
				break;
			    case GOME:
				fprintf (out, "%14.7E %14.7E %10.7f %4d     ",
					 ud->gome.gome_irr.
					 windowinf[4 * ud->main_win],
					 ud->gome.gome_irr.
					 windowinf[4 * ud->main_win + 1],
					 ud->gome.gome_irr.
					 windowinf[4 * ud->main_win + 2],
					 (int) ud->gome.gome_irr.
					 windowinf[4 * ud->main_win + 3]
				    );
				break;
			    case GOME_4:
				fprintf (out, "%14.7E %14.7E %10.7f %4d     ",
					 ud->gome_4.gome_irr.
					 windowinf[4 * ud->main_win],
					 ud->gome_4.gome_irr.
					 windowinf[4 * ud->main_win + 1],
					 ud->gome_4.gome_irr.
					 windowinf[4 * ud->main_win + 2],
					 (int) ud->gome_4.gome_irr.
					 windowinf[4 * ud->main_win + 3]
				    );
				break;

			    case  METEO:	/* L2met */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:      /* Limb OL  */
			    case MIPAS:
				fprintf (out, "---            ---            "
					 "---        ---      ");
				break;
			}
			break;
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'G':		/*  AMF (DOAS-NRT) */
		switch (what) {
		    case HEADER:
			fprintf (out, "AMFground      AMF-cloudtop   Flag AMF         ");
			break;
		    case UNIT:
			fprintf (out, "                                               ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  DOAS:	/* L2N DOAS */
				fprintf (out, "%14.7E %14.7E ",
					 ud->doas.amf_gr ,
					 ud->doas.amf_cl);
				outflag (out, ud->doas.flag_amf_flags);
				break;
			    case GOME:
				fprintf (out, "%14.7E %14.7E ",
					 ud->gome.gome_irr.
					     AMF_ground[ud->main_prod] ,
					 ud->gome.gome_irr.
					     AMF_top[ud->main_prod]);
				outflag (out, ud->gome.gome_irr.AMF_ind);
				break;				
			    case GOME_4:
				fprintf (out, "%14.7E %14.7E ",
					 ud->gome_4.gome_irr.
					     AMF_ground[ud->main_prod] ,
					 ud->gome_4.gome_irr.
					 AMF_top[ud->main_prod] );
				outflag (out, ud->gome_4.gome_irr.AMF_ind);
				break;				
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				fprintf (out, "%14.7E %14.7E ",
					 ud->nad_ol.amf_gr ,
					 ud->nad_ol.amf_cl);
				outflag (out, ud->nad_ol.flag_amf_flags);
				break;

			    case  BIAS:       /* L2N BIAS */
			    case  METEO:	/* L2met */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:      /* Limb OL  */
			    case MIPAS:
				fprintf (out, "---            ---            "
					 "---              ");				
			}
			break;
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'H':		/*  AMF (OL) */
		switch (what) {
		    case HEADER:
			fprintf (out, "AMFground      AMFgr-err      AMF-cloudtop   AMF-cldt-err   Flag AMF         ");
			break;
		    case UNIT:
			fprintf (out, "                                                                             ");
			break;
		    case DATA:
			switch (ud->method)
			{
			    case  NADIR_OL:	/* L2OL (BIAS+DOAS) */
				fprintf (out, "%14.7E %14.7E %14.7E %14.7E ",
					 ud->nad_ol.amf_gr ,
					 ud->nad_ol.amf_gr_err,
					 ud->nad_ol.amf_cl ,
					 ud->nad_ol.amf_cl_err);
				outflag (out, ud->nad_ol.flag_amf_flags);
				break;
			    case GOME_4:       /* GOME_4 */
				fprintf (out, "%14.7E %14.7E %14.7E %14.7E ",
					 ud->gome_4.gome_irr.
					 AMF_ground[ud->main_prod] ,
					 ud->gome_4.gome_irr.
					 AMF_ground_err[ud->main_prod] ,
					 ud->gome_4.gome_irr.
					 AMF_top[ud->main_prod] ,
					 ud->gome_4.gome_irr.
					 AMF_top_err[ud->main_prod]);
				outflag (out, ud->gome_4.gome_irr.AMF_ind);
				break;				
				
			    case  BIAS:       /* L2N BIAS */
			    case  DOAS:	/* L2N DOAS */
			    case  METEO:	/* L2met */
			    case  CLD_AER_OL:  /* L2ol CLOUD_AEROSL */
			    case  CLD_AER:	/* L2N CLOUD_AEROSL */
			    case LIMB_OL:      /* Limb OL  */
			    case GOME:         /* GOME  */
			    case MIPAS:
				fprintf (out, "---            ---            ---            "
					 "---            ---              ");
				break;

			}
			break;
		}
		break;

		/* I : All other values NRT-DOAS: */
		/*       - Number of fitting parameters, n_fit */
		/*       - Cross-correlation parameters, (n_fit * (n_fit-1))/2 */
		/*       - Flag describing the slant column */
		/*       - Reflectance from ground */
		/*       - Reflectance from cloud top */

		
	    /* ---------------------------------------------------- */
	    case 'I':		/* All other Parameterd DOAS */
				/*  Number fit parameters */
		if (ud->method == DOAS)
		{
		    unsigned int nfit = ud->doas.num_fit_para;
		    switch (what) {
			case HEADER:
			    fprintf (out, "Num fit-param ");
			    break;
			case UNIT:
			    fprintf (out, "#             ");
			    break;
			case DATA:
			    fprintf (out, "%4d          ",
				     ud->doas.num_fit_para);
			    break;
		    }
		    /*  Cross correllation*/
		    nfit = (nfit * (nfit-1)) / 2;
		    for (j=0; j<nfit; j++)
		    {
			switch (what) {
			    case HEADER:
				if (j==0)
				    fprintf (out, "Cross-cor  ");
				else
				    fprintf (out, "           ");
				break;
			    case UNIT:
			        fprintf (out, "           ");
				break;
			    case DATA:
				fprintf (out, "%10.7f ",
					 ud->doas.cross_corr_para[j]);
				break;
			}
		    }
		    /* Flag SCD */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Flag SCD         ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
			    outflag (out, ud->doas.flag_slant_col_flags);
			    break;
		    }

		    /* Reflectance */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Refl.grnd  Refl.cl    Refl.meas  ");
			    break;
			case UNIT:
			    fprintf (out, "                                 ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f %10.7f %10.7f ",
				     ud->doas.refl_ground ,
				     ud->doas.refl_cloud_top ,
				     ud->doas.measured_refl);
			    break;
		    }
		    break;
		}
		else
		    fprintf (out, "--- ");
		break;

				
	    /* ---------------------------------------------------- */
	    case 'J':		/* All other Parameter BIAS */
		if (ud->method == BIAS)
		{
		    unsigned int nfit = ud->bias.num_geophy_param_fit;
		    double err;
		
		    switch (what) {
			case HEADER:
			    fprintf (out, "Num fit-param ");
			    break;
			case UNIT:
			    fprintf (out, "#             ");
			    break;
			case DATA:
			    fprintf (out, "%4d          ",
				     ud->bias.num_geophy_param_fit);
			    break;
		    }
		    /*  Cross correllation*/
		    nfit = (nfit * (nfit-1)) / 2;
		    for (j=0; j<nfit; j++)
		    {
			switch (what) {
			    case HEADER:
				if (j==0)
				    fprintf (out, "Cross-cor  ");
				else
				    fprintf (out, "           ");
				break;
			    case UNIT:
			        fprintf (out, "           ");
				break;
			    case DATA:
				fprintf (out, "%10.7f ",
					 ud->bias.cross_cor_param[j]);
				break;
			}
		    }

		    /* Number of segments fitting window */
		    switch (what) {
			case HEADER:
			    fprintf (out, "NumSegm ");
			    break;
			case UNIT:
			    fprintf (out, "        ");
			    break;
			case DATA:
			    fprintf (out, "%2d      ",
				     ud->bias.num_seg_fit_win);
		    }


		    /* Height value and error  */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Height val  Error hv    ");
			    break;
			case UNIT:
			    fprintf (out, "                        ");
			    break;
			case DATA:
			    err = ud->bias.ht_val_and_err[0];
			    if (err >= 10.0 || err < 0.01 )
				fprintf (out, "%11.4E ", err );
			    else
				fprintf (out, "%11.6f ", err );
			    err = ud->bias.ht_val_and_err[1];
			    if (err >= 10.0 || err < 0.01 )
				fprintf (out, "%11.4E ", err );
			    else
				fprintf (out, "%11.6f ", err );			    
		    }
		    /* Flag hghtval  */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Flag hghtval     ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
			    outflag (out, ud->bias.flag_ht_var_usg);
			    break;
		    }
		    /* 1st-clos.  Error 1st-c  */
		    switch (what) {
			case HEADER:
			    fprintf (out, "1st-clos.     Error 1st-c   ");
			    break;
			case UNIT:
			    fprintf (out, "                            ");
			    break;
			case DATA:
			    fprintf (out, "%13.7E %13.7E ",
				     ud->bias.fit_fc_param ,
				     ud->bias.err_fit_fc_param );
			    break;
		    }
		    /* transmittance cut-off for the fitting window */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Transm.cut    ");
			    break;
			case UNIT:
			    fprintf (out, "              ");
			    break;
			case DATA:
			    fprintf (out, "%13.7E ",
				     ud->bias.rms_good_fit_win[3]);
			    break;
		    }		    
		}

		break;

		/*  all other values NADIR-OL */
	    /* ---------------------------------------------------- */
	    case 'K':
		
		if (ud->method == NADIR_OL)
		{
		    unsigned int n1 = ud->nad_ol.num_linear_param;
		    unsigned int n2 = ud->nad_ol.num_non_linear_param;
		    
		    switch (what) {
			case HEADER:
			    fprintf (out, "Num lin.fitp. Num nonlin.ftp ");
			    break;
			case UNIT:
			    fprintf (out, "#              ");
			    break;
			case DATA:
			    fprintf (out, "%4d          %4d           ",
				     ud->nad_ol.num_linear_param,
				     ud->nad_ol.num_non_linear_param);
			    break;
		    }


		    if (n1 > 0)
		    {
			/*  lin. fit parameter*/
			for (j=0; j<1; j++)
			{
			    switch (what) {
				case HEADER:
				    if (j==0)
					fprintf (out, "Lin fitpa. ");
				    else
					fprintf (out, "           ");
				    break;
				case UNIT:
				    fprintf (out, "           ");
				    break;
				case DATA:
				    fprintf (out, "%10.7f ",
					     ud->nad_ol.linear_fit_param[j]);
				    break;
			    }
			}
		    
			/*  Cross correllation*/
			n1 = (n1 * (n1-1)) / 2;
			for (j=0; j<n1; j++)
			{
			    switch (what) {
				case HEADER:
				    if (j==0)
					fprintf (out, "Lin cr-cor ");
				    else
					fprintf (out, "           ");
				    break;
				case UNIT:
				    fprintf (out, "           ");
				    break;
				case DATA:
				    fprintf (out, "%10.7f ",
					     ud->nad_ol.linear_fit_cross_corr[j]);
				    break;
			    }
			}
		    }

		    if (n2 > 0)
		    {
			/*  non lin. fit parameter*/
			for (j=0; j<1; j++)
			{
			    switch (what) {
				case HEADER:
				    if (j==0)
					fprintf (out, "NLin fitp. ");
				    else
					fprintf (out, "           ");
				    break;
				case UNIT:
				    fprintf (out, "           ");
				    break;
				case DATA:
				    fprintf (out, "%10.7f ",
					     ud->nad_ol.non_linear_fit_param[j]);
				    break;
			    }
			}
		    
			/*  nonlin Cross correllation*/
			n2 = (n2 * (n2-1)) / 2;
			for (j=0; j<n2; j++)
			{
			    switch (what) {
				case HEADER:
				    if (j==0)
					fprintf (out, "NL cr-cor  ");
				    else
					fprintf (out, "           ");
				    break;
				case UNIT:
				    fprintf (out, "           ");
				    break;
				case DATA:
				    fprintf (out, "%10.7f ",
					     ud->nad_ol.non_linear_fit_cross_corr[j]);
				    break;
			    }
			}
		    }

		    /*  Fitting output flag */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Fit-output-flag  ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
			    outflag (out, ud->nad_ol.fit_flags);
			    break;
		    }

		    /*  Temperature of reference spectrum */
		    switch (what) {
			case HEADER:
			    fprintf (out, "T ref. spectr  ");
			    break;
			case UNIT:
			    fprintf (out, "               ");
			    break;
			case DATA:
			    fprintf (out, "%14.7E ",
				     ud->nad_ol.temp_ref);
			    break;
		    }
		    break;
		}
		break;
		
	    /* --------------------------------------------------- */
		/* All other GOME , seperate for GOME / GOME_4 */		
	    case 'M':
		if (ud->method == GOME)
		{
		    switch (what) {
			case HEADER:
			    fprintf (out, "DOAS flag        ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
				/* DOAS: Flag indexing output */
			    outflag (out, ud->gome.gome_irr.DOAS_ind);
			    break;
		    }
		    /*  */
		    /* AMF: Total intensities for ground for
		       Nwin fitting windows */
		    /* AMF: Total intensities for cloud top for
		       Nwin fitting windows */
                    /* AMF: Measured Intensity for Nwin windows */
		    switch (what) {
			case HEADER:
			    fprintf (out, "AMF tot.int    AMF tot.int    AMF meas.int   ");
			    break;
			case UNIT:
			    fprintf (out, "(ground)       (cld-top)                     ");
			    break;
			case DATA:
		    fprintf (out, "%14.7E %14.7E %14.7E ",
			     ud->gome.gome_irr.
			     AMF_winground[ud->main_win] ,
			     ud->gome.gome_irr.
			     AMF_wintop[ud->main_win],
			     ud->gome.gome_irr.
			     AMF_winmeasured[ud->main_win]);
		    }
/* Statistics: Pixel contrast (3 PMDs: mean and standard deviation) ! */
/* Statistics: Pixel colour (sub pixel values) ! */
/* Statistics: Pixel colour gradient (Ch. 3) ! */
/* Statistics: Flag indexing output */
		    /*  */
		    switch (what) {
			case HEADER:
			    fprintf (out, "Stat. Pixel contrast 3 PMDs (mean/standard dev.                                           "
				     "Stat. 16 Pixel "
				     "colour         "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
                                     "Stat. col-grad "
				);
			    break;
			case UNIT:
			    fprintf (out, "                                                                                          "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
				     "               "
                                     "               ");
			    break;
			case DATA:
			    fprintf (out,
				     "%14.7E %14.7E %14.7E %14.7E %14.7E "
				     "%14.7E %14.7E %14.7E %14.7E %14.7E "
				     "%14.7E %14.7E %14.7E %14.7E %14.7E "
				     "%14.7E %14.7E %14.7E %14.7E %14.7E "
				     "%14.7E %14.7E %14.7E ",
			     ud->gome.gome_irr.stat_contrast[0],
			     ud->gome.gome_irr.stat_contrast[1],
			     ud->gome.gome_irr.stat_contrast[2],
			     ud->gome.gome_irr.stat_contrast[3],
			     ud->gome.gome_irr.stat_contrast[4],
			     ud->gome.gome_irr.stat_contrast[5],
			     ud->gome.gome_irr.stat_pixcolour[0],
			     ud->gome.gome_irr.stat_pixcolour[1],
			     ud->gome.gome_irr.stat_pixcolour[2],
			     ud->gome.gome_irr.stat_pixcolour[3],
			     ud->gome.gome_irr.stat_pixcolour[4],
			     ud->gome.gome_irr.stat_pixcolour[5],
			     ud->gome.gome_irr.stat_pixcolour[6],
			     ud->gome.gome_irr.stat_pixcolour[7],
			     ud->gome.gome_irr.stat_pixcolour[8],
			     ud->gome.gome_irr.stat_pixcolour[9],
			     ud->gome.gome_irr.stat_pixcolour[10],
			     ud->gome.gome_irr.stat_pixcolour[11],
			     ud->gome.gome_irr.stat_pixcolour[12],
			     ud->gome.gome_irr.stat_pixcolour[13],
			     ud->gome.gome_irr.stat_pixcolour[14],
			     ud->gome.gome_irr.stat_pixcolour[15],
			     ud->gome.gome_irr.stat_gradient);
			    break;
		    }
		    switch (what) {
			case HEADER:
			    fprintf (out, "Stat. flag      ");
			    break;
			case UNIT:
			    fprintf (out, "                ");
			    break;
			case DATA:
				/* DOAS: Flag indexing output */
			    outflag (out, ud->gome.gome_irr.stat_ind);
			    break;
		    }

		}
		if (ud->method == GOME_4)
		{
		    switch (what) {
			case HEADER:
			    fprintf (out, "DOAS flag        ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
				/* DOAS: Flag indexing output */
			    outflag (out, ud->gome_4.gome_irr.DOAS_ind);
			    break;
		    }
		    switch (what) {
			case HEADER:
			    fprintf (out, "O3 temp.       O3 Ring corfac Ghost VCol     ");
			    break;
			case UNIT:
			    fprintf (out, "K                             molecules/cm2  ");
			    break;
			case DATA:
		    fprintf (out, "%14.7E %14.7E %14.7E ",
			     ud->gome_4.gome_irr.ozone_temp,
			     ud->gome_4.gome_irr.ozone_ring,
			     ud->gome_4.gome_irr.gvc);
		    }
		    switch (what) {
			case HEADER:
			    fprintf (out, "Surf.height    Surf.pres.     Surf.Albedo    ");
			    break;
			case UNIT:
			    fprintf (out, "km             hPa                           ");
			    break;
			case DATA:
		    fprintf (out, "%14.7E %14.7E %14.7E ",
			     ud->gome_4.gome_irr.surface[0],
			     ud->gome_4.gome_irr.surface[1],
			     ud->gome_4.gome_irr.surface[2]);
		    }
		}

		break;

		/* Start with CLOUDS_AEROSOL */
	    /* ---------------------------------------------------- */
	    case 'S':
		if (ud->method == CLD_AER)
		{
		    unsigned int nfit;
		    switch (what) {
			case HEADER:
			    fprintf (out, "#Cld_cv ");
			    break;
			case UNIT:
			    fprintf (out, "        ");
			    break;
			case DATA:
			    fprintf (out, "%4d    ",
				     ud->cld_aer.num_pmd);
			    break;
		    }

		    nfit = ud->cld_aer.num_pmd;
		    for (j=0; j<nfit; j++)
		    {
			switch (what) {
			    case HEADER:
				if (j==0)
				    fprintf (out, "Cld_cover  ");
				else
				    fprintf (out, "           ");
				break;
			    case UNIT:
				fprintf (out, "           ");
				break;
			    case DATA:
				fprintf (out, "%10.7f ",
					 ud->cld_aer.cl_frac[j]);
				break;
			}
		    }
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'Q':		/* Average Cloud cover  */
		if (ud->method == CLD_AER)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_cover   Qual.Cldcv ");
			    break;
			case UNIT:
			    fprintf (out, "                       ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f  %4d       ",
				     ud->cld_aer.ave_clo_fract_foot,
				     ud->cld_aer.qual_frac_cloud_cov);
			    break;
		    }
		}
		if (ud->method == CLD_AER_OL)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_cover   Err. Cldcv ");
			    break;
			case UNIT:
			    fprintf (out, "                       ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f %10.7f  ",
				     ud->cld_aer_ol.cl_frac,
				     ud->cld_aer_ol.cl_frac_err);
			    break;
		    }
		}
		if (ud->method == GOME)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_cover   Err. Cldcv ");
			    break;
			case UNIT:
			    fprintf (out, "                       ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f %10.7f  ",
				     ud->gome.gome_irr.cloud[0],
				     ud->gome.gome_irr.cloud[2]);
			    break;
		    }
		}

		if (ud->method == GOME_4)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_cover   Err. Cldcv ");
			    break;
			case UNIT:
			    fprintf (out, "                       ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f %10.7f  ",
				     ud->gome_4.gome_irr.cloud[0],
				     ud->gome_4.gome_irr.cloud[2]);
			    break;
		    }
		}
		    
		break;
		
		/* ----------------------------------------------- */
 	    case 'O':		/* Cloud top pressure              */
		if (ud->method == CLD_AER)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_top_press ");
			    break;
			case UNIT:
			    fprintf (out, "hPa           ");
			    break;
			case DATA:
			    fprintf (out, "%13.7E ",
				     ud->cld_aer.cl_top_pres);
			    break;
		    }
		}
		if (ud->method == CLD_AER_OL)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_top_press Err. cldtp  ");
			    break;
			case UNIT:
			    fprintf (out, "hPa           %%           ");
			    break;
			case DATA:
			    fprintf (out, "%13.7E %10.7f ",
				     ud->cld_aer_ol.cl_top_pres,
				     ud->cld_aer_ol.cl_top_pres_err) ;
			    break;
		    }
		}
		if (ud->method == GOME )
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_top_press Err. cldtp  ");
			    break;
			case UNIT:
			    fprintf (out, "hPa           %%           ");
			    break;
			case DATA:
			    fprintf (out, "%13.7E %10.7f ",
				     ud->gome.gome_irr.cloud[1],
				     ud->gome.gome_irr.cloud[3]);
			    break;
		    }
		}
		if (ud->method == GOME_4 )
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld_top_press Err. cldtp  ");
			    break;
			case UNIT:
			    fprintf (out, "hPa           %%           ");
			    break;
			case DATA:
			    fprintf (out, "%13.7E %10.7f ",
				     ud->gome_4.gome_irr.cldtop_p[0],
				     ud->gome_4.gome_irr.cldtop_p[1]);
			    break;
		    }
		}
		break;
		/* -------------------------------------------------- */
	    case 'N':		/* Absorbing Aerosol Indicator  */
		if (ud->method == CLD_AER || ud->method == CLD_AER_OL)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "AbsAer_Ind ");
			    break;
			case UNIT:
			    fprintf (out, "           ");
			    break;
			case DATA:
			    switch (ud->method)
			    {
				case CLD_AER:
				    fprintf (out, "%10.7f ",
					     ud->cld_aer.aero_abso_ind);
				    break;
				case CLD_AER_OL:
				    fprintf (out, "%10.7f ",
					     ud->cld_aer.aero_abso_ind);
				    break;
				case BIAS:
				case DOAS:
				case METEO:
				case NADIR_OL:
				case LIMB_OL:
				case GOME:
				case GOME_4:
				case MIPAS:
				    fprintf (out, "--         ");
				    break;
			    }
		    }
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'T':		/* Eq.Lamb_Alb  */
		if (ud->method == CLD_AER)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Eq.Lamb_Alb ");
			    break;
			case UNIT:
			    fprintf (out, "            ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f  ",
				     ud->cld_aer.equiv_lam_alb);
			    break;
		    }
		}
		break;

	    /* ---------------------------------------------------- */
	    case 'R':		/* Flag VCD fitting  */
		if (ud->method == CLD_AER)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Flag Cld_Aerosol ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
			    outflag (out, ud->cld_aer.flag_output_flags);
			    break;
		    }
		}
		if (ud->method == CLD_AER_OL)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Flag Cld_param.  ");
			    break;
			case UNIT:
			    fprintf (out, "                 ");
			    break;
			case DATA:
			    outflag (out, ud->cld_aer_ol.cloud_flags);
			    break;
		    }
		}
		break;

		/* ---------------------------------------------- */
	case 'V': 		/* OL Number of PMD */
		if (ud->method == CLD_AER_OL)
		{
		    switch (what) {
			case HEADER:
			    fprintf (out, "#Subpix #cloudy #cldfree ");
			    break;
			case UNIT:
			    fprintf (out, "                         ");
			    break;
			case DATA:
			    fprintf (out, "%4d    %4d    %4d     ",
				     ud->cld_aer_ol.pmd_read,
				     ud->cld_aer_ol.pmd_read_cl[0],
				     ud->cld_aer_ol.pmd_read_cl[1]);
			    break;
		    }
		}
		break;

	    	    /* ---------------------------------------------------- */
	    case 'W':		/* Optical depth */
		if (ud->method == CLD_AER_OL)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Cld opt.dpth err_opt_dpth "
				     "Flag Aer.output  ");
			    break;
			case UNIT:
			    fprintf (out, "                          "
				     "                 ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f    %10.7f    ",
				     ud->cld_aer_ol.cl_opt_depth,
				     ud->cld_aer_ol.cl_opt_depth_err);
			    outflag (out, ud->cld_aer_ol.cl_type_flags);
			    break;
		    }
		}
		break;

	    	    /* ---------------------------------------------------- */
	    case 'X':		/* Reflectance  */
		if (ud->method == CLD_AER_OL)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Refl_cldtop  err_cldtop "
				     "Refl_surfac err_surface ");
			    break;
			case UNIT:
			    fprintf (out, "                        "
				     "                        ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f  %10.7f  %10.7f  %10.7f  ",
				     ud->cld_aer_ol.cl_reflectance,
				     ud->cld_aer_ol.cl_reflectance_err,
				     ud->cld_aer_ol.surf_reflectance,
				     ud->cld_aer_ol.surf_reflectance_err
				     );
			    break;
		    }
		}
		break;
		if (ud->method == GOME_4)
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Alb_cldtop  err_cldtop "
				     "Surf_Alb    ");
			    break;
			case UNIT:
			    fprintf (out, "                        "
				     "             ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f %10.7f %10.7f ",
				     ud->gome_4.gome_irr.cldtop_alb[0],
				     ud->gome_4.gome_irr.cldtop_alb[1],
				     ud->gome_4.gome_irr.surface[2]);
			    break;
		    }
		}
		break;

	    	    /* ---------------------------------------------------- */
	    case 'Y':		/* Aerosol diagnostics                      */
		if (ud->method == CLD_AER_OL)
		{
		    switch (what) {
			case HEADER:
			    fprintf (out, "Aeros_diagn  Flag Aer.output  ");
			    break;
			case UNIT:
			    fprintf (out, "                               ");
			    break;
			case DATA:
			    fprintf (out, "%10.7f ",
				     ud->cld_aer_ol.aero_abso_ind);
			    outflag (out, ud->cld_aer_ol.flag_output_flags);
			    break;
		    }
		}
		break;
	    /* ---------------------------------------------------- */
	    case 'Z':
		if (ud->method == CLD_AER_OL)
		{
		    unsigned int nfit;
		    switch (what) {
			case HEADER:
			    fprintf (out, "#Add_aer ");
			    break;
			case UNIT:
			    fprintf (out, "         ");
			    break;
			case DATA:
			    fprintf (out, "%4d     ",
				     ud->cld_aer_ol.num_aero_param);
			    break;
		    }

		    nfit = ud->cld_aer_ol.num_aero_param;
		    for (j=0; j<nfit; j++)
		    {
			switch (what) {
			    case HEADER:
				if (j==0)
				    fprintf (out, "Add_aer_par");
				else
				    fprintf (out, "           ");
				break;
			    case UNIT:
				fprintf (out, "           ");
				break;
			    case DATA:
				fprintf (out, "%10.7f ",
					 ud->cld_aer_ol.aero_param[j]);
				break;
			}
		    }
		}
		break;
		/* Other GOME cloud values */
	    case 'U':
		if (ud->method == GOME )
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Surface_p  CCA_cldfrc  "
				     "                            "
				     "                            "
				     "                       ");
			    break;
			case UNIT:
			    fprintf (out, "hPa                    "
				     "                            "
				     "                            "
				     "                       ");
			    break;
			case DATA:
			    fprintf (out, "%10.7E %10.7f %3d %3d %3d "
				     "%3d %3d %3d %3d %3d %3d %3d "
				     "%3d %3d %3d %3d %3d %3d ",
				     ud->gome.gome_irr.cloud[4],
				     ud->gome.gome_irr.CCA_cf,
				     ud->gome.gome_irr.CCA_subpixel[0],
				     ud->gome.gome_irr.CCA_subpixel[1],
				     ud->gome.gome_irr.CCA_subpixel[2],
				     ud->gome.gome_irr.CCA_subpixel[3],
				     ud->gome.gome_irr.CCA_subpixel[4],
				     ud->gome.gome_irr.CCA_subpixel[5],
				     ud->gome.gome_irr.CCA_subpixel[6],
				     ud->gome.gome_irr.CCA_subpixel[7],
				     ud->gome.gome_irr.CCA_subpixel[8],
				     ud->gome.gome_irr.CCA_subpixel[9],
				     ud->gome.gome_irr.CCA_subpixel[10],
				     ud->gome.gome_irr.CCA_subpixel[11],
				     ud->gome.gome_irr.CCA_subpixel[12],
				     ud->gome.gome_irr.CCA_subpixel[13],
				     ud->gome.gome_irr.CCA_subpixel[14],
				     ud->gome.gome_irr.CCA_subpixel[15]);
			    break;
		    }
		}
		if (ud->method == GOME )
		{	  
		    switch (what) {
			case HEADER:
			    fprintf (out, "Surface_p  ");
			    break;
			case UNIT:
			    fprintf (out, "hPa        ");
			    break;
			case DATA:
			    fprintf (out, "%10.7E ",
				     ud->gome_4.gome_irr.surface[1]);
			    break;
		    }
		}
		break;
	    default:
		break;
	}
		
    }
    fprintf (out, "\n");
}





/* structure for holding information  */

#define SEARCHLIST_COMMENT_MAX 200

typedef struct search_list {
    Coord_deg pos;		/* position of search point */
    Coord_deg area_pos[4];	/* corner coordinate in case of area search */
    MJD mjd;			/* time  */
				/*  comment for output */
    char comment[SEARCHLIST_COMMENT_MAX];
    user_data *ud;		/* hold user_data for single match */
    int flag;			/* user_data found? */
    double dist_minimum;	/* distance nearest point */
    int area_flag;		/* AREA / NOAREA */
} search_list;


int compare_search_list_by_time (const void *sl1,
				 const void *sl2)
{
    return MJD_compare (& (((search_list*)sl1)->mjd),
			& (((search_list*)sl2)->mjd));
}

/* read the file with a list of search points to be searched in the
   files */
int read_search_list (FILE *in, int *n_search, search_list **search,
		      int area_flag )

{
    int n;
    int N;
    char line[1000];
    char comment_tmp[1000];
    char date_tmp[1000];
    char time_tmp[1000];
				/*  count lines in file*/
    for (N = 0;  ; N++)
    {
	fgets (line, 1000, in);
	if (feof(in))
	    break;
    }
    rewind (in);
    
    if (N == 0)
	return 0;
				/*  reserve memory */
    if (! (*search = calloc (N , sizeof (search_list))))

    {
	fprintf (stderr, "Out of memory in 'read_search_list ()'!\n");
	exit (255);
    }
				/*  Now start reading */
    for (n=0; n<N; n++)
    {
	int n_read;

	fgets (line, 1000, in);
				/*  ensure a trailing '\0' */
	memset (comment_tmp, '\0', 1000);
	switch (area_flag)
	{
	    case NOAREA:   	/* usual search */
		n_read = sscanf (line, "%f %f %s %s %999c",
				 &(*search)[n].pos.lat,
				 &(*search)[n].pos.lon,
				 date_tmp,
				 time_tmp,
				 comment_tmp);
		if (n_read < 4)
		    return 0;
		break;
	    case AREA:
		n_read = sscanf (line, "%f %f %f %f %f %f %f %f %s %s %999c",
				 &(*search)[n].area_pos[0].lat,
				 &(*search)[n].area_pos[0].lon,
				 &(*search)[n].area_pos[1].lat,
				 &(*search)[n].area_pos[1].lon,
				 &(*search)[n].area_pos[2].lat,
				 &(*search)[n].area_pos[2].lon,
				 &(*search)[n].area_pos[3].lat,
				 &(*search)[n].area_pos[3].lon,
				 date_tmp,
				 time_tmp,
				 comment_tmp);
		if (n_read < 10)
		    return 0;
		break;

	    default:
		fprintf (stderr, "Fatal programm error in read_search_list ()!\n");
		exit (4);
		break;
	}
		
				/*  delete newline at end of line/comment */
	if (comment_tmp [strlen(comment_tmp) - 1] == '\n')
	    comment_tmp [strlen(comment_tmp) - 1] = '\0';
	
	strncpy (date_tmp + 12, time_tmp, 16);
	if (! ASCII_2_MJD (date_tmp, &(*search)[n].mjd))
	{
	    return 0;
	}

	if ( (area_flag==NOAREA && n_read == 5) ||
	     (area_flag==AREA   && n_read == 11) )
	    strncpy ((*search)[n].comment, comment_tmp,
		     SEARCHLIST_COMMENT_MAX);
	comment_tmp[SEARCHLIST_COMMENT_MAX - 1] = '\0';
	
	(*search)[n].flag = FPOINT_UNSET;
	(*search)[n].dist_minimum = DIST_MIN_DEFAULT;
	(*search)[n].ud = NULL;

	
    }
				/*  sort the list */
    qsort (*search, N, sizeof(search_list),
	   compare_search_list_by_time);
    *n_search = N;
    return 1;
}


/* ********************************************************************** */
/* main start */
/* ********************************************************************** */


int main(int argc, char* argv[])
{
    /* Build command line interface with getopt */
    char optstring[] = "p:d:o:nis:f:F:r:c:t:u:ab:l:h:L:D:M:Ij:k:x:OvR:Q:YN";
    int c,nline;
    char default_datastr_normal[] = "abdAB";
    char default_datastr_cld_aer[] = "abdQ";
    char default_datastr_fpoint[] = "abdyz";
    char default_datastr_limb[] = "abd";
    char default_datastr_limb_fpoint[] = "abdyz";
    char *datastring = NULL;

    char default_profile_name_ext[] = "";
    char *profile_name_ext = default_profile_name_ext;
			/* store which molecule is requested */
				/*  */
    product_id prod = {SCI_NL__2P_MAX_DS_NAME,
		       SCI_OL__2P_MAX_DS_NAME,
		       SCI_RV__2P_MAX_DS_NAME,
		       SCI_OL__2P_MAX_DS_NAME,
		       MIPAS_EMPTY,
		       GOME_L2_MAX};

    FILE *out = stdout;
    int header_flag = HEAD;
    int n_products = 0;		/* Number of files on commandline */
    int info_flag = DATAMODE;	/* INFOMODE or DATAMODE */

    user_data ud;		/* will hold the output user data */
    user_data ud_nearest;	/* temporary user_data for the search option */
    Coord_deg fpoint;			/* coordinates for fpoint mode */
    int fpoint_flag = FPOINT_UNSET; /* FPOINT_SINGLE, FPOINT_UNSET,
				       FPOINT_ALL */
    int fpoint_all_flag = FPOINT_UNSET; /* FPOINT_UNSET,
				       FPOINT_ALL */
    int fpoint_success_flag = FPOINT_UNSET; /* FPOINT_MATCH, FPOINT_UNSET */
    double rmax = RMAX_DEFAULT;
    double dist_minimum = DIST_MIN_DEFAULT;
    double dist = DIST_DEFAULT;
    char defaultcomment[] = ""; /* empty string is the default comment */
    char *comment = defaultcomment; /* point to comment string -c */

    char *ftime = NULL;		/* pointer to datestring for time restriction mode */
    MJD fmjd = {0,0U,0U};	/* MJD of date  for time restriction mode */
    double dtime = DTIME_DEFAULT; /* matching time range */
    int ftime_flag = FTIME_UNSET;
    
    double szamax = SZAMAX_DEFAULT;
    
    int ssc_flag = SSC_CLEAR;
    int ssc_number = 0;
				/*  lat,lon,gp restrictions*/
    float lat_bound[2] = {-90.0, 90.0};
    float lon_bound[2] = {-360.0, 360.0};
    int gp_bound[2] = {0, 9999};
                                /* Area for in area check */
    Coord_deg area_corners[4];
    int flag_area_check=NOAREA;

				/*  LIMB mode (PROFILE) variables */
    int limb_flag = NADIR;     	/* Default is NADIR mode  */
    int mipas_flag = MIPASPROFILE; /* default for mipas is profile output */
    int n_lmm = 0;
    int limb_mode_many [LMM_MAX];
    char default_limb_dir[] = "./";
    char *limb_dir = default_limb_dir;
    int limb_orbit_flag = LIMB_FILE_NO_ORBIT;   
				/*  Search list */
    int fspoint_flag = FPOINT_UNSET; /* FPOINT_SINGLE, FPOINT_UNSET,
				       FPOINT_ALL FPOINT_AREA FPOINT_INSIDE*/
				/*  for searching with lists*/
    int n_search = 0;
    int n_area_search = 0;
    search_list *search_list;

    int files_ordered_flag = FILES_UNORDERED;
    int n_search_start = 0;	/* used in ordered search */
				
    FILE *in_files = NULL;	/*  List with input files */
    SCIA_err err;		/* error value */

				/*  Flag for verbosity */
    int verbose_flag = NONVERBOSE;
    
    while ((c = getopt (argc, argv, optstring)) != - 1)
    {
	switch (c)
	{
				/*  Help request */
	    case 'h':
				/*  product IDs */
		if (strncmp (optarg, "prod", 3) == 0)
		{
		    prod_id_print (stdout);
		    exit (0);
		}
				/*  Date specifiers */
		if (strncmp (optarg, "dat", 3) == 0)
		{
		    help_data_spec(stdout);
		    exit (0);
		}
				/*  Limb            */
		if (strncmp (optarg, "limb", 3) == 0)
		{
		    help_limb(stdout, default_datastr_limb,
			      default_datastr_limb_fpoint);
		    exit (0);
		}
				/*  Date specifiers */
		if (strncmp (optarg, "mipas", 5) == 0)
		{
		    help_mipas(stdout);
		    exit (0);
		}
				/*  All */
		if (strncmp (optarg, "all", 3) == 0)
		{
		    help_general (stdout, default_datastr_normal,
			      default_datastr_fpoint);
		    prod_id_print (stdout);
		    help_data_spec(stdout);
		    help_limb(stdout, default_datastr_limb,
			      default_datastr_limb_fpoint);
		    help_mipas(stdout);
		    exit (0);
		}
				/*  default is the general help page */
		help_general (stdout, default_datastr_normal,
			  default_datastr_fpoint);
		exit(0);

			
	    case 'p':
		prod = prod_id_str (optarg);
		if (prod.prod_n == SCI_NL__2P_MAX_DS_NAME &&
		    prod.prod_o == SCI_OL__2P_MAX_DS_NAME &&
		    prod.prod_m == SCI_RV__2P_MAX_DS_NAME &&
		    prod.prod_l == SCI_OL__2P_MAX_DS_NAME &&
		    prod.prod_mip == MIPAS_EMPTY &&
		    prod.prod_g == GOME_L2_MAX )
		{
				/*  Invalid product specified */
				/*  give list of products */
		    prod_id_print (stderr);
		    exit (1);
		}
		break;
	    case 'd':
		datastring = optarg;
		;
		break;
	    case 'o':
		if (strcmp ("-",optarg) == 0)
		    break;
		out = fopen(optarg, "w");
		if (out == 0)
		{
		    fprintf (stderr,"Cannot open Output file: %s.\n", optarg);
		    exit (1);
		}
		break;
	    case 'n':
		header_flag = NOHEAD;
		break;
	    case 'i':
		info_flag = INFOMODE;
		break;
	    case 'I':
		info_flag = OL_INITIAL_MODE;
		break;
	    case 's':
		if ( sscanf (optarg , "%lf", &szamax ) != 1)
		{
		    fprintf (stderr,"Wrong argument for option -s %s.\n", optarg);
		    exit (2);
		}
		break;
	    case 'j':
		if ( sscanf (optarg , "%f:%f", lat_bound, lat_bound+1) != 2 ||
		    lat_bound[0] < -90.0 || lat_bound[1] > 90.0 ||
		    lat_bound[0] >= lat_bound[1] )
		{
		    fprintf (stderr,"Wrong argument for option -j : %s,\n"
			     "use lat1:lat2 with -90 <= lat1 < lat2 <= 90\n"
			     " Example: -60:30.5\n", optarg);
		    exit (2);
		}
		break;
	    case 'k':
		if ( sscanf (optarg , "%f:%f", lon_bound, lon_bound+1) != 2 ||
		    lon_bound[0] < -180.0 || lon_bound[1] > 180.0 ||
		    lon_bound[0] >= lon_bound[1] )
		{
		    fprintf (stderr,"Wrong argument for option -j : %s,\n"
			     "use lon1:lon2 with -180 <= lon1 < lon2 <= 180\n"
			     " Example: -60:30.5\n", optarg);
		    exit (2);
		}
		break;
		
	    case 'x':
		if ( sscanf (optarg , "%d:%d", gp_bound, gp_bound+1) != 2 ||
		    gp_bound[0] < 0 || gp_bound[0] > gp_bound[1] )
		{
		    fprintf (stderr,"Wrong argument for option -e : %s,\n"
			     "use gp1:gp2 with 0 <= gp1 <= gp2\n"
			     " Example: 100:250\n", optarg);
		    exit (2);
		}
		break;
		    
	    case 'f':
		if ( sscanf (optarg , "%f,%f", &fpoint.lat, &fpoint.lon ) != 2)
		{
		    fprintf (stderr,"Wrong argument for option -f %s, \n"
			     "use lat,lon. Example: 9.5,-110.0\n", optarg);
		    exit (2);
		}
		fpoint_flag = FPOINT_SINGLE;
		dist = 0.0;
		break;

	    case 'R':
		if ( sscanf (optarg , "%f,%f,%f,%f,%f,%f,%f,%f", 
			     &(area_corners[0].lat), &(area_corners[0].lon),
			     &(area_corners[1].lat), &(area_corners[1].lon),
			     &(area_corners[2].lat), &(area_corners[2].lon),
			     &(area_corners[3].lat), &(area_corners[3].lon)) 
		     != 8)
		{
		    fprintf (stderr,"Wrong argument for option -R %s,\n "
			     "use lat,lon,lat,lon,lat,lon,lat,lon\n",
			     optarg);
		    exit (2);
		}
		flag_area_check = AREA;
		break;

		
	    case 'a':
		fpoint_all_flag = FPOINT_ALL;
		break;
		
	    case 'F':
	    {
		FILE* slist;		
		if (! (slist = fopen (optarg, "r")))
		{
		    fprintf (stderr,"Cannot open search list file: -F %s.\n",
			     optarg);
		    exit (1);
		}
		if (! (read_search_list (slist,
					&n_search,
					&search_list,
					 NOAREA)))
		{
		    fprintf (stderr,"Error during reading search list file:"
			     " -F %s.\n",
			     optarg);
		    exit (1);
		}
		fclose (slist);
		fspoint_flag = FPOINT_SINGLE;
	    }
	    break;

	    case 'Q':
	    {
		FILE* slist;		
		if (! (slist = fopen (optarg, "r")))
		{
		    fprintf (stderr,"Cannot open search list file: -Q %s.\n",
			     optarg);
		    exit (1);
		}
		if (! (read_search_list (slist,
					&n_search,
					&search_list,
					 AREA )))
		{
		    fprintf (stderr,"Error during reading search list file:"
			     " -Q %s.\n",
			     optarg);
		    exit (1);
		}
		fclose (slist);
		fspoint_flag = FPOINT_AREA;
	    }
	    break;

	    case 'O':
		files_ordered_flag = FILES_ORDERED;
		break;

	    case 'c':
		comment = optarg;
		break;
	    case 'r':
		if ( (sscanf (optarg , "%lf", &rmax ) != 1) || rmax < 0.0)
		{
		    fprintf (stderr,"Wrong argument for option -r %s.\n", optarg);
		    exit (2);
		}
		break;
				/*  Time restrict option*/
	    case 't':
		ftime = optarg;
		if (! ASCII_2_MJD (ftime, &fmjd))
		{
		    fprintf (stderr,"Not a date for option -t\n %s\n",
			     optarg);
		    exit (2);
		}
				/*  check given time */
		{
		    char testdate[28];
		    UTC_String(&fmjd, testdate);
		    fprintf (stderr,"ftime: %s\n", testdate);
		}
		    
		ftime_flag = FTIME_SET;
		break;

	    case 'u':
		if ( sscanf (optarg, "%lf", &dtime) != 1 || dtime <= 0.0)
		{
		    fprintf (stderr, "Not a number for option -u : %s\n",
			     optarg);
		    exit (2);
		}
		break;
		
	    case 'b':
		switch (optarg[0])
		{
		    case 'f':
			ssc_flag = SSC_FOR;
			break;
		    case 'F':
			ssc_flag = SSC_FORLONG;
			break;
		    case 'b':
			ssc_flag = SSC_BACK;
			break;
		    case 'B':
			ssc_flag = SSC_BACKLONG;
			break;
		    case '0':
		    case '1':
		    case '2':
		    case '3':
		    case '4':
		    case '5':
		    case '6':
		    case '7':
		    case '8':
		    case '9':
		    case '-':
			ssc_flag = SSC_NUMBER;
			if (sscanf (optarg,"%d",&ssc_number) == 1)
			    break;
		    default:
			fprintf (stderr,"Wrong argument for option -b %s, "
				 "Values are f,F,b or a number (0-80 or -2).\n",
				 optarg);
			exit (2);
			break;
		}
		break;
	    case 'l':
		in_files = fopen (optarg, "r");
		if (in_files == 0)
		{
		    fprintf (stderr,"Cannot open list file: -i %s.\n", optarg);
		    exit (1);
		}
		break;
				/*  Select Limb mode */
	    case 'L':
		if (strcmp (optarg, "PROFILE") == 0)
		{
		    limb_flag = LIMBPROFILE;
		    break;
		}
		if (strcmp (optarg, "TABLE") == 0)
		{
		    limb_flag = LIMBTABLE;
		    break;
		}
		fprintf (stderr, "Illegal limb mode -L %s.\n"
			 "Use -L PROFILE  of -L TABLE.\n", optarg);
		exit(1);
		break;
				/*  Collect output files generated in
				    LIMB many */
	    case 'M':
	    {
		int n,i;
		for (n=0; n<LMM_MAX; n++)
		{
		    if (strcmp (optarg, lmm[n]) == 0)
		    {
				/*  check for already given */
			for (i=0; i<n_lmm; i++)
			{
			    if (limb_mode_many[i] == n)
				break;
			}
				/*  No? than add it */
			if ( i == n_lmm )
			    limb_mode_many [n_lmm++] = n;
			break;
		    }
		}
		if ( n == LMM_MAX )
		{
		    fprintf (stderr,
			     "Unknown output file type requested -M %s\nUse: ",
			     optarg);
		    for (n=0; n<LMM_MAX; n++)
			fprintf (stderr, " %s", lmm[n]);
		    fprintf (stderr, "\n");
		    exit (1);
		}
	    }
	    break;
				/*  Add orbit number to limb file names */
	    case 'N':
		limb_orbit_flag = LIMB_FILE_WITH_ORBIT;
		break;
				/*  Directory for LIMB Mode Many */
				/*  and for OL_INITIAL_MODE */
	    case 'D':
		limb_dir = optarg;
		break;

	    case 'v':
		verbose_flag = VERBOSE;
		break;

	    case 'Y':
		mipas_flag = MIPASTABLE;
		break;
							   		
	    default:
		fprintf (stderr, "Unknown option -%c\n", c);
		exit(1);
		break;
	}
    }
				/*  Set here, what is not set by options */
    if (fpoint_flag == FPOINT_SINGLE && fpoint_all_flag == FPOINT_ALL
	&& rmax > 0.0 )
	fpoint_flag = FPOINT_ALL;
    if (fspoint_flag == FPOINT_SINGLE && rmax==0.0 )
	fspoint_flag = FPOINT_INSIDE;
    if (fspoint_flag == FPOINT_SINGLE && fpoint_all_flag == FPOINT_ALL
	&& rmax > 0.0 )
	fspoint_flag = FPOINT_ALL;
        
    if (datastring == NULL)
    {
	switch (limb_flag)
	{
	    case NADIR:
		if (fpoint_flag == FPOINT_UNSET)
		{
		    if (prod.prod_n == SCI_NL__2P_CLOUDS_AEROSOL ||
			prod.prod_n == SCI_OL__2P_CLOUDS_AEROSOL )
			datastring = default_datastr_cld_aer;
		    else
			datastring = default_datastr_normal;
		}
		else
		    datastring = default_datastr_fpoint;
		break;
			
	    case LIMBTABLE:
		if (fpoint_flag == FPOINT_UNSET)
		    datastring = default_datastr_limb;
		else
		    datastring = default_datastr_limb_fpoint;
		break;
	    default:
	    case LIMBPROFILE:
		break;
	}
    }
    if (prod.prod_n == SCI_NL__2P_MAX_DS_NAME &&
	prod.prod_o == SCI_OL__2P_MAX_DS_NAME &&
	prod.prod_m == SCI_RV__2P_MAX_DS_NAME &&
	prod.prod_l == SCI_OL__2P_MAX_DS_NAME &&
	prod.prod_mip == MIPAS_EMPTY &&
	prod.prod_g == GOME_L2_MAX)
    {				/* O3 is default product */
	prod = prod_id_str (Generic_Prod[GEN_O3]);
    }
    if (limb_flag == LIMBPROFILE)
    {
	if (n_lmm == 0 )
	    limb_mode_many[n_lmm++] =  LMM_MAIN;
    }

    
				/*  If no products given */
				/*  spent a help message and exit */
    if (optind == argc && in_files==NULL)
    {
	help_general (stdout,
		      default_datastr_normal, default_datastr_fpoint);
	exit (1);
    }

    
    nline = 0;
    n_products = argc - optind;
    
			/* Loop over all input files */
    while (1)
    {
	info_l2N info;      /* this will contain all internal 
			   information like data offsets etc. */	
	char in_file[2001];
	char *cur_file;
					/*  specify requested output type */
	info.limb_flag = limb_flag;


	if (in_files)		/* Read filename from file */
	{
	    if ( fscanf (in_files,"%2000s", in_file) == EOF )
		break;
	    cur_file = in_file;
	}
	else			/* take filename from commandline */
	{
	    if (optind == argc)
		break;
	    cur_file = argv[optind++];
	}
	if (verbose_flag == VERBOSE)
	    fprintf (stderr, "-- %s\n", cur_file);
				/*  open file here */
	err = openL2N (cur_file, &info);
	if (err != OK)
	    {
		switch (err)
		{
		    case FILE_NOT_FOUND:
			fprintf (stderr, "Error: File not found!\n  %s\n",cur_file);
			break;
		    case MPH_ERROR:
			fprintf (stderr, "Error: Not a SCIAMACHY level 2 product!\n"
				 "  %s\n",cur_file);
			break;
		    case SPH_ERROR:
		    case DSD_ERROR:
			fprintf (stderr, "Fatal error, Corrupted product: "
				 "SPH or DSDs bad!\n  %s\n",cur_file);
			break;			
			/*  User error */
		    case SCIA_MDS_EMPTY:
			fprintf (stderr, "Warning: No data for requested species "
				 "in this product file.\n  %s\n",cur_file);
			break;
		    case SCIA_BAD_SPECIFICATION:
			fprintf (stderr, "Error: Requested species not available"
				 " in this product type.\n  %s\n",cur_file);
			break;
		    case SCIA_GEO_MISMATCH:
			fprintf (stderr, "Fatal error, Corrupted product: "
				 "Geolocation mismatch in this"
				 " product file.\n  %s\n",cur_file);
			break;
		    case SCIA_GOME_EXTRA_PRODUCT:
			fprintf (stderr, "Fatal error: Products not implemented here\n"
				 "found in GOME product!\n"
				 "Update  list in gome_L2.c/gome_L2.h :\n"
                                 " GOME_L2_MOL_NAME / GOME_L2_MOL_WIN / GOME_L2_MAX.\n");
			break;

			    ;
		    default:
			fprintf (stderr, "Unspecified proplem in this product file.\n"
				 "  %s\n",cur_file);
			break;
		}
		exit (2);
	    }
				/*  Info mode requested ?*/
	if (info_flag == INFOMODE)
	{
	    L2N_info (out, &info);
	    closeL2N (&info);
	    continue;		/* Start with next file */
	}

	if (info_flag == OL_INITIAL_MODE)
	{
	    err = write_OL_initialisation_file (limb_dir, &info);
	    closeL2N (&info);
	    continue;		/* Start with next file */
	}

	if (info.scia_product == MIPAS_L2)
	{
	    limb_flag = mipas_flag;
	    if (flag_area_check == AREA || 
		fspoint_flag == FPOINT_AREA)
	    {
		fprintf (stderr, "For MIPAS products, area search is not available (-R or -Q)!\n");
		exit(3);
	    }
	}
	    

	/* Usually you will access the data loop */
    IGNORE_ERROR: 		/* Enter again loop, ignore non-fatal error */
	while ((err = L2N_next_data (&info, prod, &ud)) == OK)
	{
	    /* read center for distance */	    
	    Coord_deg center;
	    Coord_deg *corners;
	    double earth_rad;
	    double sza;
	    MJD *mjd;
	    switch (limb_flag)
	    {
		case LIMBTABLE:
		case LIMBPROFILE:
		    center = ud.geo_L.tangent_coord[1];
		    earth_rad = ud.geo_L.earth_rad;
		    sza = ud.geo_L.sol_zen_angle_toa[1];
		    mjd = &ud.geo_L.dsr_time;
		    corners = ud.corner_limb;
		    break;
		case NADIR:
		    center = ud.geo.cen_coor_nad;
		    earth_rad = ud.geo.earth_rad;
				/* Meteo products have no earth radius */
		    if (earth_rad == 0.0)
			earth_rad = 6386.0;
		    sza = ud.geo.sol_zen_angle_toa[1];
		    mjd = &ud.geo.dsr_time;
		    corners = ud.geo.cor_coor_nad;
		    break;
		case MIPASPROFILE:
		case MIPASTABLE:
		    center = ud.geo_mipas.loc_mid;
		    earth_rad = 6386.0;
		    sza = 0.0; 	/* not available */
		    mjd = &ud.geo_mipas.dsr_time;		    
		    corners = ud.corner_limb;
		    break;
	    }
	    
		
	    /* Write always header for tablemode NADIR/LIMBTABLE */
	    /* check SSC */
	    switch (limb_flag)
	    {
		/* ******************************************************** */
		/*  Here Nadir output starts                                */
		case NADIR:
		case LIMBTABLE:
		case MIPASTABLE:
		    /* Write header, if not disabled */
		    if (nline++ == 0 && header_flag == HEAD)
		    {
			if (n_products == 1) /* only one File given */
			{
			    fprintf (out, "%s\n", ud.product);
			}
			else
			{
			    fprintf (out, "Multiple product files\n");
			}
			fprintf (out, "Prod: %s   ", ud.molecule);
			fprintf (out, "Datastring: '%s'  \n", datastring);
			switch (fpoint_flag)
			{
			    case FPOINT_SINGLE:
				fprintf (out, "Search:  %9.3f,%9.3f   "
					 "Radius: %7.2f  Nearest Match\n",
					 fpoint.lat, fpoint.lon, rmax);
				break;
			    case FPOINT_ALL:
				fprintf (out, "Search:  %9.3f,%9.3f   "
					 "Radius: %7.2f  Match All\n",
					 fpoint.lat, fpoint.lon, rmax);
				break;
			    default:
				fprintf (out, "-\n");
				break;
			}
			fprintf (out, "\n");
			outline (out, datastring, &ud, HEADER, dist, comment);
			outline (out, datastring, &ud, UNIT, dist, comment);
		    }
		    break;
		case LIMBPROFILE:
		case MIPASPROFILE:
		    break;
	    }
				/* check common restrictions */
				/* lat restrictions */
	    if (center.lat < lat_bound[0] ||
		center.lat > lat_bound[1])
		continue;
				/* lon restrictions */
	    if (center.lon < lon_bound[0] ||
		center.lon > lon_bound[1])
		continue;

	    /* SZA */
	    if (sza > szamax)
		continue;

	    /* GOME : gp restrictions */
	    if (( ud.method == GOME || ud.method == GOME_4 )&&
		(ud.geo.gp < gp_bound[0] ||
		 ud.geo.gp > gp_bound[1]))
		continue;
	    
	    /* Time restrictione */
	    if (ftime_flag == FTIME_SET )
	    {
		if (fabs (MJD_diff (&fmjd, mjd))
		    > dtime * SECOND_PER_HOUR)
		    continue;
	    }
	    
	    /* Scan Type */
	    if (ssc_flag != SSC_CLEAR && limb_flag == NADIR)
	    {
		if ( ud.method == GOME || ud.method == GOME_4 )
		{	/*  GOME SubSetcounter */
		    switch (ssc_flag)
		    {
			case SSC_FOR:
			case SSC_FORLONG:
			    if (ud.geo.subset_counter == 3)
				continue;
			    break;
			case SSC_BACK:
			case SSC_BACKLONG:
			    if (ud.geo.subset_counter < 3)
				continue;
			    break;
			case SSC_NUMBER:
			    if (ud.geo.subset_counter != ssc_number)
				continue;
			    break;
		    }
		}
		else
		{				    
		    switch (ssc_flag)
		    {
			case SSC_FOR:
			    if (ud.geo.subset_counter < 0)
				continue;
			case SSC_FORLONG:
			    if (ud.geo.subset_counter > 63)
				continue;
			    break;
			case SSC_BACK:
			    if (ud.geo.subset_counter < 64)
				continue;
			    break;
			case SSC_BACKLONG:
			    if (ud.geo.subset_counter < 64 &&
				ud.geo.subset_counter >= 0)
				continue;
			    break;
			case SSC_NUMBER:
			    if (ud.geo.subset_counter != ssc_number)
				continue;
			    break;
		    }
		}
	    }

	    /* Area restriction */
	    if (flag_area_check == AREA)
	    {
		if (! point_in_gp (area_corners, center))
		    continue;
	    }


	    /* search restriction from command line */
	    if ( fpoint_flag != FPOINT_UNSET)
	    {
				/*  usual search from center */
		if (rmax > 0.0)
		{
		    dist =
			earth_rad * circle_distance (center, fpoint);
		    
		    if ( fpoint_flag == FPOINT_ALL )
		    {
			if ( dist > rmax)
			    continue;
		    }
		    if ( fpoint_flag == FPOINT_SINGLE )
		    {
			if (dist < rmax && dist < dist_minimum)
			{
			    fpoint_success_flag = FPOINT_MATCH;
			    dist_minimum = dist;
			    ud_nearest = ud;
			}
			continue;
		    }
		}
				/*  rmax=0 : search for inside
				 groundpixel, always SINGLE */
		else
		{
		    if (point_in_gp (corners, fpoint))
		    {
				/*  previous one already found */
			if (fpoint_success_flag == FPOINT_MATCH)
			{
				/*  check if previous one is forward scan */
				/*  if it is, dont change */
			    if (ud_nearest.method == GOME)
			    {
				if (ud_nearest.geo.subset_counter < 3)
				    continue;
			    }
			    else
			    {
				if (ud_nearest.geo.subset_counter < 64)
				    continue;
			    }
			}
				/*  store the found one (new or overwriting
				    previous backscan */
			ud_nearest = ud;
			dist_minimum = 0.0;
			fpoint_success_flag = FPOINT_MATCH;
		    }
		    continue;
		}
			
	    }
	    
	    /* search list (from file)*/
	    if ( fspoint_flag !=  FPOINT_UNSET)
	    {
		int n;
		double dist;
				/*  go through all pixels to find */
		for (n=n_search_start; n < n_search; n++)
		{
		    double tdiff = MJD_diff (mjd, &search_list[n].mjd);

		    if (files_ordered_flag == FILES_ORDERED)
		    {
			/* this point is more than dtime before actual
			   groundpixel, no need to check this again because
			   of chronological files */
			if (tdiff < -dtime * SECOND_PER_HOUR)
			{
			    n_search_start = n+1;
			    /* A nearer poin cannot be found 
                                (all further points are later)
			       therefore we already write the point
			       and free the memory */
			    if (search_list[n].flag == FPOINT_MATCH)
			    {
				/* Now write it, if find for FPOINTALL */
				/*  Here, a groundpixel might be written */
				/*  more than once, if there */
				switch (limb_flag)
				{
				    case LIMBTABLE:
				    case NADIR:
				    case MIPASTABLE:
					outline (out, datastring,
						 search_list[n].ud, DATA,
						 search_list[n].dist_minimum,
						 search_list[n].comment);
					break;
					
				    case LIMBPROFILE:
					write_limb_many (limb_dir,
							 search_list[n].ud,
							 limb_mode_many,
							 n_lmm, search_list[n].comment,
							 limb_orbit_flag);
					break;
 				    case MIPASPROFILE:
//void outprofile (user_data *ud, double dist, char *comment, char *path,
					//	 double lat, double lon)
					outprofile (search_list[n].ud,
						    search_list[n].dist_minimum,
						    search_list[n].comment,
						    limb_dir, 
						    search_list[n].pos.lat, 
						    search_list[n].pos.lon);
					break;

				}
				/*  free memory from user_data */
				free (search_list[n].ud);
				search_list[n].ud = NULL;
				search_list[n].flag = FPOINT_UNSET;
			    }
			    continue;
			}

			/* this point is more than dtime after actual
			   groundpixel, no need to go further trough list */ 
			if (tdiff > dtime * SECOND_PER_HOUR)
			    break;
		    }
		    else
		    {
			if (tdiff < -dtime * SECOND_PER_HOUR)
			    continue;
				/*  simple check time */
			if (tdiff > dtime * SECOND_PER_HOUR)
			    break;
		    }

		    
				/*  check inside !!*/
		    switch (fspoint_flag)
		    {
			case FPOINT_INSIDE:

			    if (point_in_gp (corners, search_list[n].pos))
			    {
				/*  previous one already found */
				if (search_list[n].flag == FPOINT_MATCH)
				{
				    /*  check if previous one is forward scan */
				    /*  if it is, dont change */
				    if (search_list[n].ud->method == GOME ||
					search_list[n].ud->method == GOME_4 )
				    {
					if (search_list[n].ud->geo.subset_counter < 3)
					    continue;
				    }
				    else
				    {
					if (search_list[n].ud->geo.subset_counter < 64)
					    continue;
				    }
				}
				/*  store the found one (new or overwriting
				    previous backscan */
				if (search_list[n].ud == NULL)
				{
				    search_list[n].ud = malloc (sizeof(user_data));
				    if (!search_list[n].ud)
				    {
					fprintf(stderr, "Out of memory allocating"
						" search_list user_data\n"
						"Think about sorting input files\n"
						"and using -O\n");
					exit (255);
				    }
				}
				*(search_list[n].ud) =  ud;
				search_list[n].dist_minimum = 0.0;
				search_list[n].flag = FPOINT_MATCH;
			    }
			    continue;
			    break;
			
			case FPOINT_ALL:
			case FPOINT_SINGLE:
			    dist = earth_rad * circle_distance 
				(center, search_list[n].pos);
			    if ( dist > rmax )
				continue;
			    /*  only nearest point wanted, store in list */
			    if ( fspoint_flag == FPOINT_SINGLE )
			    {			
				if (dist < search_list[n].dist_minimum)
				{
				    search_list[n].dist_minimum = dist;
				    search_list[n].flag = FPOINT_MATCH;
				    /* allocate memory for user data */
				    if (search_list[n].ud == NULL)
				    {
					search_list[n].ud = malloc (sizeof(user_data));
					if (!search_list[n].ud)
					{
					    fprintf(stderr, "Out of memory allocating"
						    " search_list user_data\n"
						    "Think about sorting input files\n"
						    "and using -O\n");
					    exit (255);
					}
				    }
				    *(search_list[n].ud) =  ud;
				}
				/*  For single point, print later */
				continue;
			    }
			    break;

			case FPOINT_AREA:
			    if (! point_in_gp (
				    search_list[n].area_pos, center))
				continue;
			    break;
		    }

		    /* Now write it, if find for FPOINTALL */
		    /*  Here, a groundpixel might be written */
		    /*  more than once, if there */
			
		    switch (limb_flag)
		    {
			case LIMBTABLE:
			case NADIR:
			case MIPASTABLE:
			    outline (out, datastring, &ud, DATA, dist,
				     search_list[n].comment);
			    break;
			    
			case LIMBPROFILE:
			    write_limb_many (limb_dir, &ud, limb_mode_many, n_lmm, profile_name_ext, limb_orbit_flag);
			    break;
   			case MIPASPROFILE:
//void outprofile (user_data *ud, double dist, char *comment, char *path,
					//	 double lat, double lon)
			    outprofile (&ud,
					dist,
					search_list[n].comment,
					limb_dir, 
					search_list[n].pos.lat, 
					search_list[n].pos.lon);
			    break;			    
		    }
		}
				/*  avoid normal output, it is done in
				 the search point loop */
		continue;
	    }
		


		 
	    /* Now write it!! */
	    switch (limb_flag)
	    {
		case LIMBTABLE:
		case NADIR:
		case MIPASTABLE:
		    outline (out, datastring, &ud, DATA,dist, comment);
		    break;

		case LIMBPROFILE:
		    write_limb_many (limb_dir, &ud, limb_mode_many, n_lmm, comment, limb_orbit_flag);
		    break;
		case MIPASPROFILE:
//void outprofile (user_data *ud, double dist, char *comment, char *path,
					//	 double lat, double lon)
		    outprofile (&ud,
				dist,
				comment,
				limb_dir, 
				0.0, 0.0);
		    break;			    
	    }
	} /* End of loop over pixels/profiles */

	switch (err)
	{
	    case SCIA_END_OF_MDS: /* All MDS of file are processed */
		break;
	    case SCIA_LIMB_MDS_EMPTY:
				/*  Ignore empty MDS, hopefully this occurs
				    only in the test file available for
				    development .... */
		fprintf (stderr,
			 "Warning : An empty MDS found, ignored.\n");
		goto IGNORE_ERROR;
		
				/*  User error */
	    case SCIA_MDS_EMPTY:
		fprintf (stderr, "Warning: No data for requested species "
			 "in this product file.\n  %s\n",cur_file);
		break;
	    case SCIA_BAD_SPECIFICATION:
		fprintf (stderr, "Error: Requested species not available"
			 " in this product type.\n  %s\n",cur_file);
		break;
	    case SCIA_GEO_MISMATCH:
		fprintf (stderr, "Fatal error, Corrupted product: "
			 "Geolocation mismatch in this"
			 " product file.\n  %s\n",cur_file);
		break;
	    case SCIA_PRODUCTNAME_MISMATCH:
		fprintf (stderr, "Fatal error, molecule name from DSD and "
			 "fitwindow string do not match in this"
			 " product file.\n  %s\n",cur_file);
		break;
	    default:
		fprintf (stderr, "Unspecified problem in this product file.\n"
			   "  %s\n",cur_file);
		break;
	}
	closeL2N (&info);
    }
				/* if search for nearest point is succesfull, print it  */
    if ( fpoint_success_flag == FPOINT_MATCH )
    {
	switch (limb_flag)
	{
	    /* ******************************************************** */
	    /*  Here Nadir output starts                                */
	    case NADIR:
	    case LIMBTABLE:
	    case MIPASTABLE:
		outline (out, datastring, &ud_nearest, DATA, dist_minimum, comment);

		break;
	    case LIMBPROFILE:
		write_limb_many (limb_dir, &ud_nearest, limb_mode_many, n_lmm, comment, limb_orbit_flag);
		break;
	    case MIPASPROFILE:
//void outprofile (user_data *ud, double dist, char *comment, char *path,
		//	 double lat, double lon)
		outprofile (&ud_nearest,
			    dist_minimum,
			    comment,
			    limb_dir, 
			    fpoint.lat, fpoint.lon);
		break;			    

	}
    }
    
				/*  */
    if ( fspoint_flag ==  FPOINT_SINGLE)
    {
	int n;
				/*  go through all pixels to find */
	for (n=0; n < n_search; n++)
	{
	    if (search_list[n].flag == FPOINT_MATCH)
	    {
				/* Now write it, if find for FPOINTALL */
				/*  Here, a groundpixel might be written */
				/*  more than once, if there */
		switch (limb_flag)
		{
		    case LIMBTABLE:
		    case NADIR:
		    case MIPASTABLE:
			outline (out, datastring, search_list[n].ud, DATA,
				 search_list[n].dist_minimum,
				 search_list[n].comment);
			break;
			    
		    case LIMBPROFILE:
			write_limb_many (limb_dir, search_list[n].ud,
					 limb_mode_many, n_lmm,
					 search_list[n].comment, limb_orbit_flag);
			break;
		    case MIPASPROFILE:
//void outprofile (user_data *ud, double dist, char *comment, char *path,
					//	 double lat, double lon)
			outprofile (search_list[n].ud,
				    search_list[n].dist_minimum,
				    search_list[n].comment,
				    limb_dir, 
				    search_list[n].pos.lat,
				    search_list[n].pos.lon);
			break;			    

		}
				/*  free memory from user_data */
		free (search_list[n].ud);
	    }
	}
    }
		
    if ( fspoint_flag !=  FPOINT_UNSET)
	free (search_list);
		
    fclose (out);
    return 0;
}




/* *********************************************************************** */
/* Help text functions                                                     */

void help_general (FILE* out,
		   const char* default_datastr_normal,
		   const char* default_datastr_fpoint)
{
    fprintf
	(out,
"%s , Ver. %s\n"
"======================================================\n"
"%s is a tool for reading SCIAMACHY level-2 products (PDS Format)\n"
" to an easy to use ASCII output. Addtionally, GOME level-2 products and\n"
" parly MIPAS NRT level-2 are supported.\n"
" Readable are:\n"
"  - Near-Real-Time (NRT)     : SCI_NL__2P....N1\n"
"  - Offline Products (OL)    : SCI_OL__2P....N1  (SCIA-OL/2.1 or higher)\n"
"  - Products for Meteo-Users : SCI_RV__2P....N1\n"
"  - GOME GDP 2.n/3.n         : ymmddhhm.lv2\n"
"  - GOME GDP 4+              : yyyymmddhhmm_nnnnn.lv2\n"
"  - MIPAS NRT                : MIP_NL__2P....N1  (only SCAN INFRORMATION part)\n"
"The programm includes a geopgraphical search modus.\n"
"\n"
"Source code is available at \n"
"http://www.iup.physik.uni-bremen.de/gcvos/tools .\n"
"Send bug reports to\n"
" %s .\n"
"\n"
"Usage:\n"
"------\n"
"-- Help:\n"
"    %s -h [prod, dat, limb, intro, mipas, all]\n"
"    \n"
"-- Dataextraction: \n"
"    %s [-p prod] [-d datastring] [-o output] SCI_??_..1.n1 [...]\n"
"    \n"
"-- File content overview:\n"
"   %s -i SCI_??__2P..1.n1 [SCI_??_..2.n1 ...]\n"
"   \n"
"-- Search modus:\n"
"   %s -f lat,lon [-r rmax] [-a] [-c comment] [-p prod] [-d datastring] [-o output] SCI_??_..1.n1 [...]\n"
"   %s -F searchlist [-r rmax] [-a] [-p prod] [-d datastring] [-o output] SCI_??_..1.n1 [...]\n"
"   \n"
"-- Read Limb profile (OL product)\n"
"   %s -L TABLE [-p prod] [-d datastring] [-o output] SCI_OL_..1.n1 [...]\n"
"   SCIAL2_ascii -L PROFILE [ -D dir ] [ -M what ] SCI_OL_..1.n1 [...]\n"
"   \n"
"-- Initialisation file OL Product:\n"
"   %s -I [-D path] SCI_??_..1.n1 [...]\n"
"\n"
"Options:\n"
"--------   \n"
"-- Help:\n"
"   -h what   : Request help pages. Available are:\n"
"                - prod: List of available product-IDs (for -p)\n"
"                - dat:  List of letters in  datastring (for -d)\n"
"                - limb: Descripe limb mode\n"
"                - mipas: Descripa MIPAS specific part\n"
"                - intro: This page\n"
"                - all : print intro, prod, dat, limb, mipas\n"
"\n"
"-- Data specification:\n"
"   -p prod   : Specify product to extract, default: 'O3'. \n"
"               Use '-h prod' for a complete list.\n"
"   -d datastring : Specify the information you want in the output.\n"
"                  Default: %s. (If -f applied, default: %s.) \n"
"                  Use '-h dat' for a complete list.\n"
"\n"
"-- Output restrictions:\n"
"   -s sza        : Restrict used points to SZA less then 'sza'.\n"
"   -b type       : Restrict output by scan type (Nadir only):\n"
"                   - f: Forward scans only\n"
"                   - F: Forward scans and long Integration time scans\n"
"                   - b: Backward scans only\n"
"                   - B: Backward scans and long Integration time scans\n"
"                   GOME:\n"
"                   - f,F: Forward scans only\n"
"                   - b,B: Backward scans and long Integration time scans\n"
"   -j lat1:lat2  : Restrict to pixels with center latitude between lat1:lat2\n"
"   -k lon1:lon2  : Restrict to pixels with center longitude between lon1:lon2\n"
"   -x gp1:gp2    : GOME: Restrict to groundpixels from gp1 to gp2                \n"
"                 \n"
"-- Search Options:\n"
"   -f lat,lon    : Print out only the nearest measurement to the location\n"
"                    specified by lat,lon (see also -r -c -a -t).\n"
"   -r rmax       : Search only in a circle of radius 'rmax' km.\n"
"                    With '0', match measurement with point inside groundpixel.\n"
"                    Default: %.0fkm.\n"
"   -a            : Print out *all*, not only the nearest point inside\n"
"                    radius 'rmax'.\n"
"   -c Comment    : Arbitrary string, can be used in  output line with 'y'.\n"
"                    (Usefull for identifying your search point later on.)\n"
"   -t time       : Exclude all pixel more than 12 hours earlier or later than\n"
"                   'time'. \n"
"                   Time format: 12-APR-2001 21:55:00.000000\n"
"   -u dtime      : instead of 12 hours, use dtime hours for -t\n"
"\n"
"   -F searchlist : Search for all points given in File 'searchlist'. Format of\n"
"                   'searchlist' one line per search point:\n"
"                   lat lon  DD-MMM-YYYY HH:MM:SS.mmmmmm  Comment\n"
"                   Search the nearest measurement at lat/lon at given date\n"
"                   +- 12 hours. Comment is an arbitrary string, which can be\n"
"                   accessed with 'y' for output.\n"
"                   See also -r, -a, -u.\n"
"   -O            : The products files are assumed to be given in chronological\n"
"                   order. This is usefull to save memory, if a large\n"
"                   searchlist (>1000 entries) is given. It also speeds up\n"
"                   searching.\n"
"   -R lat,lon,.. : Give a rectangle lat,lon,lat,lon,lat,lon,lat,lon and get\n"
"                   all groundpixels with center_points inside this area.\n"
"                   Area should not be to large and not near the pole. The\n"
"                   calculations are not spherical.\n" 
"   -Q searchlist : As -F, but for area search. Format of a line in 'searchlist':\n"
"                   lat lon lat lon lat lon lat lon  DD-MMM-YYYY HH:MM:SS.mmmmmm Comment\n"
"                                   \n"
"   !! Do not mix -f and -F in one call. !!                                  \n"
"   \n"
"-- Output Options:\n"
"   -o file       : Write output to 'file' instead of stdout\n"
"   -n            : suppress header lines in output\n"
"\n"
"-- Limb Options:\n"
"   -L [TABLE, PROFILE] : Select limb profiles to be read.\n"
"              - TABLE   : One row for each profile information set.\n"
"              - PROFILE : Create independent files for each profile.\n"
"   -D dir       : Output directory for files created with '-L PROFILE'.\n"
"   -M what      : Specify the files created with '-L PROFILE' for each\n"
"                  profile.\n"
"   Use '-h limb' for detailed information about limb extraction.           \n"
"   \n"
"-- MIPAS Options:\n"
"   -D dir       : Output directory for MIPAS profile files.\n"
"   -Y           : MIPAS as table output with basic geolocation, usefull for\n"
"                  search modus.\n"
"-- Miscellaneous:\n"
"   -l file       : Read the SCIAMACHY Lv2 files listed in this file.\n"
"                   Format: One file per line\n"
"                   Files given on commandline are ignored with this option.\n"
"   -I            : Write the initialisation file for an Offline product.\n"
"   -v            : Verbose: Write some processing information to stderr\n"
"                      (processed file).\n"
	 "   \n",
	 PROG_NAME, PROGRAM_VERSION, PROG_NAME,
	 PACKAGE_BUGREPORT, PROG_NAME,PROG_NAME,
	 PROG_NAME, PROG_NAME, PROG_NAME, PROG_NAME, PROG_NAME,
	 default_datastr_normal, default_datastr_fpoint,
	 RMAX_DEFAULT);
}



void help_data_spec(FILE *out)
{
    fprintf
	(out,
"\n"
"Datastring specifiers:\n"
"======================\n"
"\n"
"The datastring, given with option '-d' specifies the columns you want in\n"
"your output, used for extraction of nadir data, with -L TABLE for limb, and\n"
"with -Y for MIPAS. Each letter in\n"
"the datastring defines one or more columns of output. The available letters\n"
"are described in the following table. Not all information is available for all\n"
"products, therefore in square brackets '[......]' the allowed products are\n"
"coded:\n"
" N/D/B  : NRT-product / NRT, only DOAS (UV-vis) / NRT, only BIAS (IR).\n"
"   O  : OL-product nadir.\n"
"   M  : Meteo-product.\n"
"   L  : OL-product limb (TABLE mode).\n"
"   G  : GOME product.\n"
"   Y  : MIPAS product.\n"
"For detailed information, have a look at the IODDs and the ATBDs of the\n"
"products.\n"
"\n"
"List of specifiers:\n"
"-------------------\n"
"\n"
"-------- Geolocation\n"
" a [NOMLGY] : Orbit number\n"
" b [NOMLGY] : time and date (ASCII)\n"
" c [NOMLGY] : frac. days since 1.1.2000\n"
" d [NOMLGY] : center coordinates of GP (lat, lon)\n"
" e [NOMLG.] : 4 Corners of Groundpixel, lat lon (1,2,3,4)\n"
" f [NO.LG.] : 3 solar zenith angles at TOA \n"
" g [NO.LG.] : 3 line-of-sight angles at TOA \n"
" h [NO.LG.] : 3 rel. azimuth  angles at TOA \n"
" i [NO.LG.] : Satellite heigt\n"
" j [NO.LG.] : Earth radius\n"
" p [.O.L..] : Subsatellite point (OL only)\n"
" k [NOMLG.] : SubsetCounter (not part of SCIA products)\n"
"             Nadir  0 -63 : Forward Scan\n"
"                    64-79 : Backward Scan\n"
"                    -2    : 10s long scan\n"
"             Limb : 0-23  : Horizontal start point\n"
"             GOME : 0,1,2 : Forward scan\n"
"                    3     : Backward scan or long scan\n"
" l [NOML..] : Integration time [s]\n"
" m [NOML..] : Integration time [1/16s]\n"
" q [....G.] : Groundpixel number (GOME)\n"
" r [....G.] : 3 solar zen., 3 line-of-sight, 3 rel. azimuth angles at satellite\n"
" \n"
" In Limb, the geolocation for a profile is adapted from a height of\n"
" about 20km (scan 28). The corner coordinates are the tangentpoints\n"
" from first (scan 1) and last scan before dark measurement (scan 34/ scan 30).\n"
"\n"
"--------------- Miscellaneous\n"
" n [NOMLGY] : Product identification record\n"
" o [NOMLGY] : Processing sofware version\n"
" y [NOMLGY] : User comment : text from commandline option -c\n"
" z [NOMLGY] : Distance of point specified with -f to center of groundpixel.\n"
"\n"
"-------- Vertical Column Densities\n"
" A [NOM.G.] : total VCD [molecules/cm2]\n"
" B [NOM.G.] : errors on total VCD to ground\n"
" C [NOM.G.] : total VCD [DU]\n"
" \n"
" In OL-nadir, additional species may be fitted (for IR products).\n"
" Use 1,2,3,.. to get access to them.\n"
"\n"
" 1 [.O....] : total VCD [molecules/cm2] and error on total VCD of\n"
"            1st fitted molecule\n"
" 2,3,4,..  : of 2nd, 3rd, 4th, .... fitted molecule \n"
"\n"
"-------- Limb profile output\n"
" L [...L..] : Limb profile of main species as VCD\n"
"              N  N*[alt. p T VCD  VCDerr] \n"
" P [...L..] : Limb profile of main species as VMR\n"
"              N  N*[alt. p T VCD  VCDerr]\n"
"              \n"
" L or P should always be the last specifier in datestring to ensure more\n"
" or less consistent output, because the number of retrieved layers\n"
" might change from profile to profile. Usually, the recommended access to\n"
" profile information is the PROFILE mode.\n"
"\n"
"\n"
" Please look at the IODD and ATBD for SCIAMACHY Level 1b to 2\n"
" Processing for more information of the following parameters.\n"
" All following values are not available for Meteo-products.\n"
"\n"
"--------- Slant Columns Densities (OL and NRT-DOAS only)\n"
" D [DO..G.] : Slant column density and Error on slant column density\n"
"             DOAS-NRT, OL and GOME\n"
"--------- Flag and Quality VCD\n"
" E [NO..G.] : DOAS-NRT : Flag describing the VCD output\n"
"             BIAS-NRT : Flag describing the BIAS VCD fitting\n"
"             OL : Flag describing the VCD output\n"
" F [NO..G.] : Fit quality parameters: - RMS\n"
"                                    - chi-squared\n"
"                                    - goodness of fit\n"
"                                    - number of iterations\n"
"--------- AirMassFactors\n"
" G [D...G.] : AMF (DOAS-NRT/GOME) : - AMF to ground for specie \n"
"                                   - AMF to cloud-top for specie\n"
"                                   - Flag describing the above AMF\n"
" H [.O..G.] : AMF (OL/GOME(GDP4+)) :- AMF to ground + Error\n"
"                                   - AMF to cloud-top + Error\n"
"                                   - AMF output flag\n"
"--------- Other values\n"
" I [D.....] : All other values NRT-DOAS:\n"
"               - Number of fitting parameters, n_fit\n"
"               - Cross-correlation parameters, (n_fit * (n_fit-1))/2\n"
"               - Flag describing the slant column\n"
"               - Reflectance from ground, cloud_top, measured\n"
" J [B.....] : All other values NRT-BIAS:\n"
"               - Number of geophysical parameters fitted  n_fit\n"
"               - Cross correlation parameters (n_fit * (n_fit-1))/2\n"
"               - Number of segments in fitting window \n"
"               - Height value and error \n"
"               - Flag describing height variable usage \n"
"               - Fitted first-closure parameter + error \n"
"               - transmittance cut-off for the fitting window \n"
" K [.O....] : All other values Nadir-OL:\n"
"               - Number of linear fitted parameters (n1)\n"
"               - Number of nonlinear fitted parameters (n2)\n"
"               - Linear fitted parameters n1 + err n1\n"
"               - Cross-correlation entries of the linear fit.\n"
"               - Nonlinear fitted parameters  n1 + err n1\n"
"               - Cross-correlation entries of the nonlinear fit.\n"
"               - Fitting output flag.\n"
"               - Temperature of reference spectrum\n"
" M [....G.] : All other values GOME:\n"
"             GDP 4+:\n"
"               - DOAS: Flag indexing output\n"
"               - Ozone temperature, Ozone Ring corr. fac., Ghost VC\n"
"               - Surface height, pressure, albedo\n"
"             GDP previous:\n"
"               - DOAS: Flag indexing output\n"
"               - AMF: Total intensities for ground for Nwin fitting windows\n"
"               - AMF: Total intensities for cloud top for Nwin fitting windows\n"
"               - AMF: Measured Intensity for Nwin windows\n"
"               - Statistics: Pixel contrast (3 PMDs: mean/ standard deviation)\n"
"               - Statistics: Pixel colour (sub pixel values)\n"
"               - Statistics: Pixel colour gradient (Ch. 3)\n"
"               - Statistics: Flag indexing output\n"
"               \n"
"-------- Cloud and Aerosol information (only available for CLOUDS_AEROSOL)\n"
" Q [NO..G.] : Fractional cloud cover + Quality(NRT) / Err(OL/GOME) of cld cover\n"
" N [NO....] : Absorbing aerosol indicator\n"
" O [NO..G.] : Cloud-top pressure (OL/GOME: + err cld-top pressure)\n"
" R [NO..G.] : NRT:  Flag describing the Cloud and Aerosol output\n"
"             OL :  Flag descr. cloud parameter output\n"
"             GOME: ICFA: Flag indexing output\n"
"NRT only: \n"
" S [N.....] : Number of PMD sub-pixels + cld fraction of PMD sub-pixel\n"
" T [N.....] : Equivalent Lambertian albedo\n"
"OL only: \n"
" V [.O....] : Number of PMD sub-pixels: all/ totally clouded / cloud-free\n"
" W [.O....] : Cloud optical depth + error opt. depth + Flag descr. cld type\n"
" X [.O..G.] : Reflectance: cld-top , err cld-top,  surface, err surface\n"
"             GOME GDP 4+: Alb. cld-top + err, surface albedo\n" 
" Y [.O....] : Aerosol indicator diagn. + Flag descr. abs. aerosol indicator\n"
" Z [.O....] : Number add. aerosol param. + add. aerosol param.\n"
"GOME only:\n"
" U [....G.] : Only GDP<4.0 : Oth. ICFA val.: Surface pressure,  CCA cld. frac,\n"
"             CCA Subpix trace\n"
"---------------------------------------------------------------\n"
" Warning: The information in the various flags is not interpretated by\n"
"          this programm, you have to check it by yourself!!\n"
"---------------------------------------------------------------\n"
"\n"
	    );
	 }


void help_limb (FILE* out,
		const char* default_datastr_limb,
		const char* default_datastr_limb_fpoint )
{
    fprintf (out,
"\n"
"Limb extraction:\n"
"================\n"
"\n"
"The SCIAMACHY OL products (SCI_OL__2P) usually contains Limb profiles.\n"
"Use the option -L [TABLE, PROFILE] to switch to Limb extraction mode.\n"
"\n"
"-- Usage:\n"
"   -----\n"
"   -L [TABLE, PROFILE]\n"
"    TABLE:   All entries for one profile are in one row of a tabular\n"
"             outputfile. Use option -d to specify details. This mode\n"
"	      corresponds to the nadir extraction method. It is\n"
"	      preferred for geolocation of profiles only, although profile\n"
"	      output is possible ('L' or 'P' in datastring). Especially in\n"
"	      conjuntion with the search options this mode is usefull.\n"
"	      The default datastring (use '-h dat' for details)\n"
"	      for this mode is '%s', in search mode it is '%s' \n"
"	      \n"
"    PROFILE: Create separate files for each profile.\n"
"             Use option -M to specify details.\n"
"\n"
"-- Options in mode 'PROFILE':\n"
"   ----------------------\n"
"   -D dir  : Directory where to create output files.\n"
"            Default is current directory.\n"
"   -N      : Add orbit number to filenames of the individual profiles.\n"
"	   \n"
"   -M [%s, %s, %s, %s, %s, %s, %s, %s, %s,\n"
"       %s, %s, %s, %s ]\n"
"       Specify the output files to be generated.\n"
"       Use repeated options to request more than one.\n"
"       'main' is the default value, if no -M is given.\n"
"       \n"
"     %8.8s  : FIRST main species, including  height, T, p, VMR,\n"
"                 VMR err, VCD, VCD err.\n"
"     %8.8s  : FIRST main species in number density, as given in\n"
"              add. diagnostics (3.0 or later). Here the \"representative height\"\n"
"              for the layer is printed.\n"
"     %8.8s  : ALL main species, including  height, T, p, VMR,\n"
"                 VMR err, VCD, VCD err.  (With one main species, same as 'main')\n"
"     %8.8s  : ALL main species in number density, as given in\n"
"              add. diagnostics (3.0 or later). (With one main species,\n"
"              same as 'num_dens')"
"     %8.8s  : Conversion factors for version > 3.0\n"
"              NumDens., Initial NumDens, Conversion factor VMR, Cf ND\n"
"     %8.8s  : Scaled profiles which have been fitted on the\n"
"                 coarse forward model grid, including height column.\n"
"     %8.8s  : Measurement Grid of used measurements.\n"
"                 Time, height, p, T, Number of fitting windows,\n"
"                 Minimum and Maximum wavelength over all fitting windows.\n"
"     %8.8s  : State vector.\n"
"                 Value, error, type of the state vector entry.\n"
"     %8.8s  : Correlation Matrix of the fit.\n"
"     %8.8s  : Residuals for the iterations.\n"
"     %8.8s  : Additional diagnostics.\n"
"     %8.8s  : Various values describing the fit.\n"
"                 RMS, Chi^2, Goodness, ....\n"
"     %8.8s  : Detailed geolocation, ....\n"
"\n"
,
	     default_datastr_limb, default_datastr_limb_fpoint,
	     lmm[0],lmm[1],lmm[2],lmm[3],lmm[4],lmm[5],lmm[6],
	     lmm[7],lmm[8],lmm[9],lmm[10],lmm[11],lmm[12],
	     lmm[0],lmm[1],lmm[2],lmm[3],lmm[4],lmm[5],lmm[6],
	     lmm[7],lmm[8],lmm[9],lmm[10],lmm[11],lmm[12]);
}


void help_mipas (FILE* out)
{
    fprintf (out,
"\n"
"MIPAS extraction:\n"
"================\n"
"\n"
"The MIPAS NRT level-2 products contains profiles for H2O,O3,HNO3,CH4,N2O,NO2.\n"
"The main information about the profiles is stored in SCAN INFORMATION MDS of\n"
"the product, which can be extracted by this tool.\n"
"As for other products, the output molecule can be selected with -p, possible\n"
"values are H2O,O3,HNO3,CH4,N2O and NO2, default is O3.\n"
"Switching to MIPAS mode is done automatically, when a MIPAS product is given.\n"
"By default, each profile is written to one file:\n"
" Example: mipas_05081_19-Feb-2003_00:49:47.698482_O3.prf\n"
"                orbit date/time                   species\n"
"With -D dir, these files are stored to directory 'dir'.\n"
"\n"
"A tabular output of the basic geolocation usefull in search mode is available\n"
"with the option -Y. Possible letters in datastring are 'abcdnoyz'.\n"
"\n"
"Search and restriction options work as for SCIAMACHY and GOME, with the\n"\
"exceptions:\n"
" - Area search (-R,-Q) are not possible with MIPAS,\n"
" - SZA restriction is not possiple.\n"
"The necessary information is not in the SCAN INFORMATION MDS. \n"
"\n");

}


