#ifndef  __SCIA_L2                       /* Avoid multiple includes */
#define  __SCIA_L2

#include "scia_defs.h"
#include "scia_common.h"
#include "bin_read.h"

/* Defintion of Lv2 SPH */

typedef struct SPH_SCI_NL__2P {
        /*  SPH descriptor */
    char sph_descriptor[28];
        /*  Value: +000 = No stripline continuity, the product is a complete
            segment Other: Stripline Counter */
    char stripline_continuity_indicator[5];
        /*  Value: +001 to NUM_SLICES Default value if no stripline continuity
            = +001 */
    char slice_position[5];
        /*  Number of slices in this stripline Default value if no continuity
            = +001 */
    char num_slices[5];
        /*  Start time of the measurement data in this product. UTC time of
            first MDSR */
    char start_time[28];
        /*  Time of the end of the measurement data in this product (last MDSR
            time). UTC time of last MDSR */
    char stop_time[28];
        /*  Latitude of the satellite nadir at the START_TIME. WGS84 latitude,
            positive = North */
    char start_lat[12];
        /*  Longitude of the satellite nadir at the START_TIME. WGS84 longitude,
            positive = East, 0 = Greenwich */
    char start_long[12];
        /*  Latitude of the satellite nadir at the STOP_TIME. WGS84 latitude,
            positive = North */
    char stop_lat[12];
        /*  Longitude of the satellite nadir at the STOP_TIME. WGS84 longitude,
            positive = East, 0 = Greenwich */
    char stop_long[12];
        /*  Spare */
    char spare_1[50];
        /*  Quality summary of the fitting errors: GOOD or FAIR or BAD&Oslash; */
    char fitting_error_sum[4];
        /*  Number of DOAS fitting windows */
    char no_of_doas_fitting_windows[5];
        /*  Wavelength range of fitting window given as a pattern of two wavelengths
            like: xxxx-yyyy e.g., 0325-0335 */
    char doas_fitting_window[7][9];
        /*  Number of BIAS fitting windows */
    char no_of_bias_fitting_windows[5];
        /*  Wave length range of fitting window and microwindows. xxxx-yyyy
            NNNN xxxx-yyyy xxxx-yyyy xxxx-yyyy */
    char bias_fitting_window[4][44];
        /*  Number of molecules to be retrieved with DOAS */
    char no_of_doas_molecules[5];
        /*  Fitting window ID and acronym of the molecule to be retrieved
            given as the following pattern: W-xxxxxx, e.g., 1-NO2&Oslash;&Oslash;&Oslash;,
            note unused characters set to the ascii blank space character
            (&Oslash;). Number of the fitting window corresponds to the
            list above. */
    char doas_molecule[22][8];
        /*  Number of molecules to be retrieved with BIAS */
    char no_of_bias_molecules[5];
        /*  Fitting window ID and acronym of the molecule to be retrieved
            given as the following pattern: W-xxxxxx, e.g., 1-NO2&Oslash;&Oslash;&Oslash;,
            note unused characters set to the ascii blank space character
            (&Oslash;). Number of the fitting window corresponds to the
            list above. */
    char bias_molecule[10][8];
} SPH_SCI_NL__2P ;



/* Array with DSD Names : declaration, definition in scia_L2.c*/

/* DSD name enum */
    extern char DS_NAME_SCI_NL__2P [50][29];

typedef enum DSD_SCI_NL__2P
{
    SCI_NL__2P_SUMMARY_QUALITY,
    SCI_NL__2P_STATE_GEOLOCATION,
    SCI_NL__2P_STATES,
    SCI_NL__2P_GEOLOCATION,
    SCI_NL__2P_CLOUDS_AEROSOL,
    SCI_NL__2P_DOAS_0_O3,
    SCI_NL__2P_DOAS_1_NO2,
    SCI_NL__2P_DOAS_1_H2O,
    SCI_NL__2P_DOAS_1_O3,
    SCI_NL__2P_DOAS_2_BRO,
    SCI_NL__2P_DOAS_2_O3_L,
    SCI_NL__2P_DOAS_2_O3_H,
    SCI_NL__2P_DOAS_2_NO2,
    SCI_NL__2P_DOAS_2_OCLO,
    SCI_NL__2P_DOAS_3_OCLO,
    SCI_NL__2P_DOAS_3_NO2,
    SCI_NL__2P_DOAS_3_O4,
    SCI_NL__2P_DOAS_4_SO2,
    SCI_NL__2P_DOAS_4_O3,
    SCI_NL__2P_DOAS_5_HCHO,
    SCI_NL__2P_DOAS_5_BRO,
    SCI_NL__2P_DOAS_5_O3_L,
    SCI_NL__2P_DOAS_5_O3_H,
    SCI_NL__2P_DOAS_5_NO2,
    SCI_NL__2P_DOAS_5_O4,
    SCI_NL__2P_DOAS_SPARE_1,
    SCI_NL__2P_DOAS_SPARE_2,
    SCI_NL__2P_BIAS_0_H2O,
    SCI_NL__2P_BIAS_0_CO2,
    SCI_NL__2P_BIAS_1_N2O,
    SCI_NL__2P_BIAS_1_H2O,
    SCI_NL__2P_BIAS_1_CH4,
    SCI_NL__2P_BIAS_2_CO,
    SCI_NL__2P_BIAS_2_H2O,
    SCI_NL__2P_BIAS_2_CH4,
    SCI_NL__2P_BIAS_SPARE_1,
    SCI_NL__2P_BIAS_SPARE_2,
    SCI_NL__2P_LEVEL_1B_PRODUCT,
    SCI_NL__2P_INITIALISATION_FILE,
    SCI_NL__2P_CLIMA_DBPR_FILE,
    SCI_NL__2P_CLIMA_DBCL_FILE,
    SCI_NL__2P_CLIMA_DBSF_FILE,
    SCI_NL__2P_CLIMA_DBCS_FILE,
    SCI_NL__2P_CLIMA_DBLB_FILE,
    SCI_NL__2P_CLIMA_DBFM_FILE,
    SCI_NL__2P_CLIMA_DBUC_FILE,
    SCI_NL__2P_CLIMA_DBCC_FILE,
    SCI_NL__2P_AMF_LOOK_UP_FILE,
    SCI_NL__2P_BSPF_LOOK_UP_FILE,
    SCI_NL__2P_AIRC_LOOK_UP_FILE,
    SCI_NL__2P_MAX_DS_NAME
} DSD_SCI_NL__2P;



typedef struct DOAS_L2N {
        /*  Start Time of DSR */
    MJD dsr_time;
        /*  Length of this DSR in bytes */
    unsigned int dsr_length;
        /*  Quality Indicator (-1 for blank MDSR, 0 otherwise) */
    char quality_flag;
        /*  Integration time for this DSR */
    unsigned short integr_time;
        /*  Number of fitting parameters, n fit */
    unsigned short num_fit_para;
        /*  Vertical column density (VCD) */
    float vcd;
        /*  Error on the VCD above */
    float vcd_err;
        /*  Flag describing the VCD output above */
    unsigned short flag_vcd_flags;
        /*  Slant column density */
    float slant_col_den;
        /*  Error on slant column density */
    float err_slant_col;
        /*  RMS, chi-2, goodness of fit for the fitting window */
    float rms_chi_2_gof[3];
        /*  Iteration number for the fitting window */
    unsigned short iter_num_fit_win;
        /*  Cross-correlation parameters */
    float cross_corr_para[DOAS_BIAS_MAX_CROSS_COR];
        /*  Flag describing the above slant column */
    unsigned short flag_slant_col_flags;
        /*  AMF to ground for specie */
    float amf_gr;
        /*  AMF to cloud-top for specie */
    float amf_cl;
        /*  Reflectance from ground */
    float refl_ground;
        /*  Reflectance from cloud top */
    float refl_cloud_top;
        /*  Measured reflectance */
    float measured_refl;
        /*  Flag describing the above AMF */
    unsigned short flag_amf_flags;
} DOAS_L2N ;


typedef struct BIAS_L2N {
        /*  Start Time of DSR */
    MJD dsr_time;
        /*  Length of this DSR in bytes */
    unsigned int dsr_length;
        /*  Quality Indicator (-1 for blank MDSR, 0 otherwise) */
    char quality_flag;
        /*  Integration time for this DSR */
    unsigned short integr_time;
        /*  Number of geophysical parameters fitted */
    unsigned short num_geophy_param_fit;
        /*  Number of segments in fitting window */
    unsigned short num_seg_fit_win;
        /*  Height value and error */
    float ht_val_and_err[2];
        /*  Flag describing height variable usage */
    unsigned short flag_ht_var_usg;
        /*  Trace gas vertical column density */
    float trc_gas_vcd;
        /*  Error on vertical column density */
    float err_vcd;
        /*  Fitted first-closure parameter */
    float fit_fc_param;
        /*  Error on fitted first-closure parameter */
    float err_fit_fc_param;
        /*  RMS, chi-squared, goodness of fit, transmittance cut-off for
            the fitting window */
    float rms_good_fit_win[4];
        /*  Iteration number for the fitting window */
    unsigned short iter_num_fit_win;
        /*  Cross correlation parameters */
    float cross_cor_param[DOAS_BIAS_MAX_CROSS_COR];
        /*  Flag describing the BIAS VCD fitting */
    unsigned short flag_bias_vcd_fit_flags;
} BIAS_L2N ;


/* Nadir geolocation structure for Level 2 NRT products */

typedef struct GeoN_L2N {
        /*  Start Time of DSR */
    MJD dsr_time;
        /*  Attachment Flag */
    char attach_flag;
        /*  Integration time for this geolocation entity */
    unsigned short integr_time;
        /*  Solar zenith angles of the start, middle and end of the integration
            time at TOA */
    float sol_zen_angle_toa[3];
        /*  Line-of-sight zenith angles of start, middle and end of the integration
            time at TOA */
    float los_zen_angle_toa[3];
        /*  Relative azimuth angles of start, middle and end of the integration
            time at TOA */
    float rel_azi_angle_toa[3];
        /*  Satellite geodetic height at the middle of the integration time */
    float sat_geod_ht;
        /*  Earth radius at the middle of the integration time */
    float earth_rad;
        /*  4 corner coordinates of the nadir ground pixel (the first coordinate
            is the one which is the first in time and flight direction, the
            second the first in time and last in flight direction, the third
            the last in time and first in flight direction and the fo */
    Coord cor_coor_nad[4];
        /*  Centre coordinate of the nadir ground pixel */
    Coord cen_coor_nad;
} GeoN_L2N ;


typedef struct CLOUDS_AEROSOL_L2N {
        /*  Start Time of DSR */
    MJD dsr_time;
        /*  Length of this DSR in bytes */
    unsigned int dsr_length;
        /*  Quality Indicator (-1 for blank MDSR, 0 otherwise) */
    char quality_flag;
        /*  Integration time for this DSR */
    unsigned short integr_time;
        /*  Number of PMD sub-pixels for this DSR */
    unsigned short num_pmd;
        /*  Cloud fraction of PMD sub-pixel */
    float cl_frac[CLOUDS_AEROSOL_L2N_MAX_PMD];
        /*  Average cloud fraction of footprint */
    float ave_clo_fract_foot;
        /*  Quality of fractional cloud cover */
    unsigned char qual_frac_cloud_cov;
        /*  Cloud-top pressure */
    float cl_top_pres;
        /*  Absorbing aerosol indicator */
    float aero_abso_ind;
        /*  Equivalent Lambertain albedo */
    float equiv_lam_alb;
        /*  Flag describing the output above */
    unsigned short flag_output_flags;
} CLOUDS_AEROSOL_L2N ;



void DOAS_L2N_getbin (FILE* unit, DOAS_L2N *var);
void DOAS_L2N_array_getbin (FILE* unit, DOAS_L2N *var, int nr);

void BIAS_L2N_getbin (FILE* unit, BIAS_L2N *var);
void BIAS_L2N_array_getbin (FILE* unit, BIAS_L2N *var, int nr);

void GeoN_L2N_getbin (FILE* unit, GeoN_L2N *var);
void GeoN_L2N_array_getbin (FILE* unit, GeoN_L2N *var, int nr);

void CLOUDS_AEROSOL_L2N_getbin (FILE* unit, CLOUDS_AEROSOL_L2N *var);
void CLOUDS_AEROSOL_L2N_array_getbin (FILE* unit, CLOUDS_AEROSOL_L2N *var,
				      int nr);



SCIA_err Read_SPH_SCI_NL__2P (FILE* unit, SPH_SCI_NL__2P *sph);


#endif /* __SCIA_L2 */

