
#include "scia_defs.h"
#include "bin_read.h"
#include "scia_common.h"

/* Identifier for the various SCIA-Products, corresponds to enum SCIA_PRODUCT*/

char SCIA_PRODUCT_ACR [SCIA_MAX_PRODUCT][12] =
{"SCI_NL__2P",
 "SCI_OL__2P",			/* to be checked!!!! */
 "SCI_RV__2P",
 "SCI_NL__1P",
 "SCI_NLC_1C",
 "MIP_NL__2P" };


/**********************************************************************
 * getbin routines for composed types, calling the routines for
 * simple types in  'bin_read_basic'
 *
 * Here are the types common to all(?) Scia Products ...
 **********************************************************************/


void MJD_getbin (FILE* unit, MJD *var)
{
  int_getbin(unit, &var->days);
  uint_getbin(unit, &var->secnd);
  uint_getbin(unit, &var->musec);
}


void MJD_array_getbin (FILE* unit, MJD *var, int nr)
{
  int n;
  for (n=0; n<nr; n++)
    {
      int_getbin(unit, &var[n].days);
      uint_getbin(unit, &var[n].secnd);
      uint_getbin(unit, &var[n].musec);
    }
}

void GeoN_getbin (FILE* unit, GeoN *var)
{
  float_getbin (unit, &var->esm_pos);
  float_array_getbin (unit, var->sza_toa, 3);
  float_array_getbin (unit, var->saa_toa, 3);
  float_array_getbin (unit, var->los_zen, 3);
  float_array_getbin (unit, var->los_azi, 3);
  float_getbin (unit, &var->sat_height);
  float_getbin (unit, &var->earth_radius);
  Coord_getbin (unit, &var->sub_sat);
  Coord_array_getbin (unit, var->corner_coord, 4);
  Coord_getbin (unit, &var->centre_coord);
}
    
void GeoN_array_getbin (FILE* unit, GeoN *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
      float_getbin (unit, &var[n].esm_pos);
      float_array_getbin (unit, var[n].sza_toa, 3);
      float_array_getbin (unit, var[n].saa_toa, 3);
      float_array_getbin (unit, var[n].los_zen, 3);
      float_array_getbin (unit, var[n].los_azi, 3);
      float_getbin (unit, &var[n].sat_height);
      float_getbin (unit, &var[n].earth_radius);
      Coord_getbin (unit, &var[n].sub_sat);
      Coord_array_getbin (unit, var[n].corner_coord, 4);
      Coord_getbin (unit, &var[n].centre_coord);
    }
}


void GeoL_getbin (FILE* unit, GeoL *var)
{
    float_getbin (unit, &var->esm_pos);
    float_getbin (unit, &var->asm_pos);
    float_array_getbin (unit, var->sza_toa, 3);
    float_array_getbin (unit, var->saa_toa, 3);
    float_array_getbin (unit, var->los_zen, 3);
    float_array_getbin (unit, var->los_azi, 3);
    float_getbin (unit, &var->sat_height);
    float_getbin (unit, &var->earth_radius);
    Coord_getbin (unit, &var->sub_sat);
    Coord_array_getbin (unit, var->tangent_ground_point, 3);
    float_array_getbin (unit, var->tangent_height, 3);
    float_getbin (unit, &var->doppler_shift);
}
    
void GeoL_array_getbin (FILE* unit, GeoL *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
	float_getbin (unit, &var[n].esm_pos);
	float_getbin (unit, &var[n].asm_pos);
	float_array_getbin (unit, var[n].sza_toa, 3);
	float_array_getbin (unit, var[n].saa_toa, 3);
	float_array_getbin (unit, var[n].los_zen, 3);
	float_array_getbin (unit, var[n].los_azi, 3);
	float_getbin (unit, &var[n].sat_height);
	float_getbin (unit, &var[n].earth_radius);
	Coord_getbin (unit, &var[n].sub_sat);
	Coord_array_getbin (unit, var[n].tangent_ground_point, 3);
	float_array_getbin (unit, var[n].tangent_height, 3);
	float_getbin (unit, &var[n].doppler_shift);
    }
}

void GeoCal_getbin (FILE* unit, GeoCal *var)
{
    float_getbin (unit, &var->esm_pos);
    float_getbin (unit, &var->asm_pos);
    float_getbin (unit, &var->sza);
    Coord_getbin (unit, &var->sub_sat);
}

void GeoCal_array_getbin (FILE* unit, GeoCal *var, int nr)
{
    int n;
    for (n=0; n<nr; n++)
    {
	float_getbin (unit, &var[n].esm_pos);
	float_getbin (unit, &var[n].asm_pos);
	float_getbin (unit, &var[n].sza);
	Coord_getbin (unit, &var[n].sub_sat);
    }
}
/* Coordinates stored and read as int */

void Coord_getbin (FILE* unit, Coord *var)
{
  int_getbin (unit, &var->lat);
  int_getbin (unit, &var->lon);
}


void Coord_array_getbin (FILE* unit, Coord *var, int nr)
{
  int n;
  for (n=0; n<nr; n++)
    {
      int_getbin (unit, &var[n].lat);
      int_getbin (unit, &var[n].lon);
    }
}

/* Coordinates stored and read as float (deg) */
void Coordf_getbin (FILE* unit, Coord_deg *var)
{
  float_getbin (unit, &var->lat);
  float_getbin (unit, &var->lon);
}


void Coordf_array_getbin (FILE* unit, Coord_deg *var, int nr)
{
  int n;
  for (n=0; n<nr; n++)
    {
      float_getbin (unit, &var[n].lat);
      float_getbin (unit, &var[n].lon);
    }
}

/* Coordinates stored as int and read as float (deg) */

void Coord_deg_getbin (FILE* unit, Coord_deg *var)
{
    int lat,lon;
    int_getbin (unit, &lat);
    int_getbin (unit, &lon);
    var->lat = lat * ANGLE_FAC;
    var->lon = lon * ANGLE_FAC;
}


void Coord_deg_array_getbin (FILE* unit, Coord_deg *var, int nr)
{
  int n;
  int lat,lon;
  for (n=0; n<nr; n++)
    {
      int_getbin (unit, &lat);
      int_getbin (unit, &lon);
      var[n].lat = lat * ANGLE_FAC;
      var[n].lon = lon * ANGLE_FAC;
    }
}


/* Reading the main product header MPH */
/* This is ASCII Format, therefore not bin... but fscanf is used */

SCIA_err Read_MPH (FILE* unit, MPH *mph)

{
    unsigned n;
    int err;
    char nl[2];
    char *tmp = (char*) mph;
    /* Zeros in the complete structure, so all strings have trailing \0 */
    for (n=0; n<sizeof(MPH); n++)
	tmp[n] = 0;
    
    err = fscanf(unit,
		 "PRODUCT=\"%62c\"%1c"
		 "PROC_STAGE=%1c%1c"
		 "REF_DOC=\"%23c\"%1c"
		 "%40c%1c"
		 "ACQUISITION_STATION=\"%20c\"%1c"
		 "PROC_CENTER=\"%6c\"%1c"
		 "PROC_TIME=\"%27c\"%1c"
		 "SOFTWARE_VER=\"%14c\"%1c"
		 "%40c%1c"
		 "SENSING_START=\"%27c\"%1c"
		 "SENSING_STOP=\"%27c\"%1c"
		 "%40c%1c"
		 "PHASE=%1c%1c"
		 "CYCLE=%4c%1c"
		 "REL_ORBIT=%6c%1c"
		 "ABS_ORBIT=%6c%1c"
		 "STATE_VECTOR_TIME=\"%27c\"%1c"
		 "DELTA_UT1=%8c<s>%1c"
		 "X_POSITION=%12c<m>%1c"
		 "Y_POSITION=%12c<m>%1c"
		 "Z_POSITION=%12c<m>%1c"
		 "X_VELOCITY=%12c<m/s>%1c"
		 "Y_VELOCITY=%12c<m/s>%1c"
		 "Z_VELOCITY=%12c<m/s>%1c"
		 "VECTOR_SOURCE=\"%2c\"%1c"
		 "%40c%1c"
		 "UTC_SBT_TIME=\"%27c\"%1c"
		 "SAT_BINARY_TIME=%11c%1c"
		 "CLOCK_STEP=%11c<ps>%1c"
		 "%32c%1c"
		 "LEAP_UTC=\"%27c\"%1c"
		 "LEAP_SIGN=%4c%1c"
		 "LEAP_ERR=%1c%1c"
		 "%40c%1c"
		 "PRODUCT_ERR=%1c%1c"
		 "TOT_SIZE=%21c<bytes>%1c"
		 "SPH_SIZE=%11c<bytes>%1c"
		 "NUM_DSD=%11c%1c"
		 "DSD_SIZE=%11c<bytes>%1c"
		 "NUM_DATA_SETS=%11c%1c"
		 "%40c%1c"
		 ,
		 mph->product, nl,
		 mph->proc_stage, nl,
		 mph->ref_doc, nl,
		 mph->spare_1, nl,
		 mph->acquisition_station, nl,
		 mph->proc_center, nl,
		 mph->proc_time, nl,
		 mph->software_ver, nl,
		 mph->spare_2, nl,
		 mph->sensing_start, nl,
		 mph->sensing_stop, nl,
		 mph->spare_3, nl,
		 mph->phase, nl,
		 mph->cycle, nl,
		 mph->rel_orbit, nl,
		 mph->abs_orbit, nl,
		 mph->state_vector_time, nl,
		 mph->delta_ut1, nl,
		 mph->x_position, nl,
		 mph->y_position, nl,
		 mph->z_position, nl,
		 mph->x_velocity, nl,
		 mph->y_velocity, nl,
		 mph->z_velocity, nl,
		 mph->vector_source, nl,
		 mph->spare_4, nl,
		 mph->utc_sbt_time, nl,
		 mph->sat_binary_time, nl,
		 mph->clock_step, nl,
		 mph->spare_5, nl,
		 mph->leap_utc, nl,
		 mph->leap_sign, nl,
		 mph->leap_err, nl,
		 mph->spare_6, nl,
		 mph->product_err, nl,
		 mph->tot_size, nl,
		 mph->sph_size, nl,
		 mph->num_dsd, nl,
		 mph->dsd_size, nl,
		 mph->num_data_sets, nl,
		 mph->spare_7, nl );

    if (err != 82)
	return MPH_ERROR;
    return OK;
}





/* Identify product by MPH */
SCIA_PRODUCT scia_product (MPH *mph)
{
    int n;
    for (n=0; n<SCIA_MAX_PRODUCT; n++)
	if (strncmp ( SCIA_PRODUCT_ACR[n] , mph->product, 10 ) == 0 )
	    return n;
    return n;			/* SCIA_MAX_PRODUCT is unknown product  */
}

/* Read the Dataset Desc */

SCIA_err Read_DSD (FILE* unit, DSD *dsd)
{
    int err;
    char nl[2];
    err=fscanf(unit,        
	   "DS_NAME=\"%28c\"%1c"
	   "DS_TYPE=%1c%1c"
	   "FILENAME=\"%62c\"%1c"
	   "DS_OFFSET=+%20u<bytes>%1c"
	   "DS_SIZE=+%20u<bytes>%1c"
	   "NUM_DSR=+%10u%1c"
	   "DSR_SIZE=%11d<bytes>%1c"
	   "%32c%1c",
	       dsd->name, nl,
	       &dsd->type, nl,
	       dsd->filename, nl,
	       &dsd->offset, nl,
	       &dsd->size, nl,
	       &dsd->num_dsr, nl,
	       &dsd->dsr_size, nl,
	       dsd->spare, nl);
/*
    dsd->name[28]='\0';
    if ( (str_ptr = strpbrk(dsd->name, " ") ) != NULL)
	*str_ptr='\0';

    dsd->filename[62]='\0';
    if ( (str_ptr = strpbrk(dsd->filename, " ") ) != NULL)
	*str_ptr='\0';
	*/
    if ( err != 16 )
	return DSD_ERROR;
    return OK;
}


/* Write the Dataset Desc */

SCIA_err Write_DSD (FILE* unit, DSD *dsd)
{
    int err;
    char nl[2] = "\n";
    err=fprintf(unit,        
	   "DS_NAME=\"%.28s\"%.1s"
	   "DS_TYPE=%c%.1s"
	   "FILENAME=\"%.62s\"%.1s"
	   "DS_OFFSET=+%020u<bytes>%.1s"
	   "DS_SIZE=+%020u<bytes>%.1s"
	   "NUM_DSR=+%010u%.1s"
	   "DSR_SIZE=%+011d<bytes>%.1s"
	   "%.32s%.1s",
	       dsd->name, nl,
	       dsd->type, nl,
	       dsd->filename, nl,
	       dsd->offset, nl,
	       dsd->size, nl,
	       dsd->num_dsr, nl,
	       dsd->dsr_size, nl,
	       dsd->spare, nl);

/*    if ( err != 16 )
      return DSD_ERROR; */
    return OK;
}





/* set the filepointer to the Begin of dataset described by this DSD */
SCIA_err set_DSD_offset (FILE* unit, DSD *dsd)
{
    if (dsd->offset == 0)
	return SCIA_DSD_NOT_ATTACHED;
    if (fseek(unit, dsd->offset, SEEK_SET) == -1)
    {
	return ERROR;
    }
    return OK;
}


    
/*********************************************************************\
 * Compare two dates MJD
 * MJD compare : 0 equal  -1 earlier  1 later
 * kb 23.04.01
\*********************************************************************/

int MJD_compare (MJD *mjd1, MJD *mjd2)
{
    if (mjd1->days < mjd2->days)
	return -1;
    if (mjd1->days > mjd2->days)
	return 1;
				/* Days are equal !! */
    if (mjd1->secnd < mjd2->secnd)
	return -1;
    if (mjd1->secnd > mjd2->secnd)
	return 1;
				/* secnd are equal !! */
    if (mjd1->musec < mjd2->musec)
	return -1;
    if (mjd1->musec > mjd2->musec)
	return 1;
				/* musec are equal !! */
    return 0;
}

/**********************************************************************
 * add 1/16 seconds to MJD
 **********************************************************************/

MJD MJD_add (MJD mjd, int sec_16)
{
    unsigned sec_16_factor = 62500;
    unsigned secnd;
    unsigned musec = mjd.musec;
    mjd.musec = (musec + sec_16 * sec_16_factor) % 1000000;
    secnd = mjd.secnd +
	(musec + sec_16 * sec_16_factor) / 1000000;
    mjd.secnd = secnd % SECOND_PER_DAY;
    mjd.days = mjd.days + secnd / SECOND_PER_DAY;
    return mjd;
}

/* Calculate mjd2 - mjd1 in seconds */
double MJD_diff (MJD *mjd1, MJD *mjd2)
{
/*    MJD *larger;
    MJD *smaller;
    double sign = 1.0;*/
    int ddays, dsecnd, dmusec;
    double diff;

    /*
    switch  (MJD_compare (mjd1, mjd2))
    {
	case -1:
	    larger = mjd2;
	    smaller = mjd1;
	    break;
	case 0:
	    return 0.0;
	    break;
	case 1:
	    larger = mjd1;
	    smaller = mjd2;
	    sign = -1.0;
	    break;
    }
    */

    ddays = mjd2->days - mjd1->days;
    dsecnd = mjd2->secnd - mjd1->secnd;
    if (dsecnd < 0)
    {
	ddays  -= 1;
	dsecnd += SECOND_PER_DAY;
    }
    dmusec = mjd2->musec - mjd1->musec;
    if (dmusec < 0)
    {
	dsecnd -= 1;
	dmusec += 1000000;
    }
    diff = (double) ddays * SECOND_PER_DAY +
	(double) dsecnd +
	(double) dmusec / MIKROSEC_PER_SEC;
	
    return diff; /* * sign; */
}
    

/*--------------------------------------------------------------------*\
** 
** routines for creation of UTC String from MJD structure 
** (musec are ignored)
** 
** Author: S. Noel, IFE/IUP Uni Bremen 
**        (Stefan.Noel@iup.physik.uni.bremen.de)
** 
** Version: 0.1, 25 Mar 2002
**
** (This routine is based on "MJD_2_ASCII" from the NADC lib.)
** 
\*--------------------------------------------------------------------*/


/*+++++++++++++++++++++++++
.IDENTifer   CALDAT
.PURPOSE     return the calendar date for given julian date
.INPUT/OUTPUT
  call as    CALDAT( ijul, &iday, &imon, &iyear );

     input:
            unsigned int ijul     :  Julian day number (starts at noon)
    output:
            int  *iday  :   number of day of the month.
            int  *imon  :   number of the month (1 = january, ... )
            int  *iyear :   number of the year

.RETURNS     nothing
.COMMENTS    static function
             Taken from "Numerical Recipies in C", by William H. Press,
	     Brian P. Flannery, Saul A. Teukolsky, and William T. Vetterling.
	     Cambridge University Press, 1988 (second printing).

-------------------------*/
/*********** NOTE: This is taken from the NADC lib *************/
static
void CALDAT( unsigned int ijul, /*@out@*/ int *iday, 
             /*@out@*/ int *imon, /*@out@*/ int *iyear )
{
     int ja, jalpha, jb, jc, jd, je;
/*
 * Gregorian Calander was adopted on Oct. 15, 1582
 */
     const unsigned int IGREG = 15U + 31U * (10U + 12U * 1582U);

     if ( ijul >= IGREG ) {
	  jalpha = (int) (((float) (ijul - 1867216) - 0.25)
			  / 36524.25);
	  ja = ijul + 1 + jalpha - (int) (0.25 * jalpha);
     } else {
	  ja = (int) ijul;
     }
     jb = ja + 1524;
     jc = (int)(6680.0 + ((float) (jb - 2439870) - 122.1) / 365.25);
     jd = (int)(365 * jc + (0.25 * jc));
     je = (int)((jb - jd) / 30.6001);
     *iday = jb - jd - (int) (30.6001 * je);
     *imon = je - 1;
     if ( *imon > 12 ) *imon -= 12;
     *iyear = jc - 4715;
     if ( *imon > 2 ) --(*iyear);
     if ( *iyear <= 0 ) --(*iyear);
}


/*--------------------------------------------------------------------*\
**       create UTC String from MJD structure 
** 	 (This routine is based on "MJD_2_ASCII" from the NADC lib.)
\*--------------------------------------------------------------------*/
void UTC_String(MJD* time, char* string)
{
  /*
   * Julian date at 01/01/2000
   */
  static const double jday_01012000 = 2451544.5;
  /*
   * array for month names
   */
  static const char *mon_str[12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" 
  };

  int  iday, ihour, imin, imon, isec, iyear;  
  unsigned int jday;

  /*
   * convert to julian day
   */

  jday = (time->days + jday_01012000 + 0.5);
  CALDAT( jday, &iday, &imon, &iyear );

  ihour = (int)(time->secnd / 3600);
  isec  = (int) time->secnd - (3600 * ihour);
  imin  = isec / 60;
  isec -= 60 * imin;
  (void) sprintf( string, "%.2d-%3s-%.4d %.2d:%.2d:%.2d.%.6u", iday, 
		  mon_str[imon-1], iyear, ihour, imin, isec, time->musec);
  
  return;
}


/*+++++++++++++++++++++++++
.IDENTifer   JULDAY
.PURPOSE     return julian day for given the calendar date
.INPUT/OUTPUT
  call as    jday = JULDAY( myclock );

     input:
            struct tm myclock  :  time in tm_format

.RETURNS     Julian day number and fractional day
.COMMENTS    static function
             Taken from "Numerical Recipies in C", by William H. Press,
	     Brian P. Flannery, Saul A. Teukolsky, and William T. Vetterling.
	     Cambridge University Press, 1988 (second printing).
-------------------------*/
static
double JULDAY( struct tm myclock )
{
     unsigned int ja, jd, ijul;
     int          jy, jm;

/*
 * Gregorian Calander was adopted on Oct. 15, 1582
 */
     const unsigned int IGREG = 15U + 31U * (10U + 12U * 1582U);

     if ( (jy = myclock.tm_year) < 0 ) jy++;
     if ( myclock.tm_mon > 2 ) {
	  jm = myclock.tm_mon + 1;
     } else {
	  jy--;
	  jm = myclock.tm_mon + 13;
     }
     ijul = (unsigned int) (floor( 365.25 * jy )
			    + floor( 30.6001 * jm ) 
			    + myclock.tm_mday + 1720995U);
/*
 * test whether to change to Gregorian calendar
 */
     jd = myclock.tm_mday + 31U * (myclock.tm_mon + 12U * myclock.tm_year);
     if ( jd >= IGREG ) {
	  ja = (unsigned int)(0.01 * jy);
	  ijul += 2U - ja + (unsigned int) (0.25 * ja);
     }
     return ijul + (myclock.tm_hour / 24. - 0.5) 
	  + (myclock.tm_min / 1440.)
	  + (myclock.tm_sec / 86400.0);
}



/*+++++++++++++++++++++++++
.IDENTifer   ASCII_2_UTC
.PURPOSE     Converts ASCII time into ESA UTC time 
.INPUT/OUTPUT
  call as    ASCII_2_UTC( ASCII_Time, &utc_day, &utc_sec );

     input:
            char *ASCII_Time      :  given as DD-MMM-YYYY HH:MM:SS.SSS
    output:
            unsigned int utc_day  :  ESA UTC days since 01.01.1950
            unsigned int utc_msec :  ESA UTC milli-seconds since midnight

.RETURNS     error status : 0 on error, 1 on success
.COMMENTS    none
-------------------------*/
int ASCII_2_MJD( const char ASCII_DateTime[],
		 MJD* mjd)
/*		  unsigned int *utc_day, unsigned int *utc_msec )*/
{
    int err;
    register int nr;

     char   ASCII_mon[4];                   /* input month (string) */
     char   ASCII_Date[12];                 /* date part of input string */
     char   ASCII_Time[16];                 /* time part of input string */

     int         mon_idx;
     double      jday, musec, sec;

     struct tm myclock;
     
     const char *mon_str[12] = {
     "JAN", "FEB", "MAR", "APR", "MAY", "JUN", 
     "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
     
     const double jday_01012000 = 2451544.5;

/*
 * initialisation
 */
     mjd->days = 0;
     mjd->secnd = 0U;
     mjd->musec = 0U;
/*
 * decomposition of input string into its date and time part
 */
     (void) strncpy( ASCII_Date, ASCII_DateTime, 11 );
     ASCII_Date[11] = '\0';
     (void) strncpy( ASCII_Time, ASCII_DateTime+12, 15 );
     ASCII_Time[15] = '\0';
/*
 * decomposition of date and time string into numbers into the tm struct
 */
     err = sscanf( ASCII_Date, " %2d %*c %3s %*c %4d ", 
		    &myclock.tm_mday, ASCII_mon, &myclock.tm_year );
     if (err != 3)
	 return 0;
     err =  sscanf( ASCII_Time,"%2d %*c %2d %*c %lf", 
		    &myclock.tm_hour, &myclock.tm_min, &sec );
     if (err != 3)
	 return 0;
				/*  adding 0.1 musec avoid rounding errors */
				/*  in last digit (otherwise f.e. 306177 */
				/*  would be printed read as 306176 */
     sec += 0.1 / MIKROSEC_PER_SEC;
	 
     myclock.tm_year = myclock.tm_year;
     myclock.tm_sec = (int) sec;
     myclock.tm_wday = 0;
     myclock.tm_yday = 0;
     myclock.tm_isdst = 0;
     musec = MIKROSEC_PER_SEC * (sec - myclock.tm_sec);
/*
 * initialize month
 */
     for ( nr = 0; nr < 3; nr ++ ) 
	  ASCII_mon[nr] = (char) toupper( (int) ASCII_mon[nr] );
     ASCII_mon[3] = '\0';

     mon_idx = 0;
     while ( mon_idx < 12 && strncmp(mon_str[mon_idx], ASCII_mon, 3) != 0 ) 
	  mon_idx++;
     if (mon_idx == 12)
	 return 0;
     myclock.tm_mon = mon_idx + 1;
				/*  Calculate Julian Day */
     jday = JULDAY( myclock );
     jday = jday -  jday_01012000;
				/*  Build MJD */
     mjd->days = (int) floor (jday);
     mjd->secnd = (jday - mjd->days) * SECOND_PER_DAY;
     mjd->musec = (unsigned int) musec;
     return 1;
}



/*--------------------------------------------------------------------*\
**       create UTC String as  yyyymmdd_hhmmss  from UTC_string
\*--------------------------------------------------------------------*/
void yyyymmdd_string(char* utc, char* yyyymmdd)
{
  /*
   * array for month names
   */
  static const char *mon_str[12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" 
  };
  static const char *MON_STR[12] = {
    "JAN", "FEB", "MAR", "APR", "MAY", "JUN", 
    "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" 
  };

  
  int  n_mon;

  for (n_mon = 0; n_mon<12; n_mon++)
  {
      if (strncmp (MON_STR[n_mon], utc+3, 3) == 0 ||
	  strncmp (mon_str[n_mon], utc+3, 3) == 0)
	  break;
  }
  /*
   * convert to yyyymmdd_hhmmss
   */
				/* "05-AUG-2002 11:15:42.188532" */
  sprintf (yyyymmdd, "%.4s%02d%.2s_%.2s%.2s%.2s",
	   utc+7,
	   n_mon + 1,
	   utc,
	   utc+12,
	   utc+15,
	   utc+18);
  
  return;
}


/*--------------------------------------------------------------------*\
**       Calculate fractional day  after 1/1/2000
\*--------------------------------------------------------------------*/

double JDAY_01012000 ( MJD* time )
{
   /* Julian date at 01/01/2000
   */
    double jday = (double) time->days +
	(double) time->secnd / SECOND_PER_DAY +
	(double) time->musec /
	((double)SECOND_PER_DAY * (double)MIKROSEC_PER_SEC);
    return jday;
}
	
